#ifndef COMPILE_SDL
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>

#include <windows.h>
#endif
#include "MBitmaps.h"

#include <time.h>

#ifdef COMPILE_SDL
/*
vraci ukazadel na sdlrect kdyz nebyl rect NULL a nastavi hodnoty do sdlrect;
jinak vraci NULL a nic nenastavuje
*/
SDL_Rect *RECTtoSDLRect(const myLPRECT rect,SDL_Rect &sdlrect)
{
 if(!rect) return NULL;
 sdlrect.x=rect->left;
 sdlrect.y=rect->top;
 sdlrect.w=rect->right-rect->left;
 sdlrect.h=rect->bottom-rect->top;
 return &sdlrect;
}

#endif

//----------------------------------------------------------------------------

UINT bitmapsBitCount=24;

UINT GetDefaultBitmapsBitCount(void){return bitmapsBitCount;}
void SetDefaultBitmapsBitCount(UINT val){bitmapsBitCount=val;}

//----------------------------------------------------------------------------

MBitmap::MBitmap()
{
#ifdef COMPILE_SDL
 s=NULL;
#else
 Btmp=NULL;
 BtmpMask=NULL;
 PaintingDC=NULL;
#endif
 sz.cx=0; sz.cy=0;
 TransparentColorPos.x=0; TransparentColorPos.y=0;
}
MBitmap::~MBitmap(){Clear();}

void MBitmap::GetSize(mySIZE &pSize)
{
 pSize.cx=sz.cx;
 pSize.cy=sz.cy;
}

void MBitmap::Clear(void)
{
#ifdef COMPILE_SDL
 if(s) SDL_FreeSurface(s); s=NULL;
#else
 if(Btmp!=NULL) DeleteObject(Btmp); Btmp=NULL;
 if(BtmpMask!=NULL) DeleteObject(BtmpMask); BtmpMask=NULL;
 if(PaintingDC!=NULL) DeleteDC(PaintingDC); PaintingDC=NULL;
#endif
 sz.cx=0; sz.cy=0;
}

#ifndef COMPILE_SDL
HBITMAP MBitmap::CreateBtmp(int width, int height)
{
 BITMAPINFO bi;
 bi.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
 bi.bmiHeader.biWidth=width;
 bi.bmiHeader.biHeight=-height;
 bi.bmiHeader.biPlanes=1;
 bi.bmiHeader.biBitCount=bitmapsBitCount;//24;
 bi.bmiHeader.biCompression=BI_RGB;
 bi.bmiHeader.biSizeImage=0;
 bi.bmiHeader.biClrUsed=0;
 bi.bmiHeader.biClrImportant=0;
 //bi.bmiColors=;
 
 HBITMAP res=CreateDIBSection(NULL,&bi,DIB_RGB_COLORS,NULL,NULL,0);
 return res;
}
#endif

void MBitmap::Assign(const MBitmap *src)
{
 Clear();
 if(src!=NULL)
   {
    sz.cx=src->sz.cx;
	sz.cy=src->sz.cy;
	#ifdef COMPILE_SDL
	if(src->s)
	  {
       Uint32 rmask=0, gmask=0, bmask=0, amask=0;

       /*#if SDL_BYTEORDER == SDL_BIG_ENDIAN
        rmask = 0xff000000;
        gmask = 0x00ff0000;
        bmask = 0x0000ff00;
        amask = 0x000000ff;
       #else
        rmask = 0x000000ff;
        gmask = 0x0000ff00;
        bmask = 0x00ff0000;
        amask = 0xff000000;
       #endif*/
	   s=SDL_CreateRGBSurface(SDL_SWSURFACE,sz.cx,sz.cy,bitmapsBitCount,rmask,gmask,bmask,amask);
	   if(s)
	     {
		  Uint32 oldflags=src->s->flags;
		  Uint8 oldalpha=src->s->format->alpha;
		  Uint32 oldcolorkey=src->s->format->colorkey;
		  if(/*oldalpha!=SDL_ALPHA_OPAQUE&&*/(oldflags&SDL_SRCALPHA))
			{
			 SDL_SetColorKey(src->s, 0, 0);
			 SDL_SetAlpha(src->s,0,0);
			}
		  SDL_BlitSurface(src->s,NULL,s,NULL);
		  if(oldalpha!=SDL_ALPHA_OPAQUE&&(oldflags&SDL_SRCALPHA))
			{
			 SDL_SetColorKey(src->s, SDL_SRCCOLORKEY, oldcolorkey);
			 SDL_SetAlpha(src->s,oldflags&(SDL_SRCALPHA),oldalpha);
			 SDL_SetColorKey(s, SDL_SRCCOLORKEY, oldcolorkey);
			 SDL_SetAlpha(s,oldflags&(SDL_SRCALPHA),oldalpha);
			}
		 }
	  }
    #else
	if(src->Btmp!=NULL)
	  {
	   HDC dc=CreateCompatibleDC(NULL),dc2=CreateCompatibleDC(NULL);
	   if(dc!=NULL&&dc2!=NULL)
	     {
		  SelectObject(dc,src->Btmp);
	      Btmp=CreateBtmp(sz.cx,sz.cy); //CreateCompatibleBitmap(dc,sz.cx,sz.cy);
		  SelectObject(dc2,Btmp);
		  BitBlt(dc2,0,0,sz.cx,sz.cy,dc,0,0,SRCCOPY);
		 }
	   if(dc!=NULL) DeleteDC(dc);
	   if(dc2!=NULL)DeleteDC(dc2);
	  }
	if(src->BtmpMask!=NULL)
	  {
	   HDC dc=CreateCompatibleDC(NULL),dc2=CreateCompatibleDC(NULL);
	   if(dc!=NULL&&dc2!=NULL)
	     {
		  SelectObject(dc,src->BtmpMask);
	      BtmpMask=CreateBtmp(sz.cx,sz.cy);//CreateCompatibleBitmap(dc,sz.cx,sz.cy);
		  SelectObject(dc2,BtmpMask);
		  BitBlt(dc2,0,0,sz.cx,sz.cy,dc,0,0,SRCCOPY);
		 }
	   if(dc!=NULL) DeleteDC(dc);
	   if(dc2!=NULL)DeleteDC(dc2);
	  }
	#endif
   }
}

bool MBitmap::Create(int Width, int Height)
{
 Clear();
 sz.cx=Width; sz.cy=Height;
 #ifdef COMPILE_SDL
 Uint32 rmask=0, gmask=0, bmask=0, amask=0;
 /*#if SDL_BYTEORDER == SDL_BIG_ENDIAN
  rmask = 0xff000000;
  gmask = 0x00ff0000;
  bmask = 0x0000ff00;
  amask = 0x000000ff;
 #else
  rmask = 0x000000ff;
  gmask = 0x0000ff00;
  bmask = 0x00ff0000;
  amask = 0xff000000;
 #endif*/
 s=SDL_CreateRGBSurface(SDL_SWSURFACE,sz.cx,sz.cy,bitmapsBitCount,rmask,gmask,bmask,amask);
 return s!=NULL;
 #else
 Btmp=CreateBtmp(Width,Height);
 return Btmp!=NULL;
 #endif
}

#ifdef COMPILE_SDL
SDL_Surface *MBitmap::GetSurface(void)
{
 return s;
}
#endif

MyHDC MBitmap::BeginPaint(void)
{
 #ifdef COMPILE_SDL
 return GetSurface();
 #else
 if(Btmp==NULL) return NULL;
 if(PaintingDC!=NULL) return PaintingDC;
 PaintingDC=CreateCompatibleDC(NULL);
 if(PaintingDC!=NULL)
   {
    SelectObject(PaintingDC,Btmp);
   }
 return PaintingDC;
 #endif
}

void MBitmap::EndPaint(void)
{
 #ifndef COMPILE_SDL
 if(PaintingDC==NULL) return;
 /*BITMAP bm;
 GetObject(PaintingDC,sizeof(bm),&bm);
 if(Btmp!=NULL) DeleteObject(Btmp);
 Btmp=CreateBitmapIndirect(&bm);*/
 DeleteDC(PaintingDC);
 PaintingDC=NULL;
 #endif
}

void MBitmap::EraseMask(void)
{
 #ifdef COMPILE_SDL
 if(s) 
   {
	SDL_SetColorKey(s, 0, SDL_MapRGB(s->format,255,255,255));
    SDL_SetAlpha(s,0,0);
   }
 #else
 if(BtmpMask!=NULL) DeleteObject(BtmpMask); BtmpMask=NULL;
 #endif
}

void MBitmap::UpdateMask(void)
{
 UpdateMask(TransparentColorPos.x,TransparentColorPos.y);
}

void MBitmap::UpdateMask(unsigned char r, unsigned char g, unsigned char b)
{
 #ifdef COMPILE_SDL
 if(s) 
   {
    SDL_SetColorKey(s,SDL_SRCCOLORKEY,SDL_MapRGB(s->format,r,g,b));
    SDL_SetAlpha(s,SDL_SRCALPHA/*|SDL_RLEACCEL*/,/*SDL_ALPHA_TRANSPARENT*/255);
   }
 #else
 COLORREF TransparentColor=RGB(r,g,b);
 if(PaintingDC!=NULL) EndPaint();
 EraseMask();
 HDC hdcImage = CreateCompatibleDC(NULL);
 HDC hdcImage2 = CreateCompatibleDC(NULL);
 if(hdcImage!=NULL&&hdcImage2!=NULL)
   {
    if(TransparentColorPos.x>=sz.cx) TransparentColorPos.x=0;
    if(TransparentColorPos.y>=sz.cy) TransparentColorPos.y=0;
    //BITMAP bm;
	SelectObject( hdcImage, Btmp);
    BtmpMask=CreateBtmp(sz.cx,sz.cy);//CreateCompatibleBitmap(hdcImage,sz.cx,sz.cy);
	SelectObject( hdcImage2, BtmpMask);
    COLORREF pixel, black=RGB(0,0,0), white=RGB(255,255,255);
	//COLORREF TransparentColor=GetPixel(hdcImage,TransparentColorPos.x ,TransparentColorPos.y);
    for(int x=0;x<sz.cx;x++)
      for(int y=0;y<sz.cy;y++)
	    {
		 pixel=GetPixel(hdcImage,x,y);
		 if(pixel==TransparentColor)
		   {
		    SetPixel(hdcImage2,x,y,white);
		    SetPixel(hdcImage,x,y,black);
		   }
		  else SetPixel(hdcImage2,x,y,black);
		}
   }
 DeleteDC( hdcImage );
 DeleteDC( hdcImage2 );
 #endif
}

void MBitmap::UpdateMask(int x, int y)
{
 TransparentColorPos.x=x; TransparentColorPos.y=y;
 unsigned char r=0,g=0,b=0;
 #ifdef COMPILE_SDL
 if(s)
   {
    if(SDL_MUSTLOCK(s))
	  {
	   SDL_LockSurface(s);
	  }
	if(!s->pixels)
	  {
       if(SDL_MUSTLOCK(s))
	     {
	      SDL_UnlockSurface(s);
	     }
	   return;
	  }
    int bpp = s->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to retrieve */
    Uint8 *p = (Uint8 *)s->pixels + y * s->pitch + x * bpp;
	Uint32 pix=0;
    switch(bpp)
	  {
       case 1:
        pix=(*p);
		break;
       case 2:
        pix=(*(Uint16 *)p);
	    break;
       case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
          pix=(p[0] << 16 | p[1] << 8 | p[2]);
         else
          pix=(p[0] | p[1] << 8 | p[2] << 16);
		break;
	   case 4:
        pix=(*(Uint32 *)p);
		break;
      }
    if(SDL_MUSTLOCK(s))
	  {
	   SDL_UnlockSurface(s);
	  }
    SDL_GetRGB(pix,s->format,&r,&g,&b);
   }
 #else
 COLORREF TransparentColor=RGB(0,0,0);
 HDC hdcImage = CreateCompatibleDC(NULL);
 if(hdcImage!=NULL)
   {
    if(TransparentColorPos.x>=sz.cx) TransparentColorPos.x=0;
    if(TransparentColorPos.y>=sz.cy) TransparentColorPos.y=0;
    //BITMAP bm;
	SelectObject( hdcImage, Btmp);
	TransparentColor=GetPixel(hdcImage,TransparentColorPos.x ,TransparentColorPos.y);
   }
 DeleteDC( hdcImage );
 r=GetRValue(TransparentColor);
 g=GetGValue(TransparentColor);
 b=GetBValue(TransparentColor);
 #endif
 UpdateMask(r,g,b);
}

void MBitmap::UpdateMask(const myPOINT pt)
{
 UpdateMask(pt.x, pt.y);
}


bool MBitmap::LoadFromFile(const char *FileName, bool pCreateMask,myPOINT pTransparentColorPos)
{
 if(!LoadFromFile(FileName)) return false;
 TransparentColorPos.x=pTransparentColorPos.x;
 TransparentColorPos.y=pTransparentColorPos.y;
 if(pCreateMask)
   {
    UpdateMask();
   }
 return true;
}

bool MBitmap::LoadFromFile(const char *FileName, bool pCreateMask)
{
 if(FileName==NULL) return false;
 Clear();
 TransparentColorPos.x=0;
 TransparentColorPos.y=0;
 
 #ifdef COMPILE_SDL
 s=SDL_LoadBMP(FileName);
 if(!s) return false;
 sz.cx=s->w; sz.cy=s->h;
 #else
 Btmp=(HBITMAP)LoadImage(NULL,FileName,IMAGE_BITMAP,0,0,LR_DEFAULTSIZE|LR_CREATEDIBSECTION|LR_LOADFROMFILE);
 if(Btmp==NULL) return false;
 BITMAP bm;
 GetObject( Btmp, sizeof(bm), &bm );    // get size of bitmap
 sz.cx=bm.bmWidth; sz.cy=bm.bmHeight;
 #endif
 if(pCreateMask)
   {
    UpdateMask();
   }
 return true;
}

#ifdef STREAM_SUPPORT_ON
bool MBitmap::LoadFromStream(MStream *strm, bool pCreateMask)
{
 if(!strm) return false;
 Clear();
 #ifdef COMPILE_SDL
 SDL_RWops rwop,*prwop;
 prwop=MStreamToSDLRWOps(strm,&rwop);
 if(!prwop) return false;
 s=SDL_LoadBMP_RW(prwop,0);
 if(s)
   {
    sz.cx=s->w; sz.cy=s->h;
   }
 if(s&&pCreateMask)
   {
    UpdateMask();
   }
 return s!=NULL;
 #else
 BITMAPFILEHEADER hdr;
 BITMAPINFO *bi;
 if(sizeof(BITMAPFILEHEADER)!=strm->read(&hdr,sizeof(BITMAPFILEHEADER))) return false;
 if(hdr.bfType!=0x4d42) return false;
 int msz=sizeof(BYTE)*(hdr.bfOffBits-sizeof(hdr));
 BYTE *bts=(BYTE *)malloc(msz);
 if(!bts) return false;
 if(msz!=strm->read(bts,msz))
   {
    free(bts);
	return false;
   }
 bi=(BITMAPINFO*)bts;

 BYTE *bits;
 Btmp=CreateDIBSection(NULL,bi,DIB_RGB_COLORS,(void **)&bits,NULL,0);
 if(Btmp)
   {
    WORD cClrBits; 
    cClrBits = (WORD)(bi->bmiHeader.biPlanes * bi->bmiHeader.biBitCount); 
    if (cClrBits == 1) cClrBits = 1; 
     else if (cClrBits <= 4) cClrBits = 4; 
     else if (cClrBits <= 8) cClrBits = 8; 
     else if (cClrBits <= 16) cClrBits = 16; 
     else if (cClrBits <= 24) cClrBits = 24; 
     else cClrBits = 32; 
    
    DWORD siz=//hdr.bfSize-hdr.bfOffBits;
       (((bi->bmiHeader.biWidth * cClrBits +31) & ~31) /8) * bi->bmiHeader.biHeight;
	strm->read(bits,siz);
	GdiFlush();
    BITMAP bm;
    GetObject( Btmp, sizeof(bm), &bm );    // get size of bitmap
    sz.cx=bm.bmWidth; sz.cy=bm.bmHeight;
    if(pCreateMask)
      {
       UpdateMask();
      }
   }

 free(bts);
 return Btmp!=NULL;
 #endif
}

#endif

bool MBitmap::SaveToFile(const char *FileName)
{
 if(!FileName) return false;
 #ifdef COMPILE_SDL
 if(!s) return false;
 return 0==SDL_SaveBMP(s,FileName);
 #else
 if(!Btmp) return false;
 HBITMAP hBmp=Btmp;
 BITMAP bmp; 
 PBITMAPINFO pbmi;
 WORD  cClrBits; 

 // Retrieve the bitmap color format, width, and height. 
 if(!GetObject(hBmp, sizeof(BITMAP), (LPSTR)&bmp)) return false;

 // Convert the color format to a count of bits. 
 cClrBits = (WORD)(bmp.bmPlanes * bmp.bmBitsPixel); 
 if (cClrBits == 1) cClrBits = 1; 
  else if (cClrBits <= 4) cClrBits = 4; 
  else if (cClrBits <= 8) cClrBits = 8; 
  else if (cClrBits <= 16) cClrBits = 16; 
  else if (cClrBits <= 24) cClrBits = 24; 
  else cClrBits = 32; 

 // Allocate memory for the BITMAPINFO structure. (This structure 
 // contains a BITMAPINFOHEADER structure and an array of RGBQUAD 
 // data structures.) 

 /*if(cClrBits != 24) 
   pbmi = (PBITMAPINFO) LocalAlloc(LPTR, 
             sizeof(BITMAPINFOHEADER) + 
             sizeof(RGBQUAD) * (1<< cClrBits)); 

 // There is no RGBQUAD array for the 24-bit-per-pixel format. 
  else */
    pbmi = (PBITMAPINFO) LocalAlloc(LPTR, 
             sizeof(BITMAPINFOHEADER)); 

 // Initialize the fields in the BITMAPINFO structure. 
 pbmi->bmiHeader.biSize = sizeof(BITMAPINFOHEADER); 
 pbmi->bmiHeader.biWidth = bmp.bmWidth; 
 pbmi->bmiHeader.biHeight = bmp.bmHeight; 
 pbmi->bmiHeader.biPlanes = bmp.bmPlanes; 
 pbmi->bmiHeader.biBitCount = bmp.bmBitsPixel; 
 if(cClrBits < 24) pbmi->bmiHeader.biClrUsed = (1<<cClrBits); 

 // If the bitmap is not compressed, set the BI_RGB flag. 
 pbmi->bmiHeader.biCompression = BI_RGB; 

 // Compute the number of bytes in the array of color 
 // indices and store the result in biSizeImage. 
 // For Windows NT, the width must be DWORD aligned unless 
 // the bitmap is RLE compressed. This example shows this. 
 // For Windows 95/98/Me, the width must be WORD aligned unless the 
 // bitmap is RLE compressed.
 pbmi->bmiHeader.biSizeImage = (((pbmi->bmiHeader.biWidth * cClrBits +31) & ~31) /8)
                                  * pbmi->bmiHeader.biHeight; 
 // Set biClrImportant to 0, indicating that all of the 
 // device colors are important. 
 pbmi->bmiHeader.biClrImportant = 0; 

 FILE *f;
 BITMAPFILEHEADER hdr;       // bitmap file-header 
 PBITMAPINFOHEADER pbih;     // bitmap info-header 
 LPBYTE lpBits;              // memory pointer 
 DWORD dwTotal;              // total count of bytes 
 DWORD cb;                   // incremental count of bytes 
 BYTE *hp;                   // byte pointer 

 pbih = (PBITMAPINFOHEADER) pbmi; 
 lpBits = (LPBYTE) GlobalAlloc(GMEM_FIXED, pbih->biSizeImage);

 if(!lpBits) 
   {
    LocalFree(pbmi);
    return false;
   }

    // Retrieve the color table (RGBQUAD array) and the bits 
    // (array of palette indices) from the DIB. 
 HDC hDC=BeginPaint();
 if(!GetDIBits(hDC, hBmp, 0, (WORD) pbih->biHeight, lpBits, pbmi, 
        DIB_RGB_COLORS)) 
   {
    EndPaint();
    LocalFree(pbmi);
    GlobalFree(lpBits);
	return false;
   }
 
 f=fopen(FileName,"wb");
 if(!f)
   {
    LocalFree(pbmi);
    GlobalFree((HGLOBAL)lpBits);
    EndPaint();
    return false;
   }

 /*HANDLE hf = CreateFile(FileName, 
                   GENERIC_READ | GENERIC_WRITE, 
                   (DWORD) 0, 
                    NULL, 
                   CREATE_ALWAYS, 
                   FILE_ATTRIBUTE_NORMAL, 
                   (HANDLE) NULL); 

 if(!hf)
   {
    LocalFree(pbmi);
    GlobalFree((HGLOBAL)lpBits);
    EndPaint();
    return false;
   }*/
    
 hdr.bfType = 0x4d42;        // 0x42 = "B" 0x4d = "M" 
 // Compute the size of the entire file. 
 hdr.bfSize = (DWORD) (sizeof(BITMAPFILEHEADER) + 
              pbih->biSize + pbih->biClrUsed 
              * sizeof(RGBQUAD) + pbih->biSizeImage); 
 hdr.bfReserved1 = 0; 
 hdr.bfReserved2 = 0; 

 // Compute the offset to the array of color indices. 
 hdr.bfOffBits = (DWORD) sizeof(BITMAPFILEHEADER) + 
                 pbih->biSize + pbih->biClrUsed 
                 * sizeof (RGBQUAD); 

 fwrite(&hdr,sizeof(BITMAPFILEHEADER),1,f);
 fwrite(pbih,sizeof(BITMAPINFOHEADER)+pbih->biClrUsed * sizeof (RGBQUAD),1,f);
 dwTotal = cb = pbih->biSizeImage; 
 hp = lpBits; 
 fwrite(hp, (int) cb, 1,f); 

 /*
 DWORD dwTmp; 
 // Copy the BITMAPFILEHEADER into the .BMP file. 
 if(!WriteFile(hf, (LPVOID) &hdr, sizeof(BITMAPFILEHEADER), 
     (LPDWORD) &dwTmp,  NULL)) 
   {
    //errhandler("WriteFile", hwnd); 
   }

 // Copy the BITMAPINFOHEADER and RGBQUAD array into the file. 
 if(!WriteFile(hf, (LPVOID) pbih, sizeof(BITMAPINFOHEADER) 
               + pbih->biClrUsed * sizeof (RGBQUAD), 
               (LPDWORD) &dwTmp, ( NULL))) 
     ;//errhandler("WriteFile", hwnd); 

 // Copy the array of color indices into the .BMP file. 
 dwTotal = cb = pbih->biSizeImage; 
 hp = lpBits; 
 if(!WriteFile(hf, (LPSTR) hp, (int) cb, (LPDWORD) &dwTmp,NULL)) 
        ;//errhandler("WriteFile", hwnd); 
 CloseHandle(hf);
 */

 fclose(f);

 // Free memory. 
 LocalFree(pbmi);
 GlobalFree((HGLOBAL)lpBits);
 EndPaint();

 return true;
 #endif
}

bool MBitmap::Draw(MyHDC dc,myLPRECT desrect, myLPRECT vysekrect)
{
#ifdef COMPILE_SDL
 if(!s||!dc||!desrect) return false;
 SDL_Rect d,v;
 return 0==SDL_BlitSurface(s,RECTtoSDLRect(vysekrect,d),dc,RECTtoSDLRect(desrect,v));
#else
 if(dc==NULL||!desrect) return false;
 if(Btmp==NULL) return false;
 if(PaintingDC!=NULL) EndPaint();
 
 DWORD mode=SRCCOPY;
 
 myRECT vysek;
 if(vysekrect)
   {
    vysek.left=vysekrect->left; vysek.top=vysekrect->top;
	vysek.right=vysekrect->right-vysek.left; vysek.bottom=vysekrect->bottom-vysek.top;
   }
  else
   {
    vysek.left=0; vysek.top=0;
	vysek.right=sz.cx; vysek.bottom=sz.cy;
   }

 if(BtmpMask!=NULL)
   {
    HDC BtmpDC=CreateCompatibleDC(dc);
    if(BtmpDC==NULL) return false;
    SelectObject(BtmpDC,BtmpMask);
    StretchBlt(dc,desrect->left,desrect->top,desrect->right-desrect->left,desrect->bottom-desrect->top,
			BtmpDC,vysek.left,vysek.top,vysek.right,vysek.bottom,
			SRCAND);
    DeleteDC(BtmpDC);
	mode=SRCPAINT;
   }
 HDC BtmpDC=CreateCompatibleDC(dc);
 if(BtmpDC==NULL) return false;
 SelectObject(BtmpDC,Btmp);
 StretchBlt(dc,desrect->left,desrect->top,desrect->right-desrect->left,desrect->bottom-desrect->top,
	BtmpDC,vysek.left,vysek.top,vysek.right,vysek.bottom,//0,0,sz.cx,sz.cy,
	mode);
 DeleteDC(BtmpDC);
 return true;
#endif
}

#ifdef COMPILE_SDL
#else
bool MBitmap::Rotate(HBITMAP *pBtmp, bool RotateToLeft, int &Width, int &Height)
{
 if(pBtmp==NULL) return false;
 HDC BtmpDC,BtmpDC2;
 HBITMAP Bmp;
 BtmpDC=CreateCompatibleDC(NULL);
 BtmpDC2=CreateCompatibleDC(NULL);
 if(BtmpDC==NULL||BtmpDC2==NULL) 
   {
    if(BtmpDC!=NULL) DeleteDC(BtmpDC);
    if(BtmpDC2!=NULL) DeleteDC(BtmpDC2);
    return false;
   }
 SelectObject(BtmpDC,*pBtmp);
 Bmp=CreateBtmp(sz.cx,sz.cy);//CreateCompatibleBitmap(BtmpDC,sz.cy,sz.cx);
 if(Bmp==NULL)
   {
    DeleteDC(BtmpDC);
    DeleteDC(BtmpDC2);
    return false;
   }
 SelectObject(BtmpDC2,Bmp);
 for(int x=0;x<sz.cx;x++)
   for(int y=0;y<sz.cy;y++)
     {
 	  if(RotateToLeft) SetPixel(BtmpDC2,y,sz.cx-x-1,GetPixel(BtmpDC,x,y));
	   else SetPixel(BtmpDC2,sz.cy-y-1,x,GetPixel(BtmpDC,x,y));
	 }
 DeleteObject(*pBtmp);
 //BITMAP bm;
 //GetObject(BtmpDC2,sizeof(bm),&bm);
 (*pBtmp)=CreateBtmp(sz.cx,sz.cy);//CreateCompatibleBitmap(BtmpDC2,sz.cy,sz.cx);
 SelectObject(BtmpDC,*pBtmp);
 BitBlt(BtmpDC,0,0,sz.cy,sz.cx,BtmpDC2,0,0,SRCCOPY);
 Width=sz.cy; Height=sz.cx;
 DeleteDC(BtmpDC); BtmpDC=NULL;
 DeleteDC(BtmpDC2); BtmpDC2=NULL;
 DeleteObject(Bmp);
 return true;
}

bool MBitmap::RotateBitmap(bool RotateToLeft, int Count)
{
 if(Btmp==NULL) return false;
 Count=Count%4;
 if(Count==0) return true;
 if(Count<0) Count+=4;
 int w=0,h=0;
 for(int i=0;i<Count;i++)
   {
    if(!Rotate(&Btmp,RotateToLeft,w,h)) return false;
	if(BtmpMask!=NULL) {if(!Rotate(&BtmpMask,RotateToLeft,w,h)) return false;}
	sz.cx=w; sz.cy=h;
   }
 return true;
}
#endif

bool MBitmap::IsEmpty(void)
{
 #ifdef COMPILE_SDL
 return s==NULL;
 #else
 return !Btmp;
 #endif
}

//---------------------------------------------------------------------------------------------

MBitmapArrayData::MBitmapArrayData(){Bitmap=NULL; ID=-1; UserData=-1;}
MBitmapArrayData::~MBitmapArrayData(){Clear();}

void MBitmapArrayData::Clear(void)
{
 if(Bitmap!=NULL) delete Bitmap; Bitmap=NULL;
 ID=-1; UserData=-1;
}

void MBitmapArrayData::Assign(const MBitmapArrayData *src)
{
 Clear();
 if(src==NULL) return;
 ID=src->ID;
 UserData=src->UserData;
 if(src->Bitmap!=NULL)
   {
    Bitmap=new MBitmap;
	Bitmap->Assign(src->Bitmap);
   }
}

//---------------------------------------------------------------------------------------------

MBitmapArray::MBitmapArray(){}
MBitmapArray::~MBitmapArray(){Clear();}

MBitmapArrayData *MBitmapArray::Find(int pID, unsigned int &pos)
{
 MBitmapArrayData *obj=NULL;
 for(unsigned int i=0;i<Array.size()&&obj==NULL;i++)
   {
    if(Array[i]!=NULL&&Array[i]->ID==pID) 
	  {
	   obj=Array[i];
	   pos=i;
	  }
   }
 return obj;
}

void MBitmapArray::Assign(const MBitmapArray *src)
{
 Clear();
 if(src==NULL) return;
 MBitmapArrayData *sobj, *obj;
 for(unsigned int i=0;i<src->Array.size();i++)
   {
    sobj=src->Array[i];
	if(sobj!=NULL)
	  {
	   obj=new MBitmapArrayData();
	   obj->Assign(sobj);
	   Array.push_back(obj);
	  }
   }
}

bool MBitmapArray::Add(int pID, MBitmap *pBitmap, int UserData)
{
 unsigned int pos;
 MBitmapArrayData *obj=Find(pID,pos);
 if(obj!=NULL) return false;
 obj=new MBitmapArrayData();
 obj->ID=pID;
 obj->Bitmap=pBitmap;
 obj->UserData=UserData;
 Array.push_back(obj);
 return true;
}

void MBitmapArray::Delete(int pID)
{
 unsigned int pos;
 MBitmapArrayData *obj=Find(pID,pos);
 if(obj!=NULL)
   {
    delete obj;
	Array.erase(Array.begin()+pos);
   } 
}

void MBitmapArray::Clear(void)
{
 MBitmapArrayData *obj;
 while(!Array.empty())
   {
    obj=Array[0];
	Array.erase(Array.begin());
	if(obj!=NULL) delete obj;
   }
}

MBitmap *MBitmapArray::Get(int pID, int *UserData)
{
 unsigned int pos;
 MBitmapArrayData *obj=Find(pID,pos);
 MBitmap *res=NULL;
 if(obj!=NULL) 
   {
    res=obj->Bitmap;
	if(UserData!=NULL) *UserData=obj->UserData;
   }
 return res;
}

int MBitmapArray::GetCount(void)
{
 return Array.size();
}

MBitmap *MBitmapArray::GetItem(int pIndex, int *UserData)
{
 if(pIndex<0||pIndex>=GetCount()) return NULL;
 MBitmapArrayData *obj=Array[pIndex];
 MBitmap *res=NULL;
 if(obj!=NULL) 
   {
    res=obj->Bitmap;
	if(UserData!=NULL) *UserData=obj->UserData;
   }
 return res;
}

#ifndef COMPILE_SDL
void MBitmapArray::RotateAll(bool RotateToLeft, int Count)
{
 MBitmapArrayData *obj;
 for(unsigned int i=0;i<Array.size();i++)
   {
    obj=Array[i];
	if(obj!=NULL&&obj->Bitmap!=NULL) obj->Bitmap->RotateBitmap(RotateToLeft,Count);
   }
}
#endif
//----------------------------------------------------------------------------------------

MAnimationTmpl::MAnimationTmpl()
{
 Clear();
}

MAnimationTmpl::~MAnimationTmpl()
{
 Clear();
}

void MAnimationTmpl::Clear(void)
{
 Initialized=false;
 ID=0;
 MoveX=0; MoveY=0;
 Step=-1;
 DrawAt.left=0; DrawAt.top=0; DrawAt.right=0; DrawAt.bottom=0;
 MoveTo.left=0; MoveTo.top=0; MoveTo.right=0; MoveTo.bottom=0;
 Repeat=0;
 ActRepeat=0;
}

bool MAnimationTmpl::GetInitialized(void){return Initialized;}

//----------------------------------------------------------------------------------------

MAnimationBitmap::MAnimationBitmap()
{
 Clear();
}

MAnimationBitmap::~MAnimationBitmap()
{
 Clear();
}

void MAnimationBitmap::Clear(void)
{
 Bitmap.Clear();
 frameSize.cx=0; frameSize.cy=0;
 frameOffset.x=0; frameOffset.y=0;
 actFrame.left=0; actFrame.top=0; actFrame.right=0; actFrame.bottom=0;
 bitmapSize.cx=0; bitmapSize.cy=0;
}

void MAnimationBitmap::Initialize(myLPRECT pDrawAt, myLPRECT pMoveTo)
{
 Initialized=false;
 if(!pDrawAt||frameSize.cx<=0||frameSize.cy<=0) return;
 DrawAt.left=pDrawAt->left; DrawAt.top=pDrawAt->top; DrawAt.right=pDrawAt->right; DrawAt.bottom=pDrawAt->bottom;
 MoveX=0; MoveY=0;
 Step=0;
 Bitmap.GetSize(bitmapSize);
 int Cnt=bitmapSize.cx/frameSize.cx;
 actFrame.left=frameOffset.x; actFrame.top=frameOffset.y; actFrame.right=frameOffset.x+frameSize.cx; actFrame.bottom=frameOffset.y+frameSize.cy;

 if(Repeat>0) Cnt*=Repeat;
 if(Cnt>0)
   {
    if(pMoveTo!=NULL)
      {
	   MoveTo.left=pMoveTo->left; MoveTo.top=pMoveTo->top; MoveTo.right=pMoveTo->right; MoveTo.bottom=pMoveTo->bottom;
	   MoveX=(MoveTo.left-DrawAt.left)/Cnt;
	   MoveY=(MoveTo.top-DrawAt.top)/Cnt;
      }
   }
  else return;
 ActRepeat=Repeat;
 Initialized=true;
 DrawAt.left+=MoveX; DrawAt.top+=MoveY; 
 DrawAt.right+=MoveX; DrawAt.bottom+=MoveY;
}

bool MAnimationBitmap::Next(void)
{
 if(!Initialized) return false;
 Step++;
 actFrame.left+=frameSize.cx; actFrame.right+=frameSize.cx;
 bool res=Step>=0&&actFrame.right<=bitmapSize.cx&&actFrame.bottom<=bitmapSize.cy;
 if(!res&&(Repeat==-1||ActRepeat>0)) 
   {
    Step=0; 
	actFrame.left=frameOffset.x; actFrame.top=frameOffset.y; actFrame.right=frameOffset.x+frameSize.cx; actFrame.bottom=frameOffset.y+frameSize.cy;
	res=true; 
	if(ActRepeat>0) ActRepeat--; 
	 else ActRepeat=0;
   }
 if(res)
   {
    DrawAt.left+=MoveX; DrawAt.top+=MoveY; DrawAt.right+=MoveX; DrawAt.bottom+=MoveY;
   }
 return res;
}

bool MAnimationBitmap::Draw(MyHDC dc,const myLPRECT pDrawAt,int pMoveX, int pMoveY)
{
 if(dc==NULL||!Initialized) return false;
 if(Step<0) return false;
 myRECT da;
 if(pDrawAt==NULL&&(pMoveX!=0||pMoveY!=0))
   {
    da.left=DrawAt.left+pMoveX; da.top=DrawAt.top+pMoveY;
    da.right=DrawAt.right+pMoveX; da.bottom=DrawAt.bottom+pMoveY;
   }
  else
   {
    da.left=DrawAt.left; da.top=DrawAt.top;
    da.right=DrawAt.right; da.bottom=DrawAt.bottom;
   }
 return Bitmap.Draw(dc,pDrawAt==NULL?&da:pDrawAt,&actFrame);
}

bool MAnimationBitmap::DrawStep(MBitmap *des, int pFrame, const myLPRECT pDrawAt)
{
 if(!des||!Initialized||pFrame<0) return false;
 myRECT r;
 r.left=frameOffset.x+pFrame*frameSize.cx; 
 r.top=frameOffset.y; 
 r.right=frameOffset.x+frameSize.cx+pFrame*frameSize.cx; 
 r.bottom=frameOffset.y+frameSize.cy;
 if(r.right>bitmapSize.cx||r.bottom>bitmapSize.cy) return false;
 myRECT r2={0,0,frameSize.cx,frameSize.cy};
 MyHDC dc=des->BeginPaint();
 bool res=Bitmap.Draw(dc,pDrawAt?pDrawAt:(&r2),&r);
 #ifndef COMPILE_SDL
 des->EndPaint();
 #endif
 return res;
}

bool MAnimationBitmap::DrawFrame(MBitmap *des, int pFrame, const myLPRECT pDrawAt,int pFrameRow)
{
 bool oldinit=Initialized;
 int froffy=frameOffset.y;
 frameOffset.y=pFrameRow*frameSize.cy;
 Initialized=true;
 Bitmap.GetSize(bitmapSize);
 bool res=DrawStep(des,pFrame,pDrawAt);
 frameOffset.y=froffy;
 Initialized=oldinit;
 return res;
}

bool MAnimationBitmap::HaveFrame(int pFrame, int pFrameRow)
{
 Bitmap.GetSize(bitmapSize);
 int px=frameOffset.x+(pFrame*frameSize.cx), py=frameOffset.y+(pFrameRow*frameSize.cy);
 return px>=0&&px+frameSize.cx<=bitmapSize.cx&&py>=0&&py+frameSize.cy<=bitmapSize.cy;
}

bool MAnimationBitmap::DrawFrame(MBitmap *des, int desleft, int destop)
{
 if(!des) return false;
 Bitmap.GetSize(bitmapSize);
 myRECT da={DrawAt.left-desleft,DrawAt.top-destop,DrawAt.right-desleft, DrawAt.bottom-destop};
 MyHDC dc=des->BeginPaint();
 bool res=Draw(dc,&da);
 #ifndef COMPILE_SDL
 des->EndPaint();
 #endif
 return res;
}

bool MAnimationBitmap::Done(void)
{
 if(!Initialized) return true;
 return (Repeat!=-1)&&(Step<0||actFrame.right>bitmapSize.cx);
}

//----------------------------------------------------------------------------------------

MAnimation::MAnimation()
{
 Bitmaps=NULL; Clear();
}

MAnimation::~MAnimation() {Clear();}

void MAnimation::Assign(const MAnimation *src)
{
 Clear();
 if(src==NULL) return;

 ID=src->ID;
 DrawAt.left=src->DrawAt.left; DrawAt.top=src->DrawAt.top;
 DrawAt.right=src->DrawAt.right; DrawAt.bottom=src->DrawAt.bottom;
 MoveTo.left=src->MoveTo.left; MoveTo.top=src->MoveTo.top;
 MoveTo.right=src->MoveTo.right; MoveTo.bottom=src->MoveTo.bottom;
 MoveX=src->MoveX; MoveY=src->MoveY;
 Step=src->Step;
 MoveOnTick=src->MoveOnTick;
 if(src->Bitmaps!=NULL)
   {
    Bitmaps=new MBitmapArray;
	Bitmaps->Assign(src->Bitmaps);
   }
 Initialized=src->Initialized;
 DrawOwnerFirst=src->DrawOwnerFirst;
 Repeat=src->Repeat;
 ActRepeat=src->ActRepeat;
}

void MAnimation::Clear(void)
{
 Initialized=false;
 DrawOwnerFirst=false;
 Repeat=0;
 ActRepeat=0;
 if(Bitmaps!=NULL) delete Bitmaps; Bitmaps=NULL;
 ID=0;
 DrawAt.left=0; DrawAt.top=0; DrawAt.right=0; DrawAt.bottom=0;
 MoveTo.left=0; MoveTo.top=0; MoveTo.right=0; MoveTo.bottom=0;
 MoveX=0; MoveY=0;
 Step=0;
 MoveOnTick=0;
}

void MAnimation::Initialize(myLPRECT pDrawAt, myLPRECT pMoveTo)
{
 Initialized=false;
 if(pDrawAt==NULL||Bitmaps==NULL) return;
 DrawAt.left=pDrawAt->left; DrawAt.top=pDrawAt->top; DrawAt.right=pDrawAt->right; DrawAt.bottom=pDrawAt->bottom;
 MoveX=0; MoveY=0;
 Step=0;
 int Cnt=Bitmaps->GetCount();
 if(Repeat>0) Cnt*=Repeat;
 if(Cnt>0)
   {
    int tm=0;
    MBitmap *obj=Bitmaps->GetItem(Step,&tm);
	if(obj!=NULL)
	  {
	   MoveOnTick=(tm+(clock()*1000/CLOCKS_PER_SEC))*CLOCKS_PER_SEC/1000;
	  }
    if(pMoveTo!=NULL)
      {
	   MoveTo.left=pMoveTo->left; MoveTo.top=pMoveTo->top; MoveTo.right=pMoveTo->right; MoveTo.bottom=pMoveTo->bottom;
	   MoveX=(MoveTo.left-DrawAt.left)/Cnt;
	   MoveY=(MoveTo.top-DrawAt.top)/Cnt;
      }
   }
 ActRepeat=Repeat;
 Initialized=true;
}

bool MAnimation::Next(void)
{
 if(Bitmaps==NULL||!Initialized) return false;
 if(clock()<MoveOnTick) return false;
 Step++;
 bool res=Step>=0&&Step<Bitmaps->GetCount();
 if(!res&&(Repeat==-1||ActRepeat>0)) {Step=0; res=true; if(ActRepeat>0) ActRepeat--; else ActRepeat=0;}
 if(res)
   {
    int tm=0;
	Bitmaps->GetItem(Step,&tm);
	MoveOnTick=(tm+(clock()*1000/CLOCKS_PER_SEC))*CLOCKS_PER_SEC/1000;
    DrawAt.left+=MoveX; DrawAt.top+=MoveY; DrawAt.right+=MoveX; DrawAt.bottom+=MoveY;
	//Draw(dc);
   }
 return res;
}

MBitmap *MAnimation::GetBitmap(void)
{
 if(Bitmaps==NULL||!Initialized) return NULL;
 if(Step<0||Step>=Bitmaps->GetCount()) return NULL;
 return Bitmaps->GetItem(Step);	
}

bool MAnimation::Draw(MyHDC dc,const myLPRECT pDrawAt,int pMoveX, int pMoveY)
{
 if(Bitmaps==NULL||dc==NULL||!Initialized) return false;
 if(Step<0||Step>=Bitmaps->GetCount()) return false;
 MBitmap *obj=Bitmaps->GetItem(Step);	
 if(obj!=NULL)
   {
    myRECT da;
    if(pDrawAt==NULL&&(pMoveX!=0||pMoveY!=0))
	  {
	   da.left=DrawAt.left+pMoveX; da.top=DrawAt.top+pMoveY;
	   da.right=DrawAt.right+pMoveX; da.bottom=DrawAt.bottom+pMoveY;
	  }
	 else
	  {
	   da.left=DrawAt.left; da.top=DrawAt.top;
	   da.right=DrawAt.right; da.bottom=DrawAt.bottom;
	  }
    return obj->Draw(dc,pDrawAt==NULL?&da:pDrawAt);
   }
 return false;
}

#ifndef COMPILE_SDL
void MAnimation::RotateAll(bool RotateToLeft, int Count)
{
 if(Bitmaps!=NULL) Bitmaps->RotateAll(RotateToLeft,Count);
}
#endif

bool MAnimation::Done(void)
{
 if(Bitmaps==NULL) return true;
 return (Repeat!=-1)&&(Step<0||Step>=Bitmaps->GetCount());
}

bool MAnimation::DrawStep(MBitmap *des, int pFrame, const myLPRECT pDrawAt)
{
 if(!des||!Bitmaps) return false;
 MBitmap *btmp=Bitmaps->GetItem(pFrame);
 if(!btmp) return false;
 mySIZE sz;
 btmp->GetSize(sz);
 myRECT r={0,0,sz.cx,sz.cy};
 MyHDC dc=des->BeginPaint();
 bool res=btmp->Draw(dc,pDrawAt?pDrawAt:(&r));
#ifndef COMPILE_SDL
 des->EndPaint();
#endif
 return res;
}

//----------------------------------------------------------------------------------------

MAnimationArray::MAnimationArray(){}
MAnimationArray::~MAnimationArray() {Clear();}

void MAnimationArray::Clear(void)
{
 MAnimation *obj;
 while(!Anims.empty())
   {
    obj=Anims[0];
	Anims.erase(Anims.begin());
	if(obj!=NULL) delete obj;
   }
}

bool MAnimationArray::Add(MBitmapArray *pBitmaps, int pID, bool pDrawOwnerFirst, int pRepeat)
{
 if(pBitmaps==NULL||Get(pID)!=NULL) return false;
 MAnimation *obj=new MAnimation;
 obj->Bitmaps=pBitmaps;
 obj->ID=pID;
 obj->DrawOwnerFirst=pDrawOwnerFirst;
 obj->Repeat=pRepeat;
 Anims.push_back(obj);
 return true;
}

MAnimation *MAnimationArray::Get(int pID)
{
 MAnimation *obj=NULL;
 for(unsigned int i=0;i<Anims.size()&&obj==NULL;i++)
   {
    if(Anims[i]!=NULL&&Anims[i]->ID==pID) obj=Anims[i];
   }
 return obj;
}

int MAnimationArray::GetCount(void){return Anims.size();}

MAnimation *MAnimationArray::GetItem(int pIndex)
{
 if(pIndex<0||pIndex>=Anims.size()) return NULL;
 return Anims[pIndex];
}
