#ifndef MBITMAPS_HEADER
#define MBITMAPS_HEADER

#ifdef COMPILE_SDL
#include <SDL.h>
#include <sdlwintypes.h>
SDL_Rect *RECTtoSDLRect(const myLPRECT rect,SDL_Rect &sdlrect);
typedef SDL_Surface * MyHDC;
#include "sdlwintypes.h"
#else
typedef HDC MyHDC;
#include <windows.h>
#endif
#include <vector>
//#include <time.h>

#define STREAM_SUPPORT_ON

#ifdef STREAM_SUPPORT_ON
#include "MStreams.h"
#endif

//----------------------------------------------------------------------------------------

//zjisteni/nastaveni, s jakou bitovou hloubkou se budou vytvaret bitmapy,
//spolecne pro cely projekt;
UINT GetDefaultBitmapsBitCount(void);
void SetDefaultBitmapsBitCount(UINT val);

//----------------------------------------------------------------------------------------

class MBitmap
{
 private:
   mySIZE sz; //velikost Bitmapy
#ifdef COMPILE_SDL
   SDL_Surface *s;
#else
   HBITMAP Btmp; //bitmapa
   HBITMAP BtmpMask; //maksa BitMapy; muze byt NULL
   MyHDC PaintingDC; //pokud bylo zavolano beginpaint, tak toto neni NULL

   bool Rotate(HBITMAP *pBtmp, bool RotateToLeft, int &Width, int &Height);
   HBITMAP CreateBtmp(int width, int height); //vraci vytvoreni DIBitmapu
#endif
 public:
   MBitmap();
   virtual ~MBitmap();

   myPOINT TransparentColorPos; //kde v bitmape je ulozena pruhledna barva, 0,0 je default

   bool Create(int Width, int Height);
#ifdef COMPILE_SDL
   SDL_Surface *GetSurface(void);
#endif
   MyHDC BeginPaint(void); //kreslim jen do Bitmapy; NUTNO ZAVOLAT EndPaint; HDC jen na predavani do funkci !
   void EndPaint(void);
   void UpdateMask(void); //podle nastavenych TrasparentColorPos
   void UpdateMask(unsigned char r, unsigned char g, unsigned char b); //podle dane barvy
   void UpdateMask(int x, int y); //podle pozice v bitmape, zaroven aktualizuje TrasparentColorPos
   void UpdateMask(const myPOINT pt);
   void EraseMask(void);
   void Assign(const MBitmap *src); //vytvori kopii toho z parametru
   bool LoadFromFile(const char *FileName, bool pCreateMask,myPOINT pTransparentColorPos);
   bool LoadFromFile(const char *FileName, bool pCreateMask=false); //automaticky bitmapa bez masky
   bool SaveToFile(const char *FileName);
#ifdef STREAM_SUPPORT_ON
   bool LoadFromStream(MStream *strm, bool pCreateMask=false); //nacte ze streamu od aktualni pozice bitmapu
#endif
   
   void GetSize(mySIZE &pSize); //vraci velikost bitmapy
   bool Draw(MyHDC dc,myLPRECT desrect, myLPRECT vysekrect=NULL); //vykresli obsah z Btmp do des do rect; pokud ma masku, tak s maskou; vysekrect rika, jaka cast obrazku se ma vykreslit, ji-li NULL, tak cely
#ifndef COMPILE_SDL
   bool RotateBitmap(bool RotateToLeft, int Count); //otoci bitmapou doleve/doprava, pocet je Count, dava se %4
#endif
   void Clear(void); //smaze alokovanou pamet pro tento objekt
   bool IsEmpty(void); //vraci, jestli je bitmapa prazdna
};


//----------------------------------------------------------------------------------------

class MBitmapArrayData
{
 private:
 public:
   MBitmapArrayData();
   virtual ~MBitmapArrayData();

   void Assign(const MBitmapArrayData *src);
   void Clear(void);
   MBitmap *Bitmap;
   int ID;
   int UserData; //volitelne data uzivatele
};

class MBitmapArray
{
 private:
   std::vector<MBitmapArrayData *> Array;
   MBitmapArrayData *Find(int pID, unsigned int &pos);
 public:
   MBitmapArray();
   virtual ~MBitmapArray();

   void Assign(const MBitmapArray *src);
   bool Add(int pID, MBitmap *pBitmap, int UserData=-1); //prida dalsi bitmapu do seznamu
   void Delete(int pID); //smaze bitmapu ze seznamu, pokud pod takovym identifikatorem nejaka je
   void Clear(void);
   MBitmap *Get(int pID, int *UserData=NULL); //najde bitmapu podle ID, vraci NULL kdyz nenasel
   int GetCount(void); //vraci pocet bitmap v seznamu
   MBitmap *GetItem(int pIndex, int *UserData=NULL); //vraci poradove bitmapu, ne podle ID
#ifndef COMPILE_SDL
   void RotateAll(bool RotateToLeft, int Count); //otoci vsemi v seznamu podle pozadavku
#endif
};

//----------------------------------------------------------------------------------------
class MAnimationTmpl
{
 private:
 protected:
   bool Initialized;
   int ActRepeat;
 public:
   MAnimationTmpl();
   virtual ~MAnimationTmpl();

   int ID;
   int MoveX,MoveY; //o kolik se posouva kterym smerem; muze byt obe 0, pak animace na miste
   int Step; //ktery krok prave delam
   myRECT DrawAt, MoveTo; //kam se kresli a kam miri
   int Repeat; //kolikrat se ma zopakovat animace, -1 je nikdy nekoncici animace; default je 0, tedy ze provede jen jednou

   virtual void Clear(void);
   virtual bool GetInitialized(void); //vraci jestli byla animace jiz inicializovana
   virtual void Initialize(myLPRECT pDrawAt, myLPRECT pMoveTo=NULL)=0; //kdyz je MoveTo NULL, tak se nehybe s obrazem
   virtual bool Next(void)=0; //posune na dalsi krok; vraci jestli chce prekreslit
   virtual bool Draw(MyHDC dc,const myLPRECT pDrawAt=NULL,int pMoveX=0, int pMoveY=0)=0; //kdyz je pDrawAt NULL, tak kresli do DrawAt
   virtual bool DrawStep(MBitmap *des, int pFrame, const myLPRECT pDrawAt)=0; //vykresli snimek cislo pFrame do des do obdelniku pDrawAt, ktery kdyz je NULL, tak kresli do [0,0,frameWidht,frameHeight]
   virtual bool Done(void)=0; //vraci jestli uz je animace u konce
};

class MAnimationBitmap: public MAnimationTmpl
{
 private:
   myRECT actFrame;
   mySIZE bitmapSize;
 public:
   MAnimationBitmap();
   virtual ~MAnimationBitmap();

   MBitmap Bitmap;
   mySIZE frameSize; //velikost jednoho snimku (kazdy snimek ma stejnou velikost
   myPOINT frameOffset; //offset prvniho frameu

   void Clear(void);
   void Initialize(myLPRECT pDrawAt, myLPRECT pMoveTo=NULL); //kdyz je MoveTo NULL, tak se nehybe s obrazem
   bool HaveFrame(int pFrame, int pFrameRow=0); //vraci, jestli nactena bitmapa obsahuje dany nimek
   bool Next(void); //posune na dalsi krok; vraci jestli chce prekreslit
   bool Draw(MyHDC dc,const myLPRECT pDrawAt=NULL,int pMoveX=0, int pMoveY=0); //kdyz je pDrawAt NULL, tak kresli do DrawAt
   bool DrawStep(MBitmap *des, int pFrame, const myLPRECT pDrawAt); //vykresli snimek cislo pFrame do des do obdelniku pDrawAt, ktery kdyz je NULL, tak kresli do [0,0,frameWidth,frameHeight]
   bool DrawFrame(MBitmap *des, int pFrame, const myLPRECT pDrawAt,int pFrameRow=0); //stejne jako DrawStep, jen poslendi parametr na vic rika, o kolikaty radek v bitmape mam zajem
   bool DrawFrame(MBitmap *des, int desleft, int destop); //vykresli aktualni Step, desleft a destop rika, jaky ma des offset vuci pDrawAt a pMoveTo z Initialize
   bool Done(void); //vraci jestli uz je animace u konce
};

//----------------------------------------------------------------------------------------

class MAnimation: public MAnimationTmpl
{
 private:
 public:
   MAnimation();
   virtual ~MAnimation();

   MBitmapArray *Bitmaps;
   mDWORD MoveOnTick; //kdy nejpozdeji ma preskocit na dalsi pohyb
   bool DrawOwnerFirst; //jestli ma nejdriv nakreslit vlastnika, tedy jestli je animace jen pridavek k "postave"; default je false

   void Assign(const MAnimation *src); //zkopiruje obsah animace ze src do tohoto
#ifndef COMPILE_SDL
   void RotateAll(bool RotateToLeft, int Count); //pokud neni Bitmaps NULL, tak zavola tuto z Bitmaps
#endif
   void Clear(void);
   bool Done(void); //vraci jestli uz je animace u konce
   MBitmap *GetBitmap(void); //vraci aktualni obrazek, ktery se prehrava
   void Initialize(myLPRECT pDrawAt, myLPRECT pMoveTo=NULL); //kdyz je MoveTo NULL, tak se nehybe s obrazem
   bool Next(void); //posune na dalsi krok; vraci jestli chce prekreslit
   bool Draw(MyHDC dc,const myLPRECT pDrawAt=NULL,int pMoveX=0, int pMoveY=0); //kdyz je pDrawAt NULL, tak kresli do DrawAt
   bool DrawStep(MBitmap *des, int pFrame, const myLPRECT pDrawAt); //vykresli snimek cislo pFrame do des do obdelniku pDrawAt, ktery kdyz je NULL, tak kresli do [0,0,frameWidht,frameHeight]
};

//----------------------------------------------------------------------------------------

class MAnimationArray
{
 private:
   std::vector<MAnimation *> Anims;
 public:
   MAnimationArray();
   ~MAnimationArray();

   void Clear(void);
   bool Add(MBitmapArray *pBitmaps, int pID, bool pDrawOwnerFirst, int pRepeat);
   MAnimation *Get(int pID); //vraci NULL kdyz nenasel s timto ID zadnou animaci
   int GetCount(void); //vraci pocet nactenych animaci
   MAnimation *GetItem(int pIndex); //vraci animaci s danym indexem; NULL kdyz neni
};


#endif
