#ifdef COMPILE_SDL
#include <stdlib.h>
#else
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>
#endif

#include "MStreams.h"
#include <string.h>

//------------------------------------------------------------------------------------------

#ifndef FreePtr
#define FreePtr(x) {if((x)!=NULL) free(x); (x)=NULL;}
#endif

//------------------------------------------------------------------------------------------

#ifdef COMPILE_SDL
#include <SDL_rwops.h>

int SDLCALL toSDL_seek(struct SDL_RWops *context, int offset, int whence)
{
 if(!context||!context->hidden.unknown.data1) return -1;
 MStream *strm=(MStream *)context->hidden.unknown.data1;
 MSeekMode mode=fsBeginning;
 switch(whence)
   {
    case SEEK_SET: mode=fsBeginning; break;
    case SEEK_CUR: mode=fcCurrent; break;
    case SEEK_END: mode=fsEnd; break;
   }
 strm->seek(mode,offset);
 return strm->tell();
}

int SDLCALL toSDL_read(struct SDL_RWops *context, void *ptr, int size, int maxnum)
{
 if(!context||!context->hidden.unknown.data1||!size) return -1;
 MStream *strm=(MStream *)context->hidden.unknown.data1;
 long val=strm->read(ptr,size*maxnum);
 return val/size;
}

int SDLCALL toSDL_write(struct SDL_RWops *context, const void *ptr, int size, int num)
{
 if(!context||!context->hidden.unknown.data1||!size) return -1;
 MStream *strm=(MStream *)context->hidden.unknown.data1;
 if(strm->write(ptr,size*num)) return num;
 return -1;
}

int SDLCALL toSDL_close(struct SDL_RWops *context)
{
 if(!context||!context->hidden.unknown.data1) return -1;
 MStream *strm=(MStream *)context->hidden.unknown.data1;
 strm->close();
 return 0;
}

SDL_RWops *MStreamToSDLRWOps(MStream *strm,SDL_RWops *rwop)
{
 if(!strm) return NULL;
 SDL_RWops *res=rwop;
 if(!res) res=SDL_AllocRW();
 if(res)
   {
    res->seek=toSDL_seek;
	res->read=toSDL_read;
	res->write=toSDL_write;
	res->close=toSDL_close;
	res->hidden.unknown.data1=strm;
   }
 return res;
}

#endif

//------------------------------------------------------------------------------------------

bool MStream::seek(long offset){return seek(fsBeginning,offset);}

int MStream::get(void)
{
 int res;
 if(1!=read(&res,1)) return -1;
 return res;
}

bool MStream::put(int val)
{
 return write(&val,1);
}

//------------------------------------------------------------------------------------------

MMemoryStream::MMemoryStream()
{
 buffer=NULL;
 close();
}

MMemoryStream::~MMemoryStream()
{
 close();
}

bool MMemoryStream::loadFromFile(const char *pFileName)
{
 close();
 if(pFileName==NULL) return false;
 FILE *f=fopen(pFileName,"rb");
 if(f==NULL) return false;
 BYTE b[1025];
 long len;
 bool res=true;
 while(!feof(f)&&res)
   {
    len=fread(b,sizeof(BYTE),1024,f);
	if(len>0) res=write(b,len);
   }
 fclose(f);
 position=0;
 return res;
}

bool MMemoryStream::saveToFile(const char *pFileName)
{
 if(pFileName==NULL||buffer==NULL) return false;
 FILE *f=fopen(pFileName,"wb");
 if(f==NULL) return false;
 bool res=length==long(fwrite(buffer,sizeof(BYTE),length,f));
 fclose(f);
 return res;
}

void MMemoryStream::close(void)
{
 if(!read_only)
   {
    FreePtr(buffer);
   }
  else
   {
    buffer=NULL;
   }
 position=0;
 length=0;
 read_only=false;
}

bool MMemoryStream::eof(void){ return position==length; }
bool MMemoryStream::is_open(void){ return true; }

int MMemoryStream::get(void)
{
 BYTE res=0;
 long val=read(&res,sizeof(BYTE));
 if(val!=sizeof(BYTE)) return -1;
 return res;
}

bool MMemoryStream::put(int val)
{
 BYTE vl=(BYTE)(val);
 return write(&vl,sizeof(BYTE));
}

bool MMemoryStream::seek(MSeekMode mode, long offset)
{
 switch(mode)
   {
    case fsBeginning:
	  position=offset;
	  break;
	case fcCurrent:
	  position=position+offset; 
	  break;
	case fsEnd:
	  position=length+offset;
	  break;
   }
 bool res=position>=0&&position<=length;
 if(position<0) position=0;
 if(position>length) position=length;
 return res;
}

long MMemoryStream::tell(void)
{
 return position;
}

int MMemoryStream::peek(void)
{
 if(buffer==NULL) return -1;
 if(position<0||position>=length) return -1;
 return int(buffer[position]);
}

bool MMemoryStream::alloc(long plength)
{
 close();
 if(length<=0) return true;
 buffer=(BYTE *)malloc(sizeof(BYTE)*plength);
 if(buffer==NULL) return false;
 length=plength;
 position=0;
 return true;
}

long MMemoryStream::get_length(void)
{
 return length;
}

long MMemoryStream::read(void *buff, long bufflen)
{
 if(buffer==NULL||eof()||buff==NULL||bufflen<=0) return -1;
 long readed=0;
 BYTE val, *b_buff=(BYTE *)buff;
 while(!eof()&&readed<bufflen)
   {
    val=buffer[position];
	b_buff[readed]=val;
	position++;
	readed++;
   }
 return readed;
}

bool MMemoryStream::write(const void *buff, long bufflen)
{
 if(buff==NULL||bufflen<=0||read_only) return false;
 if(position+bufflen>=length)
   {
	length=position+bufflen;
	//if(buffer==NULL) buffer
    buffer=(BYTE *)realloc(buffer,sizeof(BYTE)*(length));
   }
 BYTE val,*b_buff=(BYTE *)buff;
 long pos=0;
 while(pos<bufflen)
   {
    val=b_buff[pos];
	buffer[position]=val;
	position++;
	pos++;
   }
 return true;
}

bool MMemoryStream::set_memory(const void *buff, long bufflen)
{
 close();
 if(bufflen<=0||(bufflen>0&&buff==NULL)) return false;
 buffer=(BYTE *)buff;
 length=bufflen;
 position=0;
 read_only=true;
 return true;
}

const void *MMemoryStream::get_memory(void) {return buffer;}
bool MMemoryStream::get_readOnly(void){return read_only;}

//-------------------------------------------------------------------------------------------

MFileStream::MFileStream()
{
 std_file=false;
 peek_val=-1;
 peek_pos=-1;
 file=NULL;
 can_read=false;
 can_write=false;
}

MFileStream::MFileStream(const char *pFileName,const MFileOpenMode mode)
{
 file=NULL;
 std_file=false;
 peek_val=-1;
 peek_pos=-1;
 can_read=false;
 can_write=false;
 open(pFileName,mode);
}

MFileStream::~MFileStream()
{
 close();
}

bool MFileStream::open(const char *pFileName,MFileOpenMode mode)
{
 close();
 std_file=false;
 if(strcmp(pFileName,"stdin")==0)
   {
    file=stdin;
	std_file=true;
	can_read=true;
	can_write=false;
   }
  else
   {
    if(strcmp(pFileName,"stdout")==0)
      {
       file=stdout;
	   std_file=true;
	   can_read=false;
	   can_write=true;
      }
	 else
	  {
       if(strcmp(pFileName,"stderr")==0)
         {
          file=stderr;
	      std_file=true;
		  can_read=false;
		  can_write=true;
         }
	  }
   }

 if(!std_file)
   {
    char *mmode=NULL,*mmode2=NULL;
	switch(mode)
	  {
	   case foBinRead: 
	     mmode="rb";
		 can_read=true;
		 can_write=false;
		 break;
	   case foBinWrite: 
	     mmode="wb"; 
		 can_read=false;
		 can_write=true;
		 break;
	   case foBinReadWrite: 
	     mmode="r+b"; mmode2="w+b"; 
		 can_read=true;
		 can_write=true;
		 break;
	   case foBinAppend: 
	     mmode="ab"; 
		 can_read=false;
		 can_write=true;
		 break;
	   case foTxtRead: 
	     mmode="rt"; 
		 can_read=true;
		 can_write=false;
		 break;
	   case foTxtWrite: 
	     mmode="wt"; 
		 can_read=false;
		 can_write=true;
		 break;
	   case foTxtReadWrite: 
	     mmode="r+t"; mmode2="w+t"; 
		 can_read=true;
		 can_write=true;
		 break;
	   case foTxtAppend: 
	     mmode="at";
		 can_read=false;
		 can_write=true;
		 break;
	  }
	if(mmode==NULL) return false;
    file=fopen(pFileName,mmode);
	if(file==NULL&&mmode2!=NULL) file=fopen(pFileName,mmode2);
   }
 peek_pos=-1;
 peek_val=-1;
 return is_open();
}

void MFileStream::close(void)
{
 if(std_file) file=NULL;
 if(file!=NULL) fclose(file);
 file=NULL;
 std_file=false;
 peek_val=-1;
 peek_pos=-1;
 can_read=false;
 can_write=false;
}

bool MFileStream::eof(void)
{
 return !is_open()||!canread()||feof(file);
}

bool MFileStream::is_open(void)
{
 return file!=NULL;
}

int MFileStream::get(void)
{
 if(eof()||!canread()) return -1;
 int res;
 if(peek_pos!=-1) res=peek_val;
  else res=fgetc(file);
 peek_pos=-1;
 return res;
}

bool MFileStream::put(int val)
{
 if(!is_open()||!canwrite()) return false;
 if(peek_pos!=-1) seek(fsBeginning,peek_pos);
 peek_pos=-1;
 return fputc(val,file)!=EOF;
}

bool MFileStream::seek(MSeekMode mode, long offset)
{
 if(!is_open()||!canread()) return false;
 int origin=-1;
 switch(mode)
   {
    case fsBeginning: origin=SEEK_SET; break;
	case fcCurrent: origin=SEEK_CUR; break;
	case fsEnd: origin=SEEK_END; break;
	default: return false;
   }
 peek_pos=-1;
 return fseek(file,offset,origin)==0;
}

long MFileStream::tell(void)
{
 if(!is_open()||!canread()) return -1;
 long ret=ftell(file);
 if(peek_pos!=-1) ret=peek_pos;
 return ret;//ftell(file);
}

int MFileStream::peek(void)
{
 if(!is_open()||!canread()) return -1;
 if(peek_pos==-1)
   {
    peek_pos=tell();
    peek_val=fgetc(file); //get();
   }
 return peek_val;
}

FILE *MFileStream::get_stream(void)
{
 return file;
}

bool MFileStream::canread(void)
{
 return is_open()&&can_read;
}

bool MFileStream::canwrite(void)
{
 return is_open()&&can_write;
}

long MFileStream::read(void *buff, long bufflen)
{
 if(!canread()||!buff||bufflen<=0) return -1;
 return fread(buff,1,bufflen,file);
}

bool MFileStream::write(const void *buff, long bufflen)
{
 if(!canwrite()||!buff||bufflen<=0) return false;
 return fwrite(buff,1,bufflen,file)!=0;
}

//------------------------------------------------------------------------
/*
MFileReadStream::MFileReadStream(long pcachesize)
{
 file=NULL;
 cache=NULL;
 cachesize=pcachesize;
 if(cachesize<0) cachesize=1024;
 cachepos=0;
 cachemax=-1;
 filepos=0;
 peek_val=-1;
 peek_pos=-1;
}

MFileReadStream::MFileReadStream(const char *pFileName,const MFileOpenMode mode,long pcachesize)
{
 file=NULL;
 cache=NULL;
 cachesize=pcachesize;
 if(cachesize<0) cachesize=1024;
 cachepos=0;
 cachemax=-1;
 filepos=0;
 peek_val=-1;
 peek_pos=-1;
 open(pFileName,mode);
}

MFileReadStream::~MFileReadStream()
{
 if(file!=NULL) fclose(file); file=NULL;
 FreePtr(cache);
}

void MFileReadStream::read_next_block(void)
{
 if(cache==NULL) return;
 cachemax=fread(cache,sizeof(BYTE),cachesize,file);
 cachepos=0;
 peek_val=-1;
 peek_pos=-1;
}

bool MFileReadStream::open(const char *pFileName,const MFileOpenMode mode)
{
 close();
 if(pFileName==NULL) return false;

 char *mmode=NULL;
 switch(mode)
   {
    case foBinRead: 
      mmode="rb";
	  break;
	case foTxtRead: 
	  mmode="rt"; 
	  break;
  }
 if(mmode==NULL) return false;
 file=fopen(pFileName,mmode);
 cachepos=0;
 cachemax=cachesize;
 filepos=0;
 set_cachesize(cachesize);
 peek_val=-1;
 peek_pos=-1;
 return is_open();
}

void MFileReadStream::close(void)
{
 if(file!=NULL) fclose(file);
 file=NULL;
 FreePtr(cache);
}

bool MFileReadStream::eof(void)
{
 if(!is_open()) return true;
 if(cache==NULL) return feof(file)!=0;
 return cachepos<cachemax||(cachepos==cachemax&&feof(file)!=0);
}

bool MFileReadStream::is_open(void){return file!=NULL;}

int MFileReadStream::get(void)
{
 if(eof()) return -1;
 int res;
 if(peek_pos!=-1) res=peek_val;
  else
   {
    if(cache==NULL) res=fgetc(file);
	 else
	  {
       if(cachepos<cachemax)
	     {
	      res=cache[cachepos];
	      cachepos++;
	      if(cachepos==cachemax&&cachemax==cachesize&&!eof())
	        {
			 filepos=filepos+cachepos;
		     read_next_block();
		    }
		 }
	  }
   }
 peek_pos=-1;
 return res;
}

bool MFileReadStream::put(int val)
{
 return false;
}

bool MFileReadStream::seek(MSeekMode mode, long offset)
{
 if(!is_open()) return false;
 long newpos=filepos;
 int origin=-1;
 switch(mode)
   {
    case fsBeginning: origin=SEEK_SET; newpos=offset; break;
	case fcCurrent: origin=SEEK_CUR; newpos+=offset; break;
	case fsEnd: origin=SEEK_END; newpos+=offset; break;
	default: return false;
   }
 peek_pos=-1;
 bool res=false;
 if(cache!=NULL)
   {
    if(newpos<0) newpos=0;
	if(newpos>=filepos&&newpos<filepos+cachemax)
	  {
	   cachepos=newpos-filepos;
	   res=true;
	  }
	 else
	  {
	   res=fseek(file,newpos,SEEK_SET)==0;
	   filepos=newpos;
	   read_next_block();
	  }
   }
  else
   {
    res=fseek(file,offset,origin)==0;
   }
 return res;
 
}

long MFileReadStream::tell(void)
{
 if(eof()) return -1;
 long ret=ftell(file);
 if(cache!=NULL)
   {
    ret=filepos+cachepos;
   }
 if(peek_pos!=-1) ret=peek_pos;
 return ret;//ftell(file);
}

int MFileReadStream::peek(void)
{
 if(eof()) return -1;
 if(cache==NULL||cachepos==cachemax)
   {
    if(peek_pos==-1)
      {
       peek_pos=tell();
       peek_val=fgetc(file); //get();
      }
   }
  else
   {
    if(cachepos<cachemax&&cachepos>0) peek_val=cache[cachepos];
   }
 return peek_val;
}

long MFileReadStream::get_cachesize(void){return cachesize;}
void MFileReadStream::set_cachesize(long pVal)
{
 long fp=tell();
 if(fp<0) return;
 cachesize=pVal;
 FreePtr(cache);
 if(cachesize>0)
   {
    cache=(BYTE *)malloc(sizeof(BYTE)*cachesize);
   }
 filepos=fp;
 read_next_block();
}
*/
