#ifndef MSTREAMS_HEADER
#define MSTREAMS_HEADER

#ifndef COMPILE_SDL
#ifdef _DEBUG
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>
#endif
#endif

#include <stdio.h>

#ifdef COMPILE_SDL
#include <SDL_rwops.h>
class MStream;
SDL_RWops *MStreamToSDLRWOps(MStream *strm,SDL_RWops *rwop); //vraci nove alokovany SDL_AllocRW, chce pak parove zavolat SDL_FreeRW, pokud je parametr rwop NULL, jinak prenastavi u rwop hodnoty na tento stream
#endif

enum MSeekMode{fsBeginning, fcCurrent, fsEnd}; //odkud seekovat v souboru
typedef unsigned char BYTE;

class MStream 
{
 private:
 public:
   MStream(){}
   virtual ~MStream(){}

   virtual void close(void)=0;
   virtual bool eof(void)=0;
   virtual bool is_open(void)=0;
   virtual int get(void); //vrati znak z otevreneho souboru, vraci -1 kdyz nepovedlo
   virtual bool put(int val); //vsune znak do souboru, vrati false kdyz nepovedlo
   virtual bool seek(MSeekMode mode, long offset)=0; //posune podle volby ukazatel v souboru; vraci jestli povedlo
   virtual bool seek(long offset); //posune od zacatku o offset vraci jestli povedlo
   virtual long tell(void)=0; //vrati pozici v souboru; -1 kdyz chyba
   virtual int peek(void)=0; //pohled jeden znak dopredu, -1 je chyba/eof
   virtual long read(void *buff, long bufflen)=0; //nacte do promenne buff hodnotu delky bufflen od aktualni pozice; vraci kolik prenesl skutecne byteu; vraci -1 kdyz neco nepovedlo
   virtual bool write(const void *buff, long bufflen)=0; //zapise od aktualni pozice do streamu obsah buff, bufflen BYTEu, pripadne se pokusi zvetsit buffer; vraci false, kdyz nepovedlo
};

//-------------------------------------------------------------------------------------------

class MMemoryStream:public MStream
{
 private:
   long position;
   long length;
   BYTE *buffer;
   bool read_only;
 public:
   MMemoryStream();
   virtual ~MMemoryStream();

   bool loadFromFile(const char *pFileName);
   bool saveToFile(const char *pFileName);
   void close(void);
   bool eof(void);
   bool is_open(void);
   int get(void); //vrati znak z otevreneho souboru, vraci -1 kdyz nepovedlo
   bool put(int val); //vsune znak do souboru, vrati false kdyz nepovedlo
   bool seek(MSeekMode mode, long offset); //posune podle volby ukazatel v souboru; vraci jestli povedlo
   long tell(void); //vrati pozici v souboru; -1 kdyz chyba
   int peek(void); //pohled jeden znak dopredu, -1 je chyba/eof

   bool alloc(long plength); //vymaze aktualni buffer a alokuje novy, dane velikosti; position nastavi na zacatek streamu
   long get_length(void); //vraci velikost alokovaneho bufferu
   long read(void *buff, long bufflen); //nacte do promenne buff hodnotu delky bufflen od aktualni pozice; vraci kolik prenesl skutecne byteu; vraci -1 kdyz neco nepovedlo
   bool write(const void *buff, long bufflen); //zapise od aktualni pozice do streamu obsah buff, bufflen BYTEu, pripadne se pokusi zvetsit buffer; vraci false, kdyz nepovedlo
   
   bool set_memory(const void *buff, long bufflen); //vymaze obsah streamu, nastavi obsah streamu na hodnotu z parametru a bude stream ted jen ke cteni
   const void *get_memory(void); //pujci obsah pameti ven
   bool get_readOnly(void);
};

//-------------------------------------------------------------------------------------------

enum MFileOpenMode{foBinRead, foBinWrite, foBinReadWrite, foBinAppend,
				   foTxtRead, foTxtWrite, foTxtReadWrite, foTxtAppend}; 

class MFileStream:public MStream
{
 private:
   FILE *file;
   bool std_file;
   int peek_val;
   long peek_pos;
   bool can_read;
   bool can_write;
 public:
   MFileStream();
   MFileStream(const char *pFileName,const MFileOpenMode mode=foBinRead);
   virtual ~MFileStream();

   bool open(const char *pFileName,const MFileOpenMode mode=foBinRead); //pFileName muze byt "stdout","stdin","stderr", pak "otevre" tyto streamy misto klasickyho file
   void close(void);
   bool eof(void);
   bool is_open(void);
   int get(void); //vrati znak z otevreneho souboru, vraci -1 kdyz nepovedlo
   bool put(int val); //vsune znak do souboru, vrati false kdyz nepovedlo
   bool seek(MSeekMode mode, long offset); //posune podle volby ukazatel v souboru; vraci jestli povedlo
   long tell(void); //vrati pozici v souboru; -1 kdyz chyba
   int peek(void); //pohled jeden znak dopredu, -1 je chyba/eof
   FILE *get_stream(void); //pro potreby fprintf...; MUZE BYT I NULL
   bool canread(void); //vraci jestli otevreno pro cteni
   bool canwrite(void); //vraci jestli otevreno pro zapis
   long read(void *buff, long bufflen); //nacte do promenne buff hodnotu delky bufflen od aktualni pozice; vraci kolik prenesl skutecne byteu; vraci -1 kdyz neco nepovedlo
   bool write(const void *buff, long bufflen); //zapise od aktualni pozice do streamu obsah buff, bufflen BYTEu, pripadne se pokusi zvetsit buffer; vraci false, kdyz nepovedlo

   MFileStream &operator<<(const int pVal)
     {
      if(file!=NULL) fprintf(file,"%d",pVal);
	  return *this;
     }

   MFileStream &operator<<(const unsigned int pVal)
     {
      if(file!=NULL) fprintf(file,"%d",pVal);
	  return *this;
     }

   MFileStream &operator<<(const long int pVal)
     {
      if(file!=NULL) fprintf(file,"%ld",pVal);
	  return *this;
     }

   MFileStream &operator<<(const unsigned long int pVal)
     {
      if(file!=NULL) fprintf(file,"%ld",pVal);
	  return *this;
     }

   MFileStream &operator<<(const double pVal)
     {
	  FILE *f=file;//strm.get_stream();
      if(f!=NULL) fprintf(f,"%g",pVal);
	  return *this;
	 }

   MFileStream &operator<<(const char pVal)
     {
      FILE *f=file;//strm.get_stream();
      if(f!=NULL) fprintf(f,"%c",pVal);
	  return *this;
     }

   MFileStream &operator<<(const bool pVal)
     {
      FILE *f=file;//strm.get_stream();
      if(f!=NULL) fprintf(f,"%s",pVal?"1":"0");
	  return *this;
     }

   MFileStream &operator<<(const char *pVal)
     {
      FILE *f=file;//strm.get_stream();
      if(f!=NULL&&pVal!=NULL) fprintf(f,"%s",pVal);
	  return *this;
     }
};

//-----------------------------------------------------------------------------------------
/*
class MFileReadStream:public MStream
{
 private:
   FILE *file;
   BYTE *cache;
   long cachesize; //default je 1024
   long cachepos; //0 <= cachepos < cachesize
   long cachemax;
   long filepos; //cache[0] ukazuje na tuto pozici v souboru
   int peek_val;
   long peek_pos;
   
   void read_next_block(void);

 public:
   MFileReadStream(long pcachesize=1024);
   MFileReadStream(const char *pFileName,const MFileOpenMode mode=foBinRead,long pcachesize=1024);
   virtual ~MFileReadStream();

   bool open(const char *pFileName,const MFileOpenMode mode=foBinRead); //pFileName je odkaz na klasicky file; nacita po velikostech bloku cache; akceptuje jen foBinRead a foTextRead
   void close(void);
   bool eof(void);
   bool is_open(void);
   int get(void); //vrati znak z otevreneho souboru, vraci -1 kdyz nepovedlo
   bool put(int val); //vsune znak do souboru, vrati false kdyz nepovedlo
   bool seek(MSeekMode mode, long offset); //posune podle volby ukazatel v souboru; vraci jestli povedlo
   long tell(void); //vrati pozici v souboru; -1 kdyz chyba
   int peek(void); //pohled jeden znak dopredu, -1 je chyba/eof

   long get_cachesize(void);
   void set_cachesize(long pVal); //nastavi novou velikost cache, zmena se projevi az pri dalsim posuvu, premrsteni hodnoty muze nastavit vynulovani cache; cashe se naplni podle potreby
};
*/

#endif
