#include "MTimers.h"
#include <vector>

using namespace std;

MTimers mainGlobalTimers;

MTimers *GetMainTimers(void)
{
 return &mainGlobalTimers;
}

#ifdef COMPILE_SDL

void KillTimer(myHWND hWnd,SDL_TimerID timerID)
{
 SDL_RemoveTimer(timerID);
}

Uint32 OnSDLTimerCallback(Uint32 interval, void *param)
{
 //mainGlobalTimers.WorkWithTimer(NULL,int(param));
 SDL_Event user_event;
 user_event.type=SDL_USEREVENT;
 user_event.user.code=MY_SDL_TIMER_EVENT;
 user_event.user.data1=param;
 user_event.user.data2=NULL;
 SDL_PushEvent(&user_event);
 return interval;
}

SDL_TimerID SetTimer(myHWND hWnd,UINT iTimerID,UINT iElapsed,void *ptr)
{
 return SDL_AddTimer(iElapsed,OnSDLTimerCallback,(void *)(iTimerID));
}

#endif

MTimers::MTimers(void)
{
 hWnd=NULL;
 MOnTimerEvent=NULL;
}

MTimers::~MTimers()
{
 Clear();
}

void MTimers::SetHWND(myHWND phWnd)
{
 hWnd=phWnd;
}

void MTimers::Clear(void)
{
#ifndef COMPILE_LINUX
 SetAllDisabled(); //tohle by se melo volat VZDY, ale protoze na LINU 
                   //RH9 to zacykli program pri ukonceni, tak se nevola
                   //nelibi!
#endif
 Timers.clear();
}

void MTimers::SetAllDisabled(void)
{
 unsigned int i;
 for(i=0;i<Timers.size();i++)
   {
    DoEnableTimer(&(Timers[i]),false,true);
   }
}

UINT MTimers::GetNextFreeID(void)
{
 UINT  mTimerID=1;
 MTimersStruct ts;
 unsigned int i;
 for(i=0;i<Timers.size();i++)
   {
    ts=Timers[i];
	if(mTimerID==ts.iTimerID) mTimerID++;
   }
 return mTimerID;
}

bool MTimers::FindTimer(UINT pTimerID, UINT *index)
{
 bool res=false;
 MTimersStruct ts;
 unsigned int i;
 for(i=0;i<Timers.size()&&!res;i++)
   {
    ts=Timers[i];
	if(pTimerID==ts.iTimerID) 
	  {
	   res=true;
	   if(index!=NULL) *index=i;
	  }
   }
 return res;
}

bool MTimers::DoEnableTimer(MTimersStruct *ts,bool pEnabled, bool Paused)
{
 if(ts==NULL||MOnTimerEvent==NULL) return false;
 if(ts->Enabled==pEnabled&&!ts->Paused&&!Paused) return true;
 if(ts->Enabled&&(!ts->Paused||!Paused))
   {
    KillTimer(hWnd,ts->iSysTimerID);
	ts->Paused=Paused;
	if(!Paused) ts->Enabled=false;
   }
  else
   {
	ts->iSysTimerID=SetTimer(hWnd,ts->iTimerID,ts->iElapsed,NULL);
	ts->Enabled=ts->iSysTimerID!=0;
	ts->Paused=!ts->Enabled;
   }
 return ts->Enabled==pEnabled;
}

bool MTimers::TimerAvailable(UINT pEventIdentif, UINT *index)
{
 bool res=false;
 MTimersStruct ts;
 unsigned int i;
 for(i=0;i<Timers.size()&&!res;i++)
   {
    ts=Timers[i];
	if(pEventIdentif==ts.iEventIdentif) 
	  {
	   res=true;
	   if(index!=NULL) *index=i;
	  }
   }
 return res;
}

void MTimers::DelInVector(vector<MTimersStruct> *vect,UINT index)
{
 if(vect==NULL) return;
 if(index>vect->size()) return;
 vect->erase(vect->begin()+index);
}

void MTimers::SetOnTimerEvent(MOnTimerEventCallback pOnTimerEvent)
{
 MOnTimerEvent=pOnTimerEvent;
}

MOnTimerEventCallback MTimers::GetOnTimerEvent(void){return MOnTimerEvent;}
bool MTimers::AddTimer(UINT pEventIdentif, UINT pElapsed)
{
 if(TimerAvailable(pEventIdentif,NULL)) return false; 
 MTimersStruct ts;
 ts.Enabled=false;
 ts.iElapsed=pElapsed;
 ts.iEventIdentif=pEventIdentif;
 ts.iTimerID=GetNextFreeID();
 ts.Paused=false;
 if(!DoEnableTimer(&ts,true,false)) return false;

 Timers.push_back(ts);
 return true; 
}

void MTimers::DelTimer(UINT pEventIdentif)
{
 UINT index;
 if(TimerAvailable(pEventIdentif,&index))
   {
    DoEnableTimer(&(Timers[index]),false,false);
	DelInVector(&Timers,index);
   }
}

void MTimers::SetTimerEnabled(UINT pEventIdentif, bool pEnabled)
{
 UINT index;
 if(TimerAvailable(pEventIdentif,&index))
   {
    DoEnableTimer(&(Timers[index]),pEnabled,false);
   }
}

void MTimers::GetTimerEnabled(UINT pEventIdentif, bool &pEnabled)
{
 pEnabled=false;
 UINT index;
 if(TimerAvailable(pEventIdentif,&index))
   {
    pEnabled=Timers[index].Enabled;
   }
}

void MTimers::SetTimerElapsed(UINT pEventIdentif, UINT pElapsed)
{
 UINT index;
 if(TimerAvailable(pEventIdentif,&index))
   {
	Timers[index].iElapsed=pElapsed;
    if(Timers[index].Enabled)
	  {
	   DoEnableTimer(&(Timers[index]),false,false);
	   DoEnableTimer(&(Timers[index]),true,false);
	  }
   }
}

void MTimers::GetTimerElapsed(UINT pEventIdentif, UINT &pElapsed)
{
 pElapsed=0;
 UINT index;
 if(TimerAvailable(pEventIdentif,&index))
   {
    pElapsed=Timers[index].iElapsed;
   }
}

bool MTimers::WorkWithTimer(myHWND hWnd, UINT pTimerID)
{
 UINT index;
 if(MOnTimerEvent==NULL || !FindTimer(pTimerID,&index)) return false;
 if(!DoEnableTimer(&(Timers[index]),false,false)) return false;
 if(MOnTimerEvent!=NULL) MOnTimerEvent(hWnd,Timers[index].iEventIdentif);
 return DoEnableTimer(&(Timers[index]),true,false);
}

void MTimers::PauseTimers(void)
{
 unsigned int i;
 for(i=0;i<Timers.size();i++)
   {
    if(Timers[i].Enabled&&!Timers[i].Paused) 
	  {
	   DoEnableTimer(&(Timers[i]),true,true);
	  }
   }
}

void MTimers::RunTimers(void)
{
 unsigned int i;
 for(i=0;i<Timers.size();i++)
   {
    if(Timers[i].Enabled&&Timers[i].Paused) 
	  {
	   DoEnableTimer(&(Timers[i]),true,true);
	  }
   }
}
