#ifndef MTIMERS_HEADER_INC
#define MTIMERS_HEADER_INC

#ifdef COMPILE_SDL
#include <SDL.h>
//#ifndef _WINDOWS_
typedef Uint32 UINT;
typedef void * myHWND;
//#endif

#define MY_SDL_TIMER_EVENT 1

#else
#include <windows.h>
typedef HWND myHWND;
#endif
#include <vector>

typedef struct 
{
 UINT iTimerID; //ID timeru pro mne
 UINT iEventIdentif; //uzivatelem zadane ID
 bool Enabled; //jestli je prave aktivni
 UINT iElapsed; //kolik milisekund do zavolani
#ifdef COMPILE_SDL
 SDL_TimerID
#else
 UINT 
#endif
 iSysTimerID; //ID timeru ve windows
 bool Paused; //jestli je timer pozastaven 
}MTimersStruct;

typedef void (* MOnTimerEventCallback)(myHWND hWnd, UINT pEventIdentif); //callback po vyvolani timeru

class MTimers
{
 private:
   std::vector<MTimersStruct> Timers;
   myHWND hWnd;
   MOnTimerEventCallback MOnTimerEvent; //uzivatelem nastaveny OnTimerEvent; pokud neni definovany, tak Enabled automaticky na false; pri zruseni se zrusi i vsechny timery
   void SetAllDisabled(void); //zrusi sechny timery
   UINT GetNextFreeID(void); //vraci prvni volny ID pro system timer
   bool TimerAvailable(UINT pEventIdentif, UINT *index); //jestli existuje Timer podle uzivatelem zadane identifikace
   bool FindTimer(UINT pTimerID, UINT *index); //podle iTimerID hleda v timerech a vraci jestli nasel ci ne
   bool DoEnableTimer(MTimersStruct *ts,bool pEnabled, bool Paused); //nastavi podle pozadavku tento timer na Enabled true/false
   void DelInVector(std::vector<MTimersStruct> *vect,UINT index); //smaze ve vectoru vect prvek s indexem index
 public:
   MTimers(void);
   virtual ~MTimers();
   void SetHWND(myHWND phWnd); //okno, na ktere se maji vazat timery
   void SetOnTimerEvent(MOnTimerEventCallback pOnTimerEvent); //nastavi uzivatelem definovany OnTimerEvent
   MOnTimerEventCallback GetOnTimerEvent(void); //vrati prave pouzivany OnTimerEvent
   bool AddTimer(UINT pEventIdentif, UINT pElapsed); //prida timer, zapne se automaticky na enabled
   void DelTimer(UINT pEventIdentif); //odstrani timer
   void SetTimerEnabled(UINT pEventIdentif, bool pEnabled); //nastavi jestli je timer aktivni ci neni; raci false kdyz se neco nepovedlo, napriklad neni timer tohoto cisla
   void GetTimerEnabled(UINT pEventIdentif, bool &pEnabled); //vraci jestli je timer aktivni
   void SetTimerElapsed(UINT pEventIdentif, UINT pElapsed); //zmeni kdy vyvola timer
   void GetTimerElapsed(UINT pEventIdentif, UINT &pElapsed); //vrati kdy vyvola timer
   bool WorkWithTimer(myHWND hWnd, UINT pTimerID); //volana z WndProc; s parametrem wParam
   void Clear(void); //vymaze vsechny nastaveny timery
   void PauseTimers(void); //pozastavi vsechny aktivni timery
   void RunTimers(void); //spusti vsechnypozastavene timery
};

MTimers *GetMainTimers(void); //vraci globalni ukazatel na tridu timeru, nikdy se nesmi uvolnit

#endif
