#ifdef COMPILE_SDL
#include <sdlwintypes.h>
#else
#include <windows.h>
#endif
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "datafile.h"
#include "sound.h"

#ifdef COMPILE_LINUX                                                                                                                           
#include <ctype.h>
int strnicmp(const char *str1,const char *str2, int max); //ve windows je v string.h, v linuxu neni
#endif

const char *GetExeDir(void);
const char *GetDataDir(void);

#ifndef FreePtr
#define FreePtr(x) if(x!=NULL) free(x); x=NULL
#endif

class MDataFile
{
 private:
   BYTE *mem; //pamet s udaji o identifikatorech, nactena z ulozeni
   mDWORD memsize; //kolik bere v pameti mem
 public:
   MDataFile();
   ~MDataFile();

   bool DoLoad(const char *pResourceName); //nacte obsah ze souboru
   bool DoFind(const char *pIdentName, UINT *pData); //vraci false kdyz nenasel identifikator, jinak kdyz neni pData NULL, tak u nej zavola Assign na vysledek v seznamu
   void DoClear(void); //vymaze vse z pameti, parametr rika, jestli ma mazat i polozku Data z prvku v seznamu
};

MDataFile::MDataFile(){mem=NULL;memsize=0; /*DoLoad("IDENTS");*/}

MDataFile::~MDataFile(){DoClear();}

#define DATFILEHEAD "Dat@2005"
#define TXTXORVAL (1+4+16+64)
#define DATAFILENAME "Berusky1.dat"

int GetTextFileXorVal(void){return TXTXORVAL;}

bool MDataFile::DoLoad(const char *pFileName)
{
 DoClear();
 if(pFileName==NULL) {return false;}
 
 FILE *f=fopen(pFileName,"rb");
 if(!f) return false;
 int i=0,ch;
 while(ch=fgetc(f),!feof(f)&&DATFILEHEAD[i]&&ch==DATFILEHEAD[i])
   {
    i++;
   }
 if(ch!=15||feof(f)||DATFILEHEAD[i]!=0)
   {
    fclose(f);
	return false;
   }
// mDWORD memsize;
 if(1!=fread(&memsize,sizeof(mDWORD),1,f))
   {
	memsize=0;
    fclose(f);
	return false;
   }
 mem=(BYTE *)malloc(memsize);
 if(!mem)
   {
    memsize=0;
    fclose(f);
    return false;
   }
 if(1!=fread(mem,memsize,1,f))
   {
    memsize=0;
    fclose(f);
	return false;
   }
 fclose(f);
 unsigned int count;
 memcpy(&count,mem,sizeof(unsigned int));
 return true;
}

bool MDataFile::DoFind(const char *pIdentName, UINT *pData)
{
 if(pIdentName==NULL) return false;
 mDWORD pos=sizeof(unsigned int),posfrom;
 mDWORD num,val=(mDWORD)-1,identidx,idx;
 BYTE *buff=mem;
 identidx=0;
 bool have=false,done=false,first=true;
 while(!have&&!done&&pos>0&&pos<memsize)
   {
    posfrom=pos;
    done=pIdentName[identidx]==0||pIdentName[identidx+1]==0;
    while(buff[pos]!=0&&pos<memsize)
	  {
	   if(!have&&(toupper(buff[pos])==toupper(pIdentName[identidx])||(done&&!first&&buff[pos]=='#'))) {have=true; idx=pos-posfrom;}
	   pos++;
	  }
	if(pos<memsize) pos+=1;
	if(!have) {done=true;}
	 else
	  {
	   if(done&&first) 
	     {
		  first=false;
		  done=false;
		 }
	   have=false;
	   if(done) idx=pos-posfrom-1;
	   memcpy(&num,buff+(pos+sizeof(mDWORD)*(idx)),sizeof(mDWORD));
	   if(done)
	     {
		  have=num!=-1;
		  if(have) val=num;
		 }
	    else 
		 {
		  if(first) identidx++;
		 }
	   pos=num;
	  }
   }
 if(have&&pData!=NULL) *pData=val;
 return have;
}

void MDataFile::DoClear(void)
{
 FreePtr(mem);
 memsize=0;
}

MDataFile idents;

void ClearDataFile(void)
{
 idents.DoClear();
}

void WriteToLog(const char *format,...);
bool OpenDataFile(const char *pFileName);

bool LoadDataFile(const char *pFileName)
{
 if(!pFileName)
   {
    char file[1024];
	strcpy(file,GetExeDir());
	strcat(file,DATAFILENAME);
    if(!idents.DoLoad(file)||!OpenDataFile(file))
	  {
	   WriteToLog("LoadDataFile '%s' failed.",file);
	   return false;
	  }
	/*const char *ident="Data\\Graphics\\editor.bmp";
	UINT dt;
	bool have=idents.DoFind(ident,&dt);
	if(have) WriteToLog("LoadDataFile at %s find %s at %d",file,ident,dt);
	 else WriteToLog("LoadDataFile at %s not found %s",file,ident,dt);
	return true;*/
   }
  else
   {
    if(!idents.DoLoad(pFileName)||!OpenDataFile(pFileName))
	  {
	   WriteToLog("LoadDataFile '%s' failed.",pFileName);
	   return false;
	  }
   }
 return true;
}

bool FindInDataFile(const char *pIdentName, UINT *pData)
{
 return idents.DoFind(pIdentName,pData);
}

//-----------------------------------------------------------------------------

class MDataStream:public MStream 
{
 private:
   long start,length;
   FILE *file;
 public:
   MDataStream();
   virtual ~MDataStream();

   void close(void);
   bool eof(void);
   bool is_open(void);
   bool seek(MSeekMode mode, long offset);
   long tell(void);
   int peek(void);
   long read(void *buff, long bufflen);
   bool write(const void *buff, long bufflen);

   bool OpenFile(const char *pFileName); //otevre soubor ke cteni
   bool SetRange(UINT pstart, UINT plength); //nastavi rozsah cteni ze souboru, od pstart precte maximalne plength byteu; jinam stream nepusti
};

MDataStream::MDataStream()
{
 start=0;
 length=0;
 file=NULL;
}

MDataStream::~MDataStream()
{
 close();
}

void MDataStream::close(void)
{
 if(file) fclose(file);
 file=NULL;
}

bool MDataStream::eof(void)
{
 return !is_open()||ftell(file)>start+length;
}

bool MDataStream::is_open(void)
{
 return file!=NULL;
}

bool MDataStream::seek(MSeekMode mode, long offset)
{
 if(!is_open()) return false;
 int origin=-1;
 switch(mode)
   {
    case fsBeginning: origin=SEEK_SET; offset+=start; break;
	case fcCurrent: origin=SEEK_CUR; break;
	case fsEnd: origin=SEEK_END; break;
	default: return false;
   }
 bool res=(fseek(file,offset,origin)==0);
 if(res)
   {
    long pos=ftell(file);
    if(pos<start) fseek(file,start,SEEK_SET);
	 else
	  {
	   if(pos>start+length) fseek(file,start+length,SEEK_SET);
	  }
   }
 return res;
}

long MDataStream::tell(void)
{
 if(!is_open()) return -1;
 if(eof()) return length;
 return ftell(file)-start;
}

int MDataStream::peek(void){return -1;}

long MDataStream::read(void *buff, long bufflen)
{
 if(!is_open()||!buff||bufflen<=0||eof()||tell()<0) return -1;
 if(start+bufflen-ftell(file)>length) 
   bufflen=length-(ftell(file)-start);
 return fread(buff,1,bufflen,file);
}

bool MDataStream::write(const void *buff, long bufflen)
{
 return false;
}

bool MDataStream::OpenFile(const char *pFileName)
{
 close();
 if(!pFileName) return false;
 file=fopen(pFileName,"rb");
 return file!=NULL;
}

bool MDataStream::SetRange(UINT pstart, UINT plength)
{
 if(!is_open()) return false;
 start=pstart;
 length=plength;
 return 0==fseek(file,start,SEEK_SET);
}

//-----------------------------------------------------------------------------

MDataStream dataFile;

bool OpenDataFile(const char *pFileName)
{
 if(!pFileName) return false;
 return dataFile.OpenFile(pFileName);
}

MStream *GetDataFileStream(const char *pFileName)
{
 MStream *res=NULL;
 UINT dat;
 if(!pFileName||!FindInDataFile(pFileName,&dat)) return res;
 if(dataFile.SetRange(dat,4))
   {
    mDWORD val;
	if(dataFile.read(&val,4)==4)
	  {
	   if(dataFile.SetRange(dat+4,val)) res=&dataFile;
	  }
   }
 return res;
}

//-----------------------------------------------------------------------------

#ifndef COMPILE_SDL
int GetWaveID(void);
void SetWaveID(int val);
#endif

#define CFG_FILE "Berusky1.cfg"

bool ReadLine(MStream *strm, char *line, int maxline)
{
 if(!strm||strm->eof()||!line||maxline<=0) return false;
 int pos=0;
 line[0]=0;
 char ch;
 while(ch=strm->get(),!strm->eof()&&ch!=0x0a)
   {
    if(ch!=0x0d)
	  {
	   if(pos<maxline-1) {line[pos]=ch; pos++; line[pos]=0;}
	  }
   }
 return true;
}

bool LineStartsOn(const char *line, const char *txt)
{
 if(!line||!txt) return false;
 return 0==strnicmp(line,txt,strlen(txt));
}

void SetRepeatDelay(int val);
void SetRepeatInterval(int val);
int GetRepeatDelay(void);
int GetRepeatInterval(void);

void LoadSetup(void)
{
 MFileStream f;
 char txt[1024];
/*#ifdef COMPILE_LINUX
 strcpy(txt,"~/.");
#else*/
 strcpy(txt,GetExeDir());
//#endif
 strcat(txt,CFG_FILE);
 if(f.open(txt,foBinRead))
   {
    int have=0;
	bool have_SFX=false, have_Music=false,have_repeatDelay=false, have_repeatInterval=false;
	int SFXVolume=-1, MusicVolume=-1, repeatDelay=-1, repeatInterval=-1;
    while(have<4&&ReadLine(&f,txt,1024))
	  {
	   if(!have_repeatDelay&&LineStartsOn(txt,"repDelay="))
	     {
		  have++;
		  have_repeatDelay=true;
		  char *p=strchr(txt,'=');
		  if(p)
		    {
			 p++;
			 repeatDelay=atoi(p);
			}
		 }
	   if(!have_repeatDelay&&LineStartsOn(txt,"repInterval="))
	     {
		  have++;
		  have_repeatInterval=true;
		  char *p=strchr(txt,'=');
		  if(p)
		    {
			 p++;
			 repeatInterval=atoi(p);
			}
		 }
	   if(!have_SFX&&LineStartsOn(txt,"SFX="))
	     {
		  have++;
		  have_SFX=true;
		  char *p=strchr(txt,'=');
		  if(p)
		    {
			 p++;
			 SFXVolume=atoi(p);
			}
		 }
	   if(!have_Music&&LineStartsOn(txt,"Music="))
	     {
		  have++;
		  have_Music=true;
		  char *p=strchr(txt,'=');
		  if(p)
		    {
			 p++;
			 MusicVolume=atoi(p);
			}
		 }
	  }
	if(have_SFX)    SetSfxVolume(SFXVolume);
	if(have_Music)  SetMusicVolume(MusicVolume);
	if(have_repeatDelay) SetRepeatDelay(repeatDelay);
	if(have_repeatInterval) SetRepeatInterval(repeatInterval);
   }
}

void SaveSetup(void)
{
 MFileStream f;
 char txt[1024];
/*#ifdef COMPILE_LINUX
 strcpy(txt,"~/.");
#else*/
 strcpy(txt,GetExeDir());
//#endif
 strcat(txt,CFG_FILE);
 if(f.open(txt,foBinWrite))
   {
    sprintf(txt,"SFX=%d",GetSfxVolume()); f.write(txt,strlen(txt)); f.put(0x0d); f.put(0x0a);
    sprintf(txt,"Music=%d",GetMusicVolume()); f.write(txt,strlen(txt)); f.put(0x0d); f.put(0x0a);
    sprintf(txt,"repDelay=%d",GetRepeatDelay()); f.write(txt,strlen(txt)); f.put(0x0d); f.put(0x0a);
    sprintf(txt,"repInterval=%d",GetRepeatInterval()); f.write(txt,strlen(txt)); f.put(0x0d); f.put(0x0a);
   }
}
