#include "berdy.h"
#include "menu.h"


class MListChooser:public MMenuPolozka
{
 private:
   UINT lookfrom;
   vector<char *> files;
   UINT selIndex;
 public:
   MListChooser(UINT pID);
   MListChooser(UINT pID, int x, int y, int cx, int cy);
   virtual ~MListChooser();

   void EvalFiles(void);
   void SortFiles(UINT fromPos); //od ktere pozice se maji setridit
   void ClearFiles(void);

   bool CanScrollUp(void);
   bool CanScrollDown(void);
   bool DoScrollUp(void);
   bool DoScrollDown(void);

   const char *GetSelectedFile(void);
   const char *GetFileIndex(UINT idx);

   bool CanShowCursor;
   bool needDraw;

   void Draw(const myPOINT &menuOffset);
   bool KeyPress(UINT Key, UINT butts);
   void MouseLeft(int x, int y);
};


class MFileChoose:public MMenu
{
 private:
   char *bck2;
   bool done;
 public:
   MFileChoose();
   virtual ~MFileChoose();

   void SetBackground2(const char *File);
   const char *GetBackground2(void);

   void Draw(void);
   void RedrawFileList(void);
   void Cancel(void);

   const char *GetChosenFile(void);
   MListChooser *GetFilesList(void);
   MMenuEnterText *GetEnterText(void);

   void OnActivate(MResponder *resp);
   void OnAnimTimer(UINT timerID);
   void OnDeactivate(MResponder *resp);
   void KeyPress(UINT Key, UINT butts);
   void MouseLeft(int x, int y, UINT butts);
   void MenuClick(UINT menuID);
   void MenuItemChanged(void);
   
   void UpdateActivities(void);
};

//---------------------------------------------------------------------------------

MFileChoose::MFileChoose()
{
 bck2=NULL;
 done=false;
 needMainBackground=false;
}

MFileChoose::~MFileChoose()
{
 FreePtr(bck2);
}

void MFileChoose::SetBackground2(const char *File)
{
 FreePtr(bck2);
 if(File) bck2=strdup(File);
}

const char *MFileChoose::GetBackground2(void)
{
 return bck2;
}

MListChooser *MFileChoose::GetFilesList(void)
{
 MMenuPolozka *pol=GetByID(MENU_FC_FILELIST);
 if(!pol) return NULL;
 return (MListChooser*)pol;
}

MMenuEnterText *MFileChoose::GetEnterText(void)
{
 MMenuPolozka *pol=GetByID(MENU_FC_FILENAME);
 if(!pol) return NULL;
 return (MMenuEnterText*)pol;
}

void MFileChoose::MenuItemChanged(void)
{
 MMenu::MenuItemChanged();
 MListChooser *list=GetFilesList();
 if(list) list->CanShowCursor=list==GetSelected();
 MMenuEnterText *et=GetEnterText();
 if(et) 
   {
    et->CanShowCursor=et==GetSelected();
	if(!et->CanShowCursor) et->DrawTxt();
   }
 RedrawFileList();
}

void MFileChoose::Draw(void)
{
 MMenu::Draw();
 if(bck2) ScreenDrawBitmap(bck2,offset.x,offset.y);
 RedrawFileList();
}

void MFileChoose::UpdateActivities(void)
{
 MListChooser *list=GetFilesList();
 if(!list) return;
 MMenuEnterText *et=GetEnterText();

 MMenuPolozka *pol=GetByID(MENU_FC_SCROLLUP);
 if(pol) pol->Enabled=list->CanScrollUp();
 pol=GetByID(MENU_FC_SCROLLDOWN);
 if(pol) pol->Enabled=list->CanScrollDown();
 pol=GetByID(MENU_FC_OK);
 if(pol) pol->Enabled=et&&et->GetText()&&strlen(et->GetText());
}

void MFileChoose::RedrawFileList(void)
{
 if(GetMainResponder()!=this) return;

 MListChooser *list=GetFilesList();
 if(!list) return;
 
 list->Draw(offset);

 MMenuEnterText *et=GetEnterText();
 if(et) et->DrawTxt();

 UpdateActivities();

 SelectMenuItem((UINT)-2);
}

void MFileChoose::MouseLeft(int x, int y, UINT butts)
{
 MMenu::MouseLeft(x,y,butts);
 if(!(butts&MOUSE_BUTT_LEFT)) return;

 MListChooser *list=GetFilesList();
 if(!list) return;

 if(list->MouseOver(x-offset.x,y-offset.y))
   {
    list->MouseLeft(x-offset.x, y-offset.y);
	//if(list->GetSelectedFile()) chosenfile=strdup(list->GetSelectedFile());
    MMenuEnterText *et=GetEnterText();
    if(et) et->SetText(list->GetSelectedFile());
	RedrawFileList();
   }
 /*myRECT r={offset.x+27,offset.y+48,offset.x+266,offset.y+288};
 if(x>=r.left&&x<=r.right&&y>=r.top&&y<=r.bottom)
   {
	MListChooser *list=GetFilesList();
	if(!list) return;
    UINT idx=(y-r.top)/20;
	FreePtr(chosenfile);
	if(list->GetFileIndex(idx)) chosenfile=strdup(list->GetFileIndex(idx));
	RedrawFileList();
   }*/
}

void MFileChoose::OnActivate(MResponder *resp)
{
 done=false;

 MListChooser *list=GetFilesList();
 if(list) list->EvalFiles();

 MMenuEnterText *et=GetEnterText();
 if(et)	
   {
    //if(!list) 
    et->SetText(NULL);
//	 else
	  {
	   //et->SetText(list->GetSelectedFile());
	  }
   }

 MMenu::OnActivate(resp);

 if(et)
   {
    et->OnActivate(offset);
	if(list) 
	  {
	   et->SetText(list->GetSelectedFile());
	   MenuItemChanged();
	  }
   }
}

void MFileChoose::OnDeactivate(MResponder *resp)
{
 MListChooser *list=GetFilesList();
 if(list) list->ClearFiles();
 MMenuEnterText *et=GetEnterText();
 if(et) et->OnDeactivate();
 MMenu::OnDeactivate(resp);
}

void MFileChoose::OnAnimTimer(UINT timerID)
{
 MMenu::OnAnimTimer(timerID);
 MMenuEnterText *et=GetEnterText();
 if(et) et->OnAnimTimer(timerID);
}

const char *MFileChoose::GetChosenFile(void)
{
 MMenuEnterText *et=GetEnterText();
 if(et) return et->GetText();
 return NULL;
}

void MFileChoose::Cancel(void)
{
 MMenuEnterText *et=GetEnterText();
 if(et) et->SetText(NULL);
}

void MFileChoose::MenuClick(UINT menuID)
{
 MListChooser *list=GetFilesList();
 //MMenuEnterText *et=GetEnterText();
 bool redraw=false;
 switch(menuID)
   {
    case MENU_FC_ZRUSIT: 
	  {
	   Cancel();
	   break;
	  }
    case MENU_FC_SCROLLUP:if(list) {redraw=list->DoScrollUp();} break;
	case MENU_FC_SCROLLDOWN: if(list){redraw=list->DoScrollDown();} break;
	/*case MENU_FC_FILELIST:
	case MENU_FC_FILENAME:
	  {
	   if(et&&et->GetText()&&strlen(et->GetText()))
	     {
		  if(!done)
		    {
			 done=true;
		     ActivateMenuItem(MENU_FC_OK);
			}
		 }
	  }
	  break;*/
   }
 if(redraw) RedrawFileList();
}

void MFileChoose::KeyPress(UINT Key, UINT butts)
{
 MMenu::KeyPress(Key,butts);
 MListChooser *list=GetFilesList();
 MMenuEnterText *et=GetEnterText();
 if(list&&list->needDraw)
   {
    UpdateActivities();
    list->Draw(offset);
	if(et) 
	  {
	   et->SetText(list->GetSelectedFile());
	   et->DrawTxt();
	  }
   }
 switch(Key)
   {
    case KEY_RETURN:
	case KEY_SELECT:
	  {
	   if(GetMainResponder()==this&&(et==GetSelected()||list==GetSelected()))
	     {
		  UpdateActivities();
	      if(et&&et->GetText()&&strlen(et->GetText()))
	        {
		     if(!done)
		       {
			    done=true;
		        ActivateMenuItem(MENU_FC_OK);
			   }
		    }
		 }
	  }
	  break;
   }
}
//-----------------------------------------------------------------------------

MListChooser::MListChooser(UINT pID):MMenuPolozka(pID)
{
 lookfrom=0;
 selIndex=0;
 CanShowCursor=false;
 needDraw=false;
}

MListChooser::MListChooser(UINT pID, int x, int y, int cx, int cy):
   MMenuPolozka(pID,NULL,NULL,x,y,cx,cy,NULL)
{
 lookfrom=0;
 selIndex=0;
 CanShowCursor=false;
 needDraw=false;
}

MListChooser::~MListChooser()
{
 ClearFiles();
}

void MListChooser::EvalFiles(void)
{
 ClearFiles();
 SystemEvalFiles(&files,GetUserLevelDir(),"*.lv3");
 SortFiles(0);
}

void MListChooser::ClearFiles(void)
{
 while(!files.empty())
   {
    FreePtr(files[0]);
	files.erase(files.begin());
   }
 lookfrom=0;
 selIndex=0;
 needDraw=false;
}

void MListChooser::SortFiles(UINT fromPos)
{
 char *lookon=NULL;
 UINT lookid=fromPos;
 UINT i=fromPos;
 while(i<files.size())
   {
    if(!lookon) 
	  {
	   lookon=files[i];
	   lookid=i;
	  }
	 else
	  {
	   if(files[i])
	     {
		  if(strcmp(files[i],lookon)<0)
		    {
			 files[lookid]=files[i];
			 files[i]=lookon;
			 lookon=NULL;
			 i=fromPos-1;
			}
		   else
		    {
			 lookon=files[i];
			 lookid=i;
			}
		 }
	  }
    i++;
   }
}

const char *MListChooser::GetSelectedFile(void)
{
 if(selIndex<files.size()) return files[selIndex];
 return NULL;
}

#define MAX_FILE_ROWS 12

void MListChooser::Draw(const myPOINT &menuOffset)
{
 needDraw=false;
 //27,48 - 266,288
 myRECT cliprect={menuOffset.x+offset.x,menuOffset.y+offset.y,menuOffset.x+offset.x+size.cx,menuOffset.y+offset.y+size.cy};
 ScreenClearRect(&cliprect);

 int x=cliprect.left,y=cliprect.top;
 UINT i=lookfrom;
 while(i<files.size()&&i-lookfrom<MAX_FILE_ROWS)
   {
    /*if(chosenfile&&strcmp(chosenfile,files[i]))
	  {
	   myRECT r=cliprect;
	   r.top=y; r.bottom=y+20;
	   ScreenClearRect(&r,RGB(200,200,255),&r);
	  }*/
	if(CanShowCursor&&i==selIndex)
	  {
	   myRECT r;
	   r.left=cliprect.left; r.top=y; r.right=r.left+1; r.bottom=r.top+20;
	   ScreenClearRect(&r,100,100,130);
	   r.left=cliprect.right-1; r.right=cliprect.right;
	   ScreenClearRect(&r,100,100,130);
	   r.left=cliprect.left; r.right=cliprect.right; r.top=y; r.bottom=r.top+1;
	   ScreenClearRect(&r,100,100,130);
	   r.top=y+20-1; r.bottom=r.top+1;
	   ScreenClearRect(&r,100,100,130);
	  }
    PrintText(x,y,files[i],-1,NULL,NULL,NULL,NULL,&cliprect);
    y+=20;
    i++;
   }
 /*cliprect.top=menuOffset.y+305; cliprect.bottom=cliprect.top+20;
 ScreenClearRect(&cliprect);
 if(chosenfile)
   {
    PrintText(cliprect.left,cliprect.top,chosenfile,-1,NULL,NULL,NULL,NULL,&cliprect);
   }*/
}

const char *MListChooser::GetFileIndex(UINT idx)
{
 if(idx>=0&&idx<files.size()-lookfrom)
   {
    return (files[idx+lookfrom]);
   }
 return NULL;
}

bool MListChooser::CanScrollUp(void)
{
 return lookfrom>0;
}

bool MListChooser::CanScrollDown(void)
{
 return files.size()-lookfrom>MAX_FILE_ROWS;
}

bool MListChooser::DoScrollUp(void)
{
 if(!CanScrollUp()) return false;
 lookfrom--;
 return true;
}

bool MListChooser::DoScrollDown(void)
{
 if(!CanScrollDown()) return false;
 lookfrom++;
 return true;
}

bool MListChooser::KeyPress(UINT Key, UINT butts)
{
 bool res=false,haveKey=false;
 switch(Key)
   {
    case KEY_UP:
	  {
	   haveKey=true;
	   if(selIndex>0)
	     {
		  res=true;
		  selIndex--;
		  if(lookfrom>selIndex) lookfrom=selIndex;
		 }
	  }
	  break;
	case KEY_DOWN:
	  {
	   haveKey=true;
	   if(selIndex<files.size()-1)
	     {
		  res=true;
		  selIndex++;
		  if(selIndex>=lookfrom+MAX_FILE_ROWS) lookfrom=selIndex-MAX_FILE_ROWS+1;
		 }
	  }
	  break;
   }
 needDraw=res;
 return haveKey;
}

void MListChooser::MouseLeft(int x, int y)
{
 int idx=lookfrom+(y-offset.y)/20;
 if(idx>=0&&idx<files.size()) selIndex=idx;
}

//-----------------------------------------------------------------------------

MFileChoose FileChoose;

MResponder *GetFileChoose(void){return &FileChoose;}
const char *GetFileChooseResult(void){return FileChoose.GetChosenFile();}

void ActivateFileChoose(MResponder *pParent,bool pLoadDlg)
{
 FileChoose.Parent=pParent;
 if(pLoadDlg) FileChoose.SetBackground2("MENU_082");
  else FileChoose.SetBackground2("MENU_081");
 if(!FileChoose.Count())
   {
    mySIZE sz;
	ScreenGetSize(sz);
	FileChoose.offset.x=(sz.cx-427)/2;
	FileChoose.offset.y=(sz.cy-367)/2;
    FileChoose.SetBackground("MENU_080");
	FileChoose.Add(new MListChooser(MENU_FC_FILELIST,  27,48,266-27,288-48));
	FileChoose.Add(new MMenuEnterText(MENU_FC_FILENAME,27,305,266-27,20, 14,NULL));
    FileChoose.Add(new MMenuPolozka(MENU_FC_SCROLLUP,  "MENU_083","MENU_085", 281, 48, 20,20, NULL));
    FileChoose.Add(new MMenuPolozka(MENU_FC_SCROLLDOWN,"MENU_084","MENU_086", 281,269, 20,20, NULL));
    FileChoose.Add(new MMenuPolozka(MENU_FC_OK,        "MENU_087","MENU_089", 336, 43, 48,33, pParent));
    FileChoose.Add(new MMenuPolozka(MENU_FC_ZRUSIT,    "MENU_088","MENU_090", 323, 82, 78,39, pParent));
   }
 MMenuEnterText *et=FileChoose.GetEnterText();
 if(et) 
   {
    et->CanShowCursor=false;
	et->Enabled=!pLoadDlg;
   }
 MMenuPolozka *pol=FileChoose.GetByID(MENU_FC_OK);
 if(pol) pol->NextMenu=pParent;
 pol=FileChoose.GetByID(MENU_FC_ZRUSIT);
 if(pol) pol->NextMenu=pParent;

 SetMainResponder(&FileChoose);
}
