#include <SDL.h>
#include <stdio.h>
#include "MBitmaps.h"
#include "MTimers.h"
#include "berdy.h"
#include "menu.h"
#include "datafile.h"
#include "sound.h"
#include <vector>

using namespace std;

#ifdef COMPILE_LINUX
#include <ctype.h>
#include <sys/stat.h>
#else
#include <direct.h>
#endif

MTimers *Timers=NULL;
SDL_Surface *Screen=NULL;

#ifndef MAX_PATH
#define MAX_PATH 1024
#endif

char DataDir[MAX_PATH]="";
char UserLevelDir[MAX_PATH]="";

const char *GetPathDelimiter(void);

#define SDL_SUBSYSTEMS SDL_INIT_VIDEO|SDL_INIT_TIMER|SDL_INIT_AUDIO

void RepairPathDelimiter(char *buff)
{
 char newdelim=GetPathDelimiter()[0];
 if(!buff||newdelim=='\\') return;
 for(int i=0;buff[i]!=0;i++)
   {
    if(buff[i]=='\\') buff[i]=newdelim;
   }
}

const char *GetApplicationTitle(void);

#ifndef DeletePtr
  #define DeletePtr(x) {if(x) {delete (x);} (x)=NULL;}
#endif

#ifndef MAX_PATH
#define MAX_PATH 1024
#endif

void ToLower(char *buff)
{
 if(!buff) return;
 for(int i=0;buff[i]!=0;i++)
   {
    buff[i]=tolower(buff[i]);
   }
}

bool LoadBitmap(MBitmap *bmp, const char *pfile, bool pCreateMask, bool nowriteerror)
{
 if(!bmp||!pfile) return false;
 char buff[MAX_PATH];
 char *mpfile=strdup(pfile);
 const char *file=mpfile;
 if(!file) file=pfile;
  else
   {
    ToLower(mpfile);
   }
 strcpy(buff,GetDataDir());
 strcat(buff,"Graphics");
 strcat(buff,GetPathDelimiter());
 strcat(buff,file);
 strcat(buff,".bmp");
 RepairPathDelimiter(buff);
 bool res=(bmp->LoadFromFile(buff,pCreateMask));
 if(!res)
   {
    strcpy(buff,"Data\\");
    strcat(buff,"Graphics\\");
    strcat(buff,file);
    strcat(buff,".BMP");
    MStream *strm=GetDataFileStream(buff);
	if(strm) res=bmp->LoadFromStream(strm,pCreateMask);
   }
 if(!res&&!nowriteerror)
   {
    strcpy(buff,GetDataDir());
    strcat(buff,"Graphics\\");
    strcat(buff,file);
    strcat(buff,".bmp");
    RepairPathDelimiter(buff);
    WriteToLog("LoadBitmap '%s' failed",buff);
   }
 if(mpfile) free(mpfile);
 return res;
}

void ScreenDrawBitmap(const char *file, const myLPRECT where, const myLPRECT clipRegion)
{
 MBitmap bmp;
 if(LoadBitmap(&bmp,file))
   {
    ScreenDrawBitmap(&bmp,where,clipRegion);
   }
}

void ScreenDrawBitmap(const char *file, int x, int y, const myLPRECT clipRegion)
{
 MBitmap bmp;
 if(LoadBitmap(&bmp,file))
   {
    ScreenDrawBitmap(&bmp,x,y,clipRegion);
   }
}

//#define KeyStateCheck(key) ((GetKeyState(key)&0x80)!=0)
//#define KeyStateToggle(key) ((GetKeyState(key)&0x01)!=0)

//-----------------------------------------------------------------------------------
const char *GetExeDir(void);

const char *GetDataDir(void)
{
 if(DataDir[0]==0)
   {
    strcpy(DataDir,GetExeDir());
	strcat(DataDir,"Data");
	strcat(DataDir,GetPathDelimiter());
   }
 return DataDir;
}

const char *GetUserLevelDir(void)
{
 if(UserLevelDir[0]==0)
   {
    strcpy(UserLevelDir,GetExeDir());
	strcat(UserLevelDir,"Levels");
	strcat(UserLevelDir,GetPathDelimiter());
   }
 return UserLevelDir;
}

//LRESULT CALLBACK WndProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam);

void WriteSDLError(void)
{
 WriteToLog("SDLError: %s",SDL_GetError());
}

void KillMyWindow(void)
{
 SDL_Quit();
}

int gRepeatDelay=SDL_DEFAULT_REPEAT_INTERVAL*2, gRepeatInterval=SDL_DEFAULT_REPEAT_INTERVAL;

void SetRepeatDelay(int val)
{
 if(val>0){
	gRepeatDelay=val;
	SDL_EnableKeyRepeat(gRepeatDelay,gRepeatInterval);
 }
}

void SetRepeatInterval(int val)
{
 if(val>0){
	gRepeatInterval=val;
	SDL_EnableKeyRepeat(gRepeatDelay,gRepeatInterval);
 }
}

int GetRepeatDelay(void){return gRepeatDelay;}
int GetRepeatInterval(void){return gRepeatInterval;}

bool CreateMyWindow(const char *pTitle, mySIZE winSize, myPOINT winPos, int nCmdShow)
{
 if(SDL_Init(SDL_SUBSYSTEMS) == -1)
   {
    WriteSDLError();
   }
 if(SDL_INIT_VIDEO&SDL_WasInit(SDL_INIT_EVERYTHING))
   {
	MBitmap icon;
	LoadBitmap(&icon,"ICON",false,false);
	SDL_Surface *s=icon.GetSurface();
	if(s)
	  {
	   SDL_SetColorKey(s,SDL_SRCCOLORKEY,SDL_MapRGB(s->format,0,128,128));
       SDL_WM_SetIcon(s, NULL);
	  }
    Screen=SDL_SetVideoMode(winSize.cx, winSize.cy, 0, SDL_SWSURFACE/*/SDL_HWSURFACE/**/);
	if(Screen==NULL) WriteSDLError();
   }
 if(Screen==NULL)
   {
    return false;
   }
 /*if(Screen->format){
	SetDefaultBitmapsBitCount(Screen->format->BitsPerPixel);
 }else{
	SetDefaultBitmapsBitCount(8);
 }*/
 if(!(SDL_INIT_TIMER&SDL_WasInit(SDL_INIT_EVERYTHING)))
   {
    WriteSDLError();
	WriteToLog("Timer initialize error. Exit application.");
	return false;
   }

 SDL_WM_SetCaption(pTitle, NULL);
 SDL_EnableKeyRepeat(gRepeatDelay,gRepeatInterval);
 return true;
}

MResponder *Responder=NULL;

void OnTimerEventCallback(myHWND wnd, UINT id)
{
 switch(id)
   {
    case ANIM_TIMER1:
	case ANIM_TIMER2:
	  {
	   if(Responder) Responder->OnAnimTimer(id);
	  }
	  break;
   }
}

//--------------------------------------------------------------------------

void SetAnimTimerEnabled(UINT timerID, bool pVal)
{
 if(Timers) Timers->SetTimerEnabled(timerID,pVal);
}

bool GetAnimTimerEnabled(UINT timerID)
{
 bool res=false;
 if(Timers) Timers->GetTimerEnabled(timerID,res);
 return res;
}

void SetAnimTimerElapsed(UINT timerID, UINT pVal)
{
 if(Timers) Timers->SetTimerElapsed(timerID,pVal);
}

UINT GetAnimTimerElapsed(UINT timerID)
{
 UINT res=0;
 if(Timers) Timers->GetTimerElapsed(timerID,res);
 return res;
}

//--------------------------------------------------------------------------

void SetMainResponder(MResponder *resp)
{
 MResponder *old=Responder;
 if(Responder) Responder->OnDeactivate(resp);
 if(Responder!=old) return; //nekdo zmenil responder v OnDeactivate
 Responder=resp;
 if(Responder) Responder->OnActivate(old);
}
MResponder *GetMainResponder(void){return Responder;}

bool done=false;
bool ProcessMessage(SDL_Event *msg);
bool LoadDataFile(const char *pFileName);

int RunApplication(void)
{
 DataDir[0]=0;
 UserLevelDir[0]=0;

 mySIZE winSize={640,480};
 myPOINT winPos={100,100}; //nepouziva se

 if(!LoadDataFile(NULL))
   {
    WriteToLog("InitGame failed");
	return 1;
   }

  
 if(!CreateMyWindow(GetApplicationTitle(),winSize,winPos,0)) 
   {
    KillMyWindow();
    return 2;
   }
 

 Timers=GetMainTimers();
 Timers->SetOnTimerEvent(OnTimerEventCallback);
 Timers->AddTimer(ANIM_TIMER1,150);
 Timers->SetTimerEnabled(ANIM_TIMER1,false);
 
 Timers->AddTimer(ANIM_TIMER2,5000);
 Timers->SetTimerEnabled(ANIM_TIMER2,false);
 
 SetOnMenuActivate(OnMenuActivate);
 if(!InitGame())
   {
    WriteToLog("InitGame failed");
    DeletePtr(Timers);
    KillMyWindow();
	FreeSoundAndMusic();
	return 3;
   }

 Timers->SetTimerEnabled(ANIM_TIMER1,true);
 Timers->SetTimerEnabled(ANIM_TIMER2,true);

 SDL_Event event;
 done=Screen==NULL;
 while(!done)
   {
    if(!done) done=!SDL_WaitEvent(NULL);
	while(!done&&SDL_PollEvent(&event))
	  {
	   done=!ProcessMessage(&event);
	  }
	if(done) Screen=NULL;
	ScreenPaint();
   }
 Timers->Clear();
 ClearIntro();
 FreeSoundAndMusic();
 KillMyWindow();
 return 0;
}

UINT DecodeMouseKeys(Uint8 prm, Uint8 except=0)
{
 UINT res=MOUSE_BUTT_NONE;
 Uint8 state=SDL_GetMouseState(NULL, NULL);
 if((SDL_GetModState()&KMOD_CTRL)&&except!=MOUSE_BUTT_CTRL) res=res | MOUSE_BUTT_CTRL;
 if((state&SDL_BUTTON(SDL_BUTTON_LEFT))&&except!=MOUSE_BUTT_LEFT) res=res | MOUSE_BUTT_LEFT;
 if((state&SDL_BUTTON(SDL_BUTTON_MIDDLE))&&except!=MOUSE_BUTT_MIDDLE) res=res | MOUSE_BUTT_MIDDLE;
 if((state&SDL_BUTTON(SDL_BUTTON_RIGHT))&&except!=MOUSE_BUTT_RIGHT) res=res | MOUSE_BUTT_RIGHT;
 if((SDL_GetModState()&KMOD_SHIFT)&&except!=MOUSE_BUTT_SHIFT) res=res | MOUSE_BUTT_SHIFT;
 return res;
}

MResponder *GetEndScreen(void);

bool ProcessMessage(SDL_Event *msg)
{
 if(!msg) return false;
 switch(msg->type)
   {
    case SDL_USEREVENT:
	  {
	   switch(msg->user.code)
	     {
		  case MY_SDL_TIMER_EVENT:
		    {
			 if(Timers) Timers->WorkWithTimer(NULL,int(msg->user.data1));
			}
			break;
		 }
	  }
	  break;
    case SDL_QUIT:
      {
       if(Timers) Timers->Clear();
       done=true;
       return false;
      }
	/*case WM_TIMER:
	  {
	   if(Timers!=NULL)
	     {
		  Timers->WorkWithTimer(hWnd,wParam);
	      return 0;
		 }
	  }
	  break;*/
	case SDL_KEYDOWN:
	  {
	   UINT val=0;
	   if((SDL_GetModState()&KMOD_SHIFT)||(SDL_GetModState()&KMOD_CAPS)) val=val|MOUSE_BUTT_SHIFT;
	   if((SDL_GetModState()&KMOD_CTRL)) val=val|MOUSE_BUTT_CTRL;
	   if(Responder!=NULL&&(msg->key.keysym.sym!=SDLK_TAB||!(SDL_GetModState()&KMOD_ALT))) Responder->KeyPress(msg->key.keysym.sym,val);
	   if(SDL_GetModState()&KMOD_ALT&&(!(SDL_GetModState()&KMOD_CTRL))&&msg->key.keysym.sym==SDLK_F4)
	     {
		  //return false;
		  if(GetMainResponder()==GetEndScreen()) return false;
		   else SetMainResponder(GetEndScreen());
		 }
	  }
	  break;
	case SDL_MOUSEMOTION:   if(Responder!=NULL) Responder->MouseMove(msg->motion.x,msg->motion.y, DecodeMouseKeys(msg->motion.state));   break;
	case SDL_MOUSEBUTTONUP:
	  if(Responder!=NULL)
	    {
	     switch(msg->button.button)
	       {
		    case SDL_BUTTON_LEFT:  Responder->MouseLeft(msg->button.x,msg->button.y,DecodeMouseKeys(msg->button.button,MOUSE_BUTT_LEFT)); break;
		    case SDL_BUTTON_MIDDLE:Responder->MouseLeft(msg->button.x,msg->button.y,DecodeMouseKeys(msg->button.button,MOUSE_BUTT_MIDDLE)); break;
		    case SDL_BUTTON_RIGHT: Responder->MouseLeft(msg->button.x,msg->button.y,DecodeMouseKeys(msg->button.button,MOUSE_BUTT_RIGHT)); break;
		   } 
		}
	  break;
	case SDL_MOUSEBUTTONDOWN:
	  if(Responder!=NULL)
	    {
	     switch(msg->button.button)
	       {
		    case SDL_BUTTON_LEFT:  Responder->MouseLeft(msg->button.x,msg->button.y,DecodeMouseKeys(msg->button.button)); break;
		    case SDL_BUTTON_MIDDLE:Responder->MouseLeft(msg->button.x,msg->button.y,DecodeMouseKeys(msg->button.button)); break;
		    case SDL_BUTTON_RIGHT: Responder->MouseLeft(msg->button.x,msg->button.y,DecodeMouseKeys(msg->button.button)); break;
		   } 
		}
	  break;
   }
 return true;
}

//---------------------------------------------------------------------------------

vector<SDL_Rect> screenUpdateRects;

void ScreenDrawBitmap(MBitmap *bmp, const myLPRECT where, const myLPRECT clipRegion)
{
 if(!Screen||!bmp||!bmp->GetSurface()) return;
 SDL_Rect w, *pw=NULL;
 if(where)
   {
	pw=RECTtoSDLRect(where,w);
   }
 if(clipRegion)
   {
    SDL_Rect r;
	SDL_SetClipRect(Screen,RECTtoSDLRect(clipRegion,r));
   }
 SDL_BlitSurface(bmp->GetSurface(),NULL,Screen,pw);
 if(clipRegion)
   {
    SDL_SetClipRect(Screen,NULL);
   }
 if(pw)
   {
	//SDL_UpdateRect(Screen,pw->x,pw->y,pw->w,pw->h);
   }
  else
   {
    //SDL_UpdateRect(Screen,0,0,0,0);
	screenUpdateRects.clear();
	w.x=0; w.y=0;w.w=0; w.h=0;
   }
 screenUpdateRects.push_back(w);
}

void ScreenDrawBitmap(MBitmap *bmp, int x, int y, const myLPRECT clipRegion)
{
 if(!bmp) return;
 myRECT r;
 mySIZE sz;
 bmp->GetSize(sz);
 r.left=x; r.top=y; r.right=x+sz.cx; r.bottom=y+sz.cy;
 ScreenDrawBitmap(bmp,&r,clipRegion);
}

bool ScreenGetBitmap(MBitmap *bmp, const myLPRECT where)
{
 if(!Screen||!bmp) return false;
 //bmp->Clear();
 SDL_Surface *sfc=bmp->GetSurface();
 if(!sfc)
   {
    //return false;
    mySIZE sz;
    if(where){ sz.cx=where->right-where->left; sz.cy=where->bottom-where->top;}
     else {ScreenGetSize(sz);}
    bmp->Create(sz.cx,sz.cy);
    sfc=bmp->GetSurface();
   }
 if(!sfc) return false;
 SDL_Rect r,*pr=NULL;
 if(where)
   {
	pr=RECTtoSDLRect(where,r);
   }
 /*if(pr)
   {
    SDL_UpdateRect(Screen,r.x,r.y,r.w,r.h);
   }
  else
   {
    SDL_UpdateRect(Screen,0,0,0,0);
   }*/
 if(SDL_MUSTLOCK(Screen))
   {
    SDL_LockSurface(Screen);
   }
 bool res=0==SDL_BlitSurface(Screen,pr,sfc,NULL);
 if(SDL_MUSTLOCK(Screen))
   {
    SDL_UnlockSurface(Screen);
   }
 return res;
}

void ScreenGetBitmap(MBitmap *bmp,int x, int y)
{
 if(bmp)
   {
    mySIZE sz;
	bmp->GetSize(sz);
	myRECT r={x,y,x+sz.cx,y+sz.cy};
	ScreenGetBitmap(bmp,&r);
   }
}

void QuitGame(void)
{
 Screen=NULL;
 SDL_Quit(); //PostQuitMessage(0);
}

void ScreenClearRect(myLPRECT rect, unsigned char r, unsigned char g, unsigned char b, const myLPRECT clipRegion)
{
 //Screen.ClearRect(rect,col,clipRegion);
 if(!Screen) return;
 SDL_Rect w, *pw=NULL;
 if(rect)
   {
    //w.x=where.left; w.y=where.top; w.w=where.right-where.left; w.h=where.bottom-where.top;
	pw=RECTtoSDLRect(rect,w);
   }
 if(clipRegion)
   {
    SDL_Rect r;
	SDL_SetClipRect(Screen,RECTtoSDLRect(clipRegion,r));
   }
 SDL_FillRect(Screen,pw,SDL_MapRGB(Screen->format,r,g,b));
 if(clipRegion)
   {
  	SDL_SetClipRect(Screen,NULL);
   }

 
 if(pw)
   {
    //SDL_UpdateRect(Screen,pw->x, pw->y,pw->w, pw->h);
   }
  else
   {
    //SDL_UpdateRect(Screen,0,0,0,0);
	w.x=0; w.y=0; w.w=0; w.h=0;
   }
 screenUpdateRects.push_back(w);
}

//---------------------------------------------------------------------------------
/*
void MyDrawText(HDC dc,myLPRECT r,const char *pText)
{
 LOGFONT Font;
 Font.lfHeight=12;
 Font.lfWeight=0;
 Font.lfWidth=0;
 Font.lfEscapement=0;
 Font.lfOrientation=0;
 Font.lfItalic=FALSE;
 Font.lfUnderline=FALSE;
 Font.lfStrikeOut=FALSE;
 Font.lfCharSet=EASTEUROPE_CHARSET;
 Font.lfOutPrecision=OUT_CHARACTER_PRECIS; //OUT_DEFAULT_PRECIS
 Font.lfClipPrecision=CLIP_CHARACTER_PRECIS; //CLIP_DEFAULT_PRECIS
 Font.lfQuality=DEFAULT_QUALITY;
 Font.lfPitchAndFamily=DEFAULT_PITCH;
 strcpy(Font.lfFaceName,"Arial");
 HGDIOBJ oldobj=SelectObject(dc,CreateFontIndirect(&Font));
 int oldbk=SetBkMode(dc,OPAQUE);//TRANSPARENT);
 DrawText(dc,pText,-1,r,0);
 SetBkMode(dc,oldbk);
 DeleteObject(SelectObject(dc,oldobj));
}
*/

void ScreenGetSize(mySIZE &sz)
{
 if(!Screen) return;
 sz.cx=Screen->w; sz.cy=Screen->h;
}

void ScreenPaint(void)
{
 if(!Screen) {
	screenUpdateRects.clear();
	return;
 }
 //SDL_UpdateRect(Screen,0,0,0,0);
 SDL_Rect r;
 while(!screenUpdateRects.empty()){
	r=screenUpdateRects[0];
	screenUpdateRects.erase(screenUpdateRects.begin());
	SDL_UpdateRect(Screen,r.x,r.y,r.w,r.h);
	if(!r.x&&!r.y&&!r.w&&!r.h) break;
 }
 screenUpdateRects.clear();
}
//-------------------------------------------------------------------------------------

UINT GetInputableKey(UINT Key)
{
 #define VK_0 SDLK_0
 #define VK_9 SDLK_9
 #define VK_A SDLK_a
 #define VK_Z SDLK_z

 UINT res=0;
 if(Key==SDLK_SPACE) res=' ';
  else
   {
    if(Key>=SDLK_KP0&&Key<=SDLK_KP9) res='0'+Key-SDLK_KP0;
	 else
	  {
	   if(Key>=VK_0&&Key<=VK_9) res='0'+Key-VK_0;
	    else
		 {
		  if(Key>=VK_A&&Key<=VK_Z) res='A'+Key-VK_A;
		 }
	  }
   }
 return res;
}

void DrawRect(MBitmap *bmp, const myLPRECT rect, unsigned char r, unsigned char g, unsigned char b)
{
 if(!bmp||!bmp->GetSurface()) return;
 SDL_Rect rr,*pr;
 pr=RECTtoSDLRect(rect,rr);
 SDL_FillRect(bmp->GetSurface(),pr,SDL_MapRGB(bmp->GetSurface()->format,r,g,b));
}

//----------------------------------------------------------------------------

void ForceDirectories(const char *pDir)
{
 if(!pDir) return;
 char *dir=strdup(pDir),*p,*t;
 if(dir)
   {
    t=dir;
    while(p=strchr(t,GetPathDelimiter()[0]),p!=NULL)
	  {
	   p[0]=0;
	   mkdir(dir
		#ifdef COMPILE_LINUX
		  , S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IWGRP|S_IXGRP|S_IROTH|S_IWOTH|S_IXOTH
		#endif
		);
	   p[0]=GetPathDelimiter()[0];
	   t=p+1;
	  }
    free(dir);
   }
}

//-----------------------------------------------------------------------------

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

void hraj_hudbu_levelu(void)
{
 #define LEVEL_HUDEB_OK  26
 #define LEVEL_HUDEB_LOW 22

 static int ok_hudby[60] = {0,1,2,4,3,5,6,7,8,9,10,11,12,13,14,15,
                            16,17,18,19,20,21,22,23,24,25,0,0,0,0};
 /*static int low_hudby[60] = {0,1,2,4,3,5,6,7,8,11,12,14,16,17,18,
                             19,20,21,22,23,24,25,0,0,0,0};*/
 static int hrali[60] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
 static int *p_hudby;
 int i,j = 0,
     skladeb;


 stop_modul();

 p_hudby = ok_hudby;
 skladeb = LEVEL_HUDEB_OK;

 i = rand()%skladeb;

 while(hrali[i] != 0) {
   if(++i >= skladeb)
     i = 0;
   j++;
   if(j >= skladeb) {
     i = 0;
     for(j = 0; j < skladeb; j++)
        hrali[j] = 0;
   }
 }

 hrali[i] = 1;
 hraj_modul(p_hudby[i],TRUE);
}

void hraj_hudbu_menu(void)
{
 /* 1,11,13,14,16,19,28
 */
 #define MENU_HUDEB 5

 static int hudby[30] = {4,5,6,7,8};
 static int hrali[30] = {0,0,0,0,0};
 int i,j = 0;

 stop_modul();

 i = rand()%MENU_HUDEB;
 while(hrali[i] != 0) {
   if(++i >= MENU_HUDEB)
     i = 0;
   j++;
   if(j >= MENU_HUDEB) {
     i = 0;
     for(j = 0; j < MENU_HUDEB; j++)
        hrali[j] = 0;
   }
 }

 hrali[i] = 1;
 hraj_modul(hudby[i],TRUE);
}

void hraj_hudbu_credits(void)
{
 /* 8,17,27
 */
 #define CREDIT_HUDEB 4

 static int hudby[30] = {0,1,2,3};
 static int hrali[30] = {0,0,0,0};
 int i,j = 0;

 stop_modul();

 i = rand()%CREDIT_HUDEB;
 while(hrali[i] != 0) {
   if(++i >= CREDIT_HUDEB)
     i = 0;
   j++;
   if(j >= CREDIT_HUDEB) {
     i = 0;
     for(j = 0; j < CREDIT_HUDEB; j++)
        hrali[j] = 0;
   }
 }

 hrali[i] = 1;
 hraj_modul(hudby[i],TRUE);
}

void hraj_hudbu_intro(void)
{
 #define INTRO_MOD 26
 hraj_modul(INTRO_MOD,FALSE);
}

void hraj_hudbu_outro(int cislo)
{
 hraj_modul(cislo,TRUE);
}

bool IsKeyDown(UINT key)
{
 int keys=-1;
 SDL_PumpEvents();
 Uint8 *buff=SDL_GetKeyState(&keys);
 if(buff&&key<keys)
   {
    return buff[key]!=0;
   }
 return false;
}

void SendKeyDown(UINT key)
{
 SDL_Event e;
 e.type=SDL_KEYDOWN;
 e.key.keysym.sym=(SDLKey)key;
 SDL_PushEvent(&e);
}
