#include "menu.h"
#include "MBitmaps.h"
#include "sound.h"

#ifdef COMPILE_LINUX
int stricmp(const char *str1, const char *str2);
#endif

//-----------------------------------------------------------------------------

MOnMenuActivate OnMenuActivateFunc=NULL;
void SetOnMenuActivate(MOnMenuActivate func){OnMenuActivateFunc=func;}
MOnMenuActivate GetOnMenuItemActivate(void){return OnMenuActivateFunc;}

//-----------------------------------------------------------------------------

void DrawMainMenuBackground(void);

//-----------------------------------------------------------------------------

MMenuPolozka::MMenuPolozka(UINT pID)
{
 ID=pID;
 selected=false;
 offset.x=0; offset.y=0;
 size.cx=-1; size.cy=-1;
 btmpNoSelect=NULL;
 btmpSelect=NULL;
 NextMenu=NULL;
 Slider=NULL;
 Enabled=true;
 selImgNm=NULL;
 noSelImgNm=NULL;
}

MMenuPolozka::MMenuPolozka(UINT pID, const char *selImg, const char *noSelImg, int x, int y, int cx, int cy,MResponder *nextMenu)
{
 ID=pID;
 selected=false;
 btmpNoSelect=NULL;
 btmpSelect=NULL;
 SetOffset(x,y);
 SetSize(cx,cy);
 NextMenu=nextMenu;
 Slider=NULL;
 Enabled=true;
 selImgNm=NULL;
 noSelImgNm=NULL;
 SetSelImg(selImg);
 SetNoSelImg(noSelImg);
 /*LoadBitmap(btmpNoSelect,noSelImg);
 LoadBitmap(btmpSelect,selImg);*/
}

MMenuPolozka::~MMenuPolozka()
{
 if(btmpNoSelect) delete btmpNoSelect; btmpNoSelect=NULL;
 if(btmpSelect) delete btmpSelect; btmpSelect=NULL;
 NextMenu=NULL;
 SetSelImg(NULL);
 SetNoSelImg(NULL);
}

void MMenuPolozka::SetSelImg(const char *selImg)
{
 if(selImgNm) free(selImgNm); selImgNm=NULL;
 if(selImg) selImgNm=strdup(selImg);
}

const char *MMenuPolozka::GetSelImg(void){return selImgNm;}

void MMenuPolozka::SetNoSelImg(const char *noSelImg)
{
 if(noSelImgNm) free(noSelImgNm); noSelImgNm=NULL;
 if(noSelImg) noSelImgNm=strdup(noSelImg);
}

const char *MMenuPolozka::GetNoSelImg(void){return noSelImgNm;}

void MMenuPolozka::AllocCache(void)
{
 if(!btmpNoSelect) btmpNoSelect=new MBitmap;
 if(!btmpSelect) btmpSelect=new MBitmap;
 LoadBitmap(btmpNoSelect,GetNoSelImg());
 LoadBitmap(btmpSelect,GetSelImg());
}

void MMenuPolozka::FreeCache(void)
{
 if(btmpNoSelect) delete btmpNoSelect; btmpNoSelect=NULL;
 if(btmpSelect) delete btmpSelect; btmpSelect=NULL;
}

bool MMenuPolozka::MouseOver(int x, int y)
{
 if(!Enabled) return false;
 return ((x>=offset.x&&x<=offset.x+size.cx)&&
         (y>=offset.y&&y<=offset.y+size.cy)
        );
}

void MMenuPolozka::Draw(const myPOINT &menuOffset)
{
 MBitmap *btmp;
 if(selected) btmp=btmpSelect;
  else btmp=btmpNoSelect;
 ScreenDrawBitmap(btmp,menuOffset.x+offset.x, menuOffset.y+offset.y);
}

void MMenuPolozka::SetOffset(int x, int y){offset.x=x; offset.y=y;}
void MMenuPolozka::SetSize(int cx, int cy){size.cx=cx; size.cy=cy;}

bool MMenuPolozka::KeyPress(UINT Key, UINT butts)
{
 return false;
}
//-------------------------------------------------------------------
MMenuSlider::MMenuSlider(UINT pID):MMenuPolozka(pID)
{
 btmpSliderBack=NULL;
 horzSlider=true;
 slideSize=0;
 slidePos=0;
 Owner=NULL;
 OnMove=NULL;
}

MMenuSlider::MMenuSlider(UINT pID, const char *selImg, const char *noSelImg, int x, int y, int cx, int cy, int sz, bool phorz, MResponder *nextMenu):
   MMenuPolozka(pID,selImg,noSelImg, x, y, cx,cy, nextMenu)
{
 btmpSliderBack=NULL;
 horzSlider=phorz;
 slidePos=0;
 Owner=NULL;
 OnMove=NULL;
 slideSize=sz;
}

MMenuSlider::~MMenuSlider()
{
 if(btmpSliderBack) delete btmpSliderBack; btmpSliderBack=NULL;
}

void MMenuSlider::InitSliderBack(MBitmap *backbmp)
{
 if(btmpSliderBack&&backbmp)
   {
    btmpSliderBack->Create(size.cx+(horzSlider?slideSize:0),size.cy+(horzSlider?0:slideSize));
	//MMenuSlider *s=new MMenuSlider(MENU_OPTIONS_ZVUK2  ,"MENU_058","MENU_058", 25,85, 11,27, 200,true,OptionsMenu.background);
    myRECT to={0,0,size.cx+(horzSlider?slideSize:0),size.cy+(horzSlider?0:slideSize)};
	myRECT vyr={offset.x,offset.y,offset.x+to.right,offset.y+to.bottom};

	MyHDC dc=btmpSliderBack->BeginPaint();
	backbmp->Draw(dc,&to,&vyr);
	btmpSliderBack->EndPaint();
   }
}

void MMenuSlider::AllocCache(void)
{
 MMenuPolozka::AllocCache();
 if(!btmpSliderBack) btmpSliderBack=new MBitmap;
 if(btmpNoSelect) btmpNoSelect->UpdateMask(TRANSPARENT_COLOR);
 if(btmpSelect) btmpSelect->UpdateMask(TRANSPARENT_COLOR);
}

void MMenuSlider::FreeCache(void)
{
 if(btmpSliderBack) delete btmpSliderBack; btmpSliderBack=NULL;
 MMenuPolozka::FreeCache();
}

void MMenuSlider::CallOnMove(void)
{
 if(OnMove)OnMove(this);
 if(ID==MENU_OPTIONS_ZVUK2) PlaySfx(SFX_MENU_SKOK,SFX_PRIORITY3);
}

bool MMenuSlider::MouseOver(int x, int y)
{
 if(!Enabled) return false;
 return ((x>=offset.x&&x<=offset.x+size.cx+(horzSlider?slideSize:0))&&
         (y>=offset.y&&y<=offset.y+size.cy+(horzSlider?0:slideSize))
        );
}

void MMenuSlider::Draw(const myPOINT &menuOffset)
{
 if(Owner) Owner->Draw(menuOffset);
 if(btmpSliderBack)
   {
    ScreenDrawBitmap(btmpSliderBack,menuOffset.x+offset.x, menuOffset.y+offset.y);
   }
 myPOINT pt;
 pt.x=menuOffset.x; pt.y=menuOffset.y;
 if(horzSlider) pt.x+=slidePos;
  else pt.y+=slidePos;
 MMenuPolozka::Draw(pt);
}

//-------------------------------------------------------------------

MMenu::MMenu()
{
 backImgNm=NULL;
 background=NULL;
 needMainBackground=true;
 actSlider=NULL;
 backImgNm=NULL;
 CanFreeBackground=true;
 Clear();
}

MMenu::~MMenu()
{
 Clear();
 if(background!=NULL) delete background; background=NULL;
}

void MMenu::Clear(void)
{
 offset.x=0; offset.y=0;
 lastchoice=0;
 defchoice=-2;
 ParentMenu=NULL;
 ClearPolozky();
 if(backImgNm) free(backImgNm); backImgNm=NULL;
}

void MMenu::ClearPolozky(void)
{
 while(!polozky.empty())
   {
    if(polozky[0]!=NULL) delete polozky[0];
	polozky.erase(polozky.begin());
   }
}

void MMenu::SetBackground(const char *backImg)
{
 if(backImgNm) free(backImgNm); backImgNm=NULL;
 if(backImg) backImgNm=strdup(backImg);
}

const char *MMenu::GetBackground(void){return backImgNm;}

void MMenu::AllocCache(void)
{
 if(!background) background=new MBitmap;
 if(background&&GetBackground()) LoadBitmap(background,GetBackground());
 MMenuPolozka *p;
 MMenuSlider *sld;
 for(unsigned int i=0;i<polozky.size();i++)
   {
    p=polozky[i];
	if(p)
	  {
	   p->AllocCache();
	   sld=dynamic_cast<MMenuSlider *>(p);
	   if(sld)
	     {
		  sld->InitSliderBack(background);
		 }
	  }
   }
}

void MMenu::FreeCache(void)
{
 MMenuPolozka *p;
 for(unsigned int i=0;i<polozky.size();i++)
   {
    p=polozky[i];
	if(p)
	  {
	   p->FreeCache();
	  }
   }
 if(CanFreeBackground)
   {
    if(background) delete background; background=NULL;
   }
}

void MMenu::SetOffset(int x, int y){offset.x=x; offset.y=y;}

UINT MMenu::Add(MMenuPolozka *polozka)
{
 if(!polozka) return (UINT)-1;
 polozky.push_back(polozka);
 return polozky.size()-1;
}

void MMenu::Del(UINT pos)
{
 if(polozky.size()>pos)
   {
    if(polozky[pos]!=NULL) delete polozky[pos];
	polozky.erase(polozky.begin()+pos);
   }
}

MMenuPolozka *MMenu::Get(UINT pos)
{
 if(polozky.size()>pos)
   {
    return polozky[pos];
   }
 return NULL;
}

MMenuPolozka *MMenu::GetByID(UINT polID)
{
 MMenuPolozka *p;
 for(int i=0;i<polozky.size();i++)
   {
    p=polozky[i];
	if(p&&p->ID==polID) return p;
   }
 return NULL;
}

UINT MMenu::Count(void)
{
 return polozky.size();
}

void MMenu::DrawSelected(void)
{
 MMenuPolozka *p;
 for(UINT i=0;i<polozky.size();i++)
   {
    p=polozky[i];
	if(p&&p->selected)
	  {
	   p->Draw(offset);
	  }
   }
}

void MMenu::Draw(void)
{
 ScreenDrawBitmap(background,offset.x, offset.y);
 MMenuPolozka *p;
 for(UINT i=0;i<polozky.size();i++)
   {
    p=polozky[i];
	if(p)
	  {
	   p->Draw(offset);
	  }
   }
}

void MMenu::SelectByMouse(int x, int y)
{
 UINT ll=lastchoice;

 x=x-offset.x; y=y-offset.y;
 bool any_hit=false;
 MMenuPolozka *p;
 for(UINT i=0;i<polozky.size();i++)
   {
    p=polozky[i];
	if(p)
	  {
	   bool ls=p->selected;
	   p->selected=p->MouseOver(x,y);
	   any_hit=any_hit||p->selected;
	   if(ls!=p->selected)
	     {
		  MMenuSlider *sldr=dynamic_cast<MMenuSlider *>(p);
		  if(sldr)
		    {
			 if(sldr->Owner) 
			   {
			    sldr->Owner->selected=true;
				sldr->Owner->Draw(offset);
				p->selected=false;
				lastchoice=i-1;
			   }
			}
		  p->Draw(offset);
		 }
	   if(p->selected) lastchoice=i;
	  }
   }
 if(!any_hit) SelectMenuItem(lastchoice);
  else 
   {
    if(ll!=lastchoice) MenuItemChanged();
   }
}

void MMenu::SelectMenuItem(UINT pos, bool dir_forward)
{
 UINT ll=lastchoice;

 if(pos==UINT(-2)) pos=lastchoice;
 if(pos==UINT(-1)) pos=polozky.size()-1;
 if(pos>=polozky.size()) pos=0;
 bool first=true,have=false;
 UINT from=pos;
 MMenuPolozka *p;
 UINT i=pos;
 while((first||i!=from)&&!have&&i<polozky.size())
   {
    first=false;
	p=polozky[i];
	if(!(dynamic_cast<MMenuSlider*>(p)))
	  {
	   if(p->Enabled) 
	     {
		  pos=i;
		  have=true;
		 }
	  }
    if(dir_forward) i++;
	 else i--;
	if(i==UINT(-1)) i=polozky.size()-1;
	if(i>=polozky.size()) i=0;
   }
 if(!have) pos=UINT(-1);
 for(i=0;i<polozky.size();i++)
   {
    p=polozky[i];
	if(p)
	  {
	   bool ls=p->selected;
	   p->selected=i==pos;
	   if(ls!=p->selected)
	     {
		  p->Draw(offset);
		 }
	   if(p->selected) lastchoice=i;
	  }
   }
 if(ll!=lastchoice) MenuItemChanged();
}

void MMenu::ActivateMenuItem(UINT itemID)
{
 if(itemID!=UINT(-1))
   {
    MMenuPolozka *pol=GetByID(itemID);
	if(pol!=NULL&&pol->Enabled)
	  {
	   for(UINT i=0;i<polozky.size();i++)
	     {
		  if(polozky[i]==pol) 
		    {
			 lastchoice=i;
			 pol->selected=true;
			 PlaySfx(SFX_MENU_SKOK,SFX_PRIORITY3);
			 break;
			}
	     }
	  }
   }
 if(lastchoice<0||lastchoice>=polozky.size()) return;
 MMenuPolozka *p=polozky[lastchoice];
 if(p)
   {
    if(!p->selected)
	  {
	   p->selected=true;
	   p->Draw(offset);
	   PlaySfx(SFX_MENU_SKOK,SFX_PRIORITY3);
	  }
	 else
	  {
	   PlaySfx(SFX_MENU_KLIK,SFX_PRIORITY3);
	   if(OnMenuActivateFunc) OnMenuActivate(p->ID);
	   if(p->NextMenu) SetMainResponder(p->NextMenu);
	  }
   }
}

void MMenu::MenuItemChanged(void)
{
 PlaySfx(SFX_MENU_SKOK,SFX_PRIORITY3);
}

MMenuPolozka *MMenu::GetSelected(void)
{
 MMenuPolozka *res=NULL;
 if(lastchoice>=0&&lastchoice<polozky.size()) res=polozky[lastchoice];
 return res;
}

MMenuPolozka *MMenu::GetOverMouse(int x,int y)
{
 MMenuPolozka *p;
 for(unsigned int i=0;i<polozky.size();i++)
   {
    p=polozky[i];
	if(p&&p->MouseOver(x-offset.x, y-offset.y)) return p;
   }
 return NULL;
}

void MMenu::Cancel(void){}

void MMenu::KeyPress(UINT Key,UINT butts)
{
 MMenuPolozka *pol=GetSelected();
 if(pol&&pol->KeyPress(Key,butts)) return;

 switch(Key)
   {
    case KEY_ESCAPE:
	  if(ParentMenu||Parent) Cancel();
	  if(ParentMenu!=NULL) SetMainResponder(ParentMenu);
	   else
	    {
		 if(Parent!=NULL) SetMainResponder(Parent);
		}
	  break;
	case KEY_RETURN:
	case KEY_SELECT:
	case KEY_SPACE:
	  ActivateMenuItem();
	  break;
	case KEY_UP:
	case KEY_LEFT:
	  {
	   MMenuPolozka *pol=GetSelected();
	   if(!pol||!pol->Slider||(pol->Slider->horzSlider&&Key==KEY_UP)||(!pol->Slider->horzSlider&&Key==KEY_LEFT))
	     {
	      SelectMenuItem(lastchoice-1,false);
		 }
		else
		 {
		  if(pol->Slider->slidePos>0) 
		    {
			 pol->Slider->slidePos--;
			 pol->Slider->Draw(offset);
			 pol->Slider->CallOnMove();
			}
		 }
	  }
	  break;
	case KEY_DOWN:
	case KEY_RIGHT:
	case KEY_TAB:
	  {
	   MMenuPolozka *pol=GetSelected();
	   if(!pol||!pol->Slider||(pol->Slider->horzSlider&&(Key==KEY_DOWN||Key==KEY_TAB))||(!pol->Slider->horzSlider&&(Key==KEY_RIGHT||Key==KEY_TAB)))
	     {
	      SelectMenuItem(lastchoice+1);
		 }
		else
		 {
		  if(pol->Slider->slidePos<pol->Slider->slideSize) 
		    {
			 pol->Slider->slidePos++;
			 pol->Slider->Draw(offset);
			 pol->Slider->CallOnMove();
			}
		 }
	  }
	  break;
   }
}

void MMenu::MouseMove(int x, int y, UINT butts)
{
 if((butts&MOUSE_BUTT_LEFT)&&actSlider)
   {
    bool zmena=false;
	if(actSlider->horzSlider)
	  {
	   if(x-offset.x<actSlider->offset.x+(actSlider->size.cx/2))
	     {
		  zmena=actSlider->slidePos!=0;
		  actSlider->slidePos=0;
		 }
		else
		 {
		  if(x-offset.x>actSlider->offset.x+actSlider->slideSize-(actSlider->size.cx/2))
		    {
			 zmena=actSlider->slidePos!=actSlider->slideSize;
			 actSlider->slidePos=actSlider->slideSize;
			}
		   else
		    {
			 zmena=actSlider->slidePos!=x-offset.x-actSlider->offset.x;
			 actSlider->slidePos=x-offset.x-actSlider->offset.x-(actSlider->size.cx/2);
			}
		 }
	  }
	 else
	  {
	   if(y-offset.y<actSlider->offset.y+(actSlider->size.cy/2))
	     {
		  zmena=actSlider->slidePos!=0;
		  actSlider->slidePos=0;
		 }
		else
		 {
		  if(y-offset.y>actSlider->offset.y+actSlider->slideSize-(actSlider->size.cy/2))
		    {
			 zmena=actSlider->slidePos!=actSlider->slideSize;
			 actSlider->slidePos=actSlider->slideSize;
			}
		   else
		    {
			 zmena=actSlider->slidePos!=y-offset.y-actSlider->offset.y;
			 actSlider->slidePos=y-offset.y-actSlider->offset.y-(actSlider->size.cy/2);
			}
		 }
	  }
	if(zmena) 
	  {
	   actSlider->Draw(offset);
	   actSlider->CallOnMove();
	  }
   }
  else
   {
    SelectByMouse(x,y);
   }
}

void MMenu::MouseLeft(int x, int y, UINT butts)
{
 if(!(butts&MOUSE_BUTT_LEFT)) 
   {
    actSlider=NULL;
    return;
   }
 bool have=false;
 if(lastchoice>=0&&lastchoice<polozky.size())
   {
    MMenuPolozka *p=polozky[lastchoice];
	if(p&&p->MouseOver(x-offset.x,y-offset.y)) 
	  {
	   have=true;
	   ActivateMenuItem();
	  }
   }
 if(!have)
   {
    MMenuPolozka *p=GetOverMouse(x,y);
	actSlider=dynamic_cast<MMenuSlider *>(p);
	if(actSlider) MouseMove(x,y,butts);
   }
}

void MMenu::MouseMiddle(int x, int y, UINT butts)
{
}

void MMenu::MouseRight(int x, int y, UINT butts)
{
}

void MMenu::OnAnimTimer(UINT timerID)
{
}

void MMenu::OnActivate(MResponder *resp)
{
 if(resp==ParentMenu||resp==Parent)
   {
    if(defchoice!=UINT(-1)) lastchoice=defchoice;
    if(resp)
	  {
	   AllocCache();
	  }
   }
 if(needMainBackground) DrawMainMenuBackground();
 SelectMenuItem(defchoice);
 Draw();
}

void MMenu::OnDeactivate(MResponder *resp)
{
 if((resp==ParentMenu||resp==Parent)&&resp)
   {
    FreeCache();
   }
}

//--------------------------------------------------------------------------

class MMenuSOcima:public MMenu
{
 private:
 public:
   MMenuSOcima();
   virtual ~MMenuSOcima();
   
   void OnActivate(MResponder *resp);
   void OnDeactivate(MResponder *resp);
   void OnAnimTimer(UINT timerID);
};

MMenuSOcima::MMenuSOcima(){}
MMenuSOcima::~MMenuSOcima(){}

void MMenuSOcima::OnActivate(MResponder *resp)
{
 if(!GetAnimTimerEnabled(ANIM_TIMER1) || GetAnimTimerElapsed(ANIM_TIMER1)!=OCI_ANIM_DELAY)
   {
    SetAnimTimerElapsed(ANIM_TIMER1,OCI_ANIM_DELAY);
	SetAnimTimerEnabled(ANIM_TIMER1,true);
   }
 MMenu::OnActivate(resp);
}

void MMenuSOcima::OnDeactivate(MResponder *resp)
{
 MMenu::OnDeactivate(resp);
 if(!(dynamic_cast<MMenuSOcima*>(resp)))
   {
    SetAnimTimerEnabled(ANIM_TIMER1,false);
   }
}

void MMenuSOcima::OnAnimTimer(UINT timerID)
{
 switch(timerID)
   {
    case ANIM_TIMER1: 
	  DoOciAnim();
	  break;
   }
 MMenu::OnAnimTimer(timerID);
}


//--------------------------------------------------------------------------

/*class MEnterText:public MResponder
{
 private:
 public:
};*/

#define MAX_PASS_LEN 5

class MMenuEnterPass:public MMenuSOcima
{
 private:
   MMenuEnterText *GetEnterText(void);
 public:
   MMenuEnterPass();
   virtual ~MMenuEnterPass();

   void KeyPress(UINT Key,UINT butts);
   
   void OnActivate(MResponder *resp);
   void OnDeactivate(MResponder *resp);
   void OnAnimTimer(UINT timerID);
};

//--------------------------------------------------------------------------
void SetLevelAndEpisodeByPass(const char *pass);
//--------------------------------------------------------------------------

MMenuEnterText::MMenuEnterText(UINT pID):MMenuPolozka(pID)
{
 txtbackground=NULL;
 Initialize(0,0,0,0,0,NULL);
}

MMenuEnterText::MMenuEnterText(UINT pID, int x, int y, int cx, int cy, int pmaxlen, const char *txt):
   MMenuPolozka(pID,NULL,NULL,x,y,cx,cy,NULL)
{
 txtbackground=NULL;
 Initialize(x,y,cx,cy,pmaxlen,txt);
}

MMenuEnterText::~MMenuEnterText()
{
 if(txtbackground) delete txtbackground;
 txtbackground=NULL;
}

void MMenuEnterText::Initialize(int x, int y, int cx, int cy, int pmaxlen, const char *txt)
{
 curpos.x=x; curpos.y=y;
 where.left=x; where.top=y; where.right=x+cx; where.bottom=y+cy;
 buff[0]=0;
 znaku=0;
 showed=true;
 if(pmaxlen>=MAX_ENTER_TEXT) pmaxlen=MAX_ENTER_TEXT-1;
 maxlen=pmaxlen;
 if(txt&&strlen(txt)<maxlen) 
   {
    strcpy(buff,txt);
	znaku=strlen(txt);
   }
 if(!txtbackground) txtbackground=new MBitmap;
 CanShowCursor=true;
}

void MMenuEnterText::DrawTxt(void)
{
 ScreenDrawCursor(curpos.x, curpos.y,false);
 ScreenDrawBitmap(txtbackground,where.left, where.top);
 PrintText(where.left, where.top,buff,-1,NULL,NULL,NULL,&curpos);
 if(CanShowCursor) ScreenDrawCursor(curpos.x, curpos.y,showed);
}

bool MMenuEnterText::KeyPress(UINT Key,UINT butts)
{
 if(!Enabled) return false;
 if(Key==KEY_BACKSPACE)
   {
    if(znaku>0) znaku--;
	buff[znaku]=0;
	DrawTxt();
   }
  else
   {
    UINT key2=GetInputableKey(Key);
    if((key2>=(UINT)'a'&&key2<=(UINT)'z')||
       (key2>=(UINT)'A'&&key2<=(UINT)'Z')||
	   (key2>=(UINT)'0'&&key2<=(UINT)'9'))
      {
       if(znaku<maxlen)
	     {
	      buff[znaku]=(char)key2;
	      znaku++;
	      buff[znaku]=0;
		  DrawTxt();
		  ScreenPaint();
	     }
      }
     else
      {
       return false;
      }
   }
 return true;
}

void MMenuEnterText::OnAnimTimer(UINT timerID)
{
 if(!Enabled) return;
 switch(timerID)
   {
    case ANIM_TIMER2:
	  showed=!showed;
	  if(CanShowCursor) ScreenDrawCursor(curpos.x, curpos.y,showed);
	  break;
   }
}

void MMenuEnterText::OnActivate(const myPOINT &menuOffset)
{
 where.left=offset.x+menuOffset.x; where.top=offset.y+menuOffset.y;
 where.right=where.left+size.cx; where.bottom=where.top+size.cy;

 ScreenGetBitmap(txtbackground,&where);
 znaku=0;
 buff[0]=0;
 showed=true;
 curpos.x=where.left; curpos.y=where.top;
 if(Enabled)
   {
    SetAnimTimerElapsed(ANIM_TIMER2,400);
    SetAnimTimerEnabled(ANIM_TIMER2,true);
    if(CanShowCursor) ScreenDrawCursor(curpos.x, curpos.y,showed);
   }
}

void MMenuEnterText::OnDeactivate(void)
{
 SetAnimTimerEnabled(ANIM_TIMER2,false);
 if(txtbackground) txtbackground->Clear();
}

void MMenuEnterText::SetText(const char *txt)
{
 buff[0]=0; znaku=0;
 if(!txt||strlen(txt)>=maxlen) return;
 strcpy(buff,txt);
 znaku=strlen(buff);
}

const char *MMenuEnterText::GetText(void){return buff;}

//--------------------------------------------------------------------------
MMenuEnterPass::MMenuEnterPass()
{
}

MMenuEnterPass::~MMenuEnterPass(){}

MMenuEnterText *MMenuEnterPass::GetEnterText(void)
{
 MMenuPolozka *pol=GetByID(MENU_PASSWORD_PASS);
 if(pol) return (MMenuEnterText *)pol;
 return NULL;
}

void MMenuEnterPass::KeyPress(UINT Key,UINT butts)
{
 bool processed=false;
 MMenuEnterText *et=GetEnterText();
 if(et) processed=et->KeyPress(Key,butts);
 if(!processed) MMenu::KeyPress(Key,butts);
  else
   {
    if(et->GetText()&&strlen(et->GetText())==MAX_PASS_LEN)
	  {
	   SetLevelAndEpisodeByPass(et->GetText());
	  }
   }
}
   
void MMenuEnterPass::OnActivate(MResponder *resp)
{
 MMenuSOcima::OnActivate(resp);
 MMenuEnterText *et=GetEnterText();
 if(et)
   {
	et->SetText(NULL);
    et->OnActivate(offset);
   }
}

void MMenuEnterPass::OnDeactivate(MResponder *resp)
{
 MMenuSOcima::OnDeactivate(resp);
 MMenuEnterText *et=GetEnterText();
 if(et) et->OnDeactivate();
}

void MMenuEnterPass::OnAnimTimer(UINT timerID)
{
 MMenuSOcima::OnAnimTimer(timerID);
 MMenuEnterText *et=GetEnterText();
 if(et) et->OnAnimTimer(timerID);
}

//--------------------------------------------------------------------------

class MWizardPagePrvek
{
 public:
   MWizardPagePrvek();
   ~MWizardPagePrvek();
 
   int x,y; //pozice
   char *text;
   bool isBitmap; //jestli je v text ulozeno jmeno bitmapy
   UINT themeID;
   int frameX;
   int frameY;
   int frameSzX; 
   int frameSzY;

   bool blockFormatted;
   UINT major,minor; //kdyz je text a neni inicializovano
};

class MWizardPage
{
 private:
 public:
   MWizardPage();
   ~MWizardPage();

   vector<MWizardPagePrvek *> prvky;
   void Clear(void);
   void Draw(const myPOINT &offset);
};

#define WIZID_OVLADANI  1
#define WIZID_PRAVIDLA  2

class MMenuHelpWizard:public MMenu
{
 private:
   vector<MWizardPage *>pages;
   UINT actPage;
 public:
   MMenuHelpWizard();
   virtual ~MMenuHelpWizard();

   void AllocCache(void);
   void FreeCache(void);

   UINT AddPage(void);
   void DelPage(UINT pos);
   void ClearPages(void);
   void NextPage(void);
   void PrevPage(void);
   bool HaveNextPage(void);
   bool HavePrevPage(void);
   bool ActivatePage(UINT pos); //kdyz se nepodari, tak se postavi na prvni stranku, je-li

   //pracuje s aktivni strankou
   void AddPageText(int x, int y, UINT major, UINT minor, bool blockFormatted=false);
   void AddPageBitmap(int x, int y, UINT themeID, const char *bitmapname, int frameX, int frameY, int frameSzX=20, int frameSzY=20);

   void ActivateMenuItem(UINT itemID);
   void Update(bool forward); //obnovi itemy ve wizardu (enabled stavy sipek)
   void Draw(void);

   UINT wizID; //ID wizzardu, inicializuje se podle nej v OnActivate

   void OnActivate(MResponder *resp);
   void OnAnimTimer(UINT timerID);
   void OnDeactivate(MResponder *resp);
};

MWizardPagePrvek::MWizardPagePrvek()
{
 x=0;y=0; text=NULL; isBitmap=false; blockFormatted=false; major=0; minor=0;
 themeID=(UINT)-1; frameX=0; frameY=0; frameSzX=0; frameSzY=0;
}
MWizardPagePrvek::~MWizardPagePrvek()
{
 if(text) free(text); text=NULL;
}
 
MWizardPage::MWizardPage(){}
MWizardPage::~MWizardPage(){ Clear();}

void MWizardPage::Clear(void)
{
 while(!prvky.empty())
   {
    MWizardPagePrvek *p=prvky[0];
	prvky.erase(prvky.begin());
	if(p) delete p;
   }
}

void MWizardPage::Draw(const myPOINT &offset)
{
 MAnimationBitmap abmp;
 MBitmap bmp;
 MWizardPagePrvek *p;
 for(int i=0;i<prvky.size();i++)
   {
    p=prvky[i];
	if(p)
	  {
	   if(p->isBitmap) 
	     {
		  if(LoadThemeBitmap(p->themeID,&(abmp.Bitmap),p->text,false))
		    {
			 abmp.frameOffset.x=0;
			 abmp.frameOffset.y=0;
			 abmp.frameSize.cx=p->frameSzX;
			 abmp.frameSize.cy=p->frameSzY;
			 if(bmp.Create(p->frameSzX,p->frameSzY))
			   {
			    abmp.DrawFrame(&bmp,p->frameX,NULL,p->frameY);
				bmp.UpdateMask(TRANSPARENT_COLOR);
				ScreenDrawBitmap(&bmp,p->x+offset.x,p->y+offset.y);
			   }
			}
		 }
	    else
		 {
		  if(p->blockFormatted) PrintTextTwoColBlock(p->x+offset.x,p->y+offset.y,p->text,". ",10,true);
		   else PrintTextAlign(p->x+offset.x,p->y+offset.y,p->text);
		 }
	  }
   }
}

MMenuHelpWizard::MMenuHelpWizard()
{
 actPage=(UINT)-1;
}

MMenuHelpWizard::~MMenuHelpWizard()
{
 ClearPages();
}

void MMenuHelpWizard::AllocCache(void)
{
 MMenu::AllocCache();
 MWizardPage *p;
 MWizardPagePrvek *pr;
 for(unsigned int i=0;i<pages.size();i++)
   {
    p=pages[i];
	if(p)
	  {
	   for(unsigned int j=0;j<p->prvky.size();j++)
	     {
		  pr=p->prvky[j];
		  if(pr&&!pr->isBitmap&&!pr->text)
		    {
			 pr->text=LoadText("Help",pr->major,pr->minor);
			}
		 }
	  }
   }
}

void MMenuHelpWizard::FreeCache(void)
{
 MMenu::FreeCache();
 MWizardPage *p;
 MWizardPagePrvek *pr;
 for(unsigned int i=0;i<pages.size();i++)
   {
    p=pages[i];
	if(p)
	  {
	   for(unsigned int j=0;j<p->prvky.size();j++)
	     {
		  pr=p->prvky[j];
		  if(pr&&!pr->isBitmap&&pr->text)
		    {
			 free(pr->text);
			 pr->text=NULL;
			}
		 }
	  }
   }
}

UINT MMenuHelpWizard::AddPage(void)
{
 MWizardPage *page=new MWizardPage();
 if(page)
   {
    pages.push_back(page);
	actPage=pages.size()-1;
	return actPage;
   }
 return (UINT)-1;
}

void MMenuHelpWizard::DelPage(UINT pos)
{
 if(pages.size()>pos)
   {
    MWizardPage *p=pages[pos];
	if(pos==actPage) actPage=(UINT)-1;
	pages.erase(pages.begin()+pos);
	if(p) delete p;
   }
}

void MMenuHelpWizard::ClearPages(void)
{
 MWizardPage *p;
 while(!pages.empty())
   {
    p=pages[0];
	pages.erase(pages.begin());
	if(p) delete p;
   }
 actPage=(UINT)-1;
}

void MMenuHelpWizard::NextPage(void){if(HaveNextPage()) actPage++;}
void MMenuHelpWizard::PrevPage(void){if(HavePrevPage()) actPage--;}
bool MMenuHelpWizard::HaveNextPage(void){return pages.size()>actPage+1;}
bool MMenuHelpWizard::HavePrevPage(void){return actPage-1<pages.size();}

bool MMenuHelpWizard::ActivatePage(UINT pos)
{
 if(pos<pages.size()) actPage=pos;
  else actPage=0;
 Update(true);
 return actPage!=0||pos==0;
}

void MMenuHelpWizard::AddPageText(int x, int y, UINT major, UINT minor, bool blockFormatted)
{
 if(actPage>=pages.size()) return;
 MWizardPage *pg=pages[actPage];
 if(!pg) return;
 MWizardPagePrvek *p=new MWizardPagePrvek;
 if(!p) return;
 p->blockFormatted=blockFormatted;
 p->isBitmap=false;
 //p->text=txtloader.ReadText(major,minor);
 p->x=x;
 p->y=y;
 p->major=major;
 p->minor=minor;
 pg->prvky.push_back(p);
}

void MMenuHelpWizard::AddPageBitmap(int x, int y, UINT themeID, const char *bitmapname, int frameX, int frameY, int frameSzX, int frameSzY)
{
 if(!bitmapname||actPage>=pages.size()) return;
 MWizardPage *pg=pages[actPage];
 if(!pg) return;
 MWizardPagePrvek *p=new MWizardPagePrvek;
 if(!p) return;
 p->isBitmap=true;
 p->text=strdup(bitmapname);
 p->themeID=themeID;
 p->frameX=frameX;
 p->frameY=frameY;
 p->frameSzX=frameSzX; p->frameSzY=frameSzY;
 p->x=x;
 p->y=y;
 pg->prvky.push_back(p);
}

void MMenuHelpWizard::ActivateMenuItem(UINT itemID)
{
 MMenuPolozka *pol=GetSelected();
 if(!pol||(pol->ID!=MENU_WIZ_PREV&&pol->ID!=MENU_WIZ_NEXT)) MMenu::ActivateMenuItem(itemID);
  else
   {
    switch(pol->ID)
	  {
	   case MENU_WIZ_PREV:
	     if(HavePrevPage()) PrevPage();
	     break;
	   case MENU_WIZ_NEXT:
	     if(HaveNextPage()) NextPage();
	     break;
	  }
	Update(pol->ID==MENU_WIZ_PREV);
	Draw();
   }
}

void MMenuHelpWizard::Update(bool forward)
{
 MMenuPolozka *p;
 p=GetByID(MENU_WIZ_PREV); if(p) p->Enabled=HavePrevPage();
 p=GetByID(MENU_WIZ_NEXT); if(p) p->Enabled=HaveNextPage();
 SelectMenuItem((UINT)-2,forward);
}

void MMenuHelpWizard::Draw(void)
{
 MMenu::Draw();
 if(actPage<pages.size())
   {
    MWizardPage *p=pages[actPage];
	if(p) p->Draw(offset);
   }
}

void MMenuHelpWizard::OnActivate(MResponder *resp)
{
 ClearPages();
 ClearPolozky();
 switch(wizID)
   {
    case WIZID_OVLADANI:
	  {
	   SetBackground("MENU_056");
	   AddPage(); 
	   AddPageText(20,100,1,1,true);
	   AddPage(); 
	   AddPageText(20,100,1,2,true);
	  }
	  break;
	case WIZID_PRAVIDLA:
	  {
	   int x,y,i;
	   SetBackground("MENU_057");
	   AddPage();

	   AddPageText(20,95,2,1);
	   AddPageText(20,140,2,2);
	   AddPageText(20,280,2,3);

	   x=20; y=310;
	   for(i=1;i<8;i++) 
	     {
          AddPageBitmap(x,y,(UINT)-1,"BOXES",i,0);
          x+=30;
         }
	   AddPageText(20,350,2,4);
	   x=20; y=380;
	   for(i=0;i<5;i++) 
	     {
		  AddPageBitmap(x,y,(UINT)-1,"TNT",i,0);
		  x+=30;
		 }
	   
	   AddPage();
	   AddPageText(20,95,2,5);

	   AddPageText(20,140,2,6);
	   AddPageBitmap(20,165,(UINT)-1,"KEYS",0,0);

 	   AddPageText(20,200,2,7);
	   x=20; y=230;
	   for(i=0; i<3; i ++) 
	     {
	      AddPageBitmap(x,y,(UINT)-1,"EXIT",0,i);
		  x+=30;
		 }
	   AddPageBitmap(x,y,1,"EXIT",0,0); x+=30;
	   AddPageBitmap(x,y,2,"EXIT",0,0); x+=30;

	   AddPageText(20,270,2,8);
	   x=20; y=300;
	   AddPageBitmap(x,y,1,"STONE",0,0); x+=30;
	   AddPageBitmap(x,y,2,"STONE",0,0); x+=30;

	   AddPageText(20,340,2,9);
	   x=20; y=370;
	   AddPageBitmap(x,y,1,"TOOL",0,0); x+=30;
	   AddPageBitmap(x,y,2,"TOOL",0,0); x+=30;
	   
	   AddPage();
	   AddPageText(20,95,2,10);

	   AddPageText(20,140,2,11);
	   x=20; y=210;
	   for(i=1; i<6; i++) 
	     {
		  AddPageBitmap(x,y,1,"KEYS",i,0);
		  x+=60;
		 }
	   x=50;
	   for(i=1; i<6; i++) 
	     {
		  AddPageBitmap(x,y,2,"KEYS",i,0);
		  x+=60;
		 }

	   AddPageText(20,250,2,12);
	   x=20; y=300;
	   AddPageBitmap(x,y,1,"DOORKEY",0,0); x+=20;
	   AddPageBitmap(x,y,1,"DOOR",0,0); x+=20;
	   AddPageBitmap(x,y,1,"DOORKEY",0,2); x+=20;
	   x+=10;
	   AddPageBitmap(x,y,2,"DOORKEY",0,0); x+=20;
	   AddPageBitmap(x,y,2,"DOOR",0,0); x+=20;
	   AddPageBitmap(x,y,2,"DOORKEY",0,2); x+=20;
	   
	   AddPageText(20,340,2,13);
	   x=20; y=410;
	   AddPageBitmap(x,y,1,"DOORCOLS",0,2); x+=20;
	   AddPageBitmap(x,y,1,"DOOR",0,0); x+=20;
	   AddPageBitmap(x,y,1,"DOORCOLS",0,3); x+=20;
	   x+=10;
	   AddPageBitmap(x,y,2,"DOORCOLS",0,2); x+=20;
	   AddPageBitmap(x,y,2,"DOOR",0,0); x+=20;
	   AddPageBitmap(x,y,2,"DOORCOLS",0,3); x+=20;

	   AddPage();
	   AddPageText(20,95,2,14); 

	   AddPageText(20,140,2,15);
	   x=20; y=220;
	   AddPageBitmap(x,y,1,"DOOROT",0,0); x+=20;
	   AddPageBitmap(x,y,1,"DOOR",1,2); x+=20;
	   AddPageBitmap(x,y,1,"DOOROT",0,1); x+=20;
	   x+=10;
	   AddPageBitmap(x,y,2,"DOOROT",0,0); x+=20;
	   AddPageBitmap(x,y,2,"DOOR",1,2); x+=20;
	   AddPageBitmap(x,y,2,"DOOROT",0,1); x+=20;
	   x=20; y=260;
	   AddPageBitmap(x,y,1,"DOOROT",1,0); x+=20;
	   AddPageBitmap(x,y,1,"DOOR",0,2); x+=20;
	   AddPageBitmap(x,y,1,"DOOROT",1,1); x+=20;
	   x+=10;
	   AddPageBitmap(x,y,2,"DOOROT",1,0); x+=20;
	   AddPageBitmap(x,y,2,"DOOR",0,2); x+=20;
	   AddPageBitmap(x,y,2,"DOOROT",1,1); x+=20;

	   AddPageText(15,320,2,16);
	  }
	  break;
   }
 if(!pages.empty())
   {
	Add(new MMenuPolozka(MENU_WIZ_PREV,"MENU_069","MENU_068", 231,445, 17,34));
	Add(new MMenuPolozka(MENU_WIZ_ZPET,"MENU_067","MENU_066", 320-65,445, 131,33,ParentMenu));
	defchoice=Add(new MMenuPolozka(MENU_WIZ_NEXT,"MENU_071","MENU_070", 320+7+66,445, 17,34));
   }
 MMenu::OnActivate(resp);
   {
    MMenuPolozka *p;
	for(UINT i=0;i<Count();i++)
	  {
	   p=Get(i);
	   if(p)
	     {
		  if(p->btmpNoSelect) 
		    {
			 //if(i==2) p->btmpNoSelect->TransparentColorPos.x=10;
			 p->btmpNoSelect->UpdateMask(TRANSPARENT_COLOR);
			}
		  if(p->btmpSelect) 
		    {
			 //if(i==2) p->btmpSelect->TransparentColorPos.x=10;
			 p->btmpSelect->UpdateMask(TRANSPARENT_COLOR);
			}
		 }
	  }
   }
 ScreenClearRect();
 SelectMenuItem(defchoice);
 ActivatePage(0);
 Draw();
}

void MMenuHelpWizard::OnAnimTimer(UINT timerID)
{
}

void MMenuHelpWizard::OnDeactivate(MResponder *resp)
{
 MMenu::OnDeactivate(resp);
 ClearPages();
 ClearPolozky();
 ScreenClearRect();
}

//--------------------------------------------------------------------------
class MMainMenu:public MMenuSOcima
{
 private:
 public:
   MMainMenu();
   virtual ~MMainMenu();

   void OnActivate(MResponder *resp);
   void MenuClick(UINT menuID);
};

MMainMenu::MMainMenu():MMenuSOcima(){}
MMainMenu::~MMainMenu(){}
void MMainMenu::OnActivate(MResponder *resp)
{
 bool skip=false;
 if(resp==GetFileChoose())
   {
    const char *file=GetFileChooseResult();
	if(file&&file[0]!=0) skip=RunUserLevel(file);
   }
 if(!skip)
   {
    ScreenClearRect();
    MMenuSOcima::OnActivate(resp);
    Draw();
	ScreenPaint();
    hraj_hudbu_menu();
   }
}

void MMainMenu::MenuClick(UINT menuID)
{
 if(menuID==MENU_MAIN_OPEN) ActivateFileChoose(this,true);
}

class MInGameMenuShowHide:public MResponder
{
 private:
   int i,to,step;
   MBitmap p1,p2,p3,p4,p5;
 public:
   MInGameMenuShowHide();
   virtual ~MInGameMenuShowHide();

   bool Show; //jestli zobrazuje nebo skryva
   MInGameMenu *menu; //koho kreslim, jen pujceny

   void KeyPress(UINT Key,UINT butts);
   void MouseMove(int x, int y, UINT butts);
   void MouseLeft(int x, int y, UINT butts);
   void MouseMiddle(int x, int y, UINT butts);
   void MouseRight(int x, int y, UINT butts);

   void OnAnimTimer(UINT timerID);
   void OnActivate(MResponder *resp);
   void OnDeactivate(MResponder *resp);
};

MInGameMenuShowHide::MInGameMenuShowHide(){Show=true;menu=NULL;}
MInGameMenuShowHide::~MInGameMenuShowHide(){menu=NULL;}

void MInGameMenuShowHide::KeyPress(UINT Key,UINT butts){}
void MInGameMenuShowHide::MouseMove(int x, int y, UINT butts){}
void MInGameMenuShowHide::MouseLeft(int x, int y, UINT butts){}
void MInGameMenuShowHide::MouseMiddle(int x, int y, UINT butts){}
void MInGameMenuShowHide::MouseRight(int x, int y, UINT butts){}

void MInGameMenuShowHide::OnAnimTimer(UINT timerID)
{
 if(timerID!=ANIM_TIMER1||!menu||menu->Count()<5) return;
 ScreenDrawBitmap(&p1,menu->offset.x,                                   menu->offset.y+menu->Get(0)->offset.y-100+i);
 ScreenDrawBitmap(&p2,menu->offset.x+menu->Get(1)->offset.x-200+4+(i*2),menu->offset.y+menu->Get(1)->offset.y+0);
 ScreenDrawBitmap(&p3,menu->offset.x+menu->Get(2)->offset.x+200-(i*2),  menu->offset.y+menu->Get(2)->offset.y+0);
 ScreenDrawBitmap(&p4,menu->offset.x+menu->Get(3)->offset.x-200+4+(i*2),menu->offset.y+menu->Get(3)->offset.y+0);
 ScreenDrawBitmap(&p5,menu->offset.x,                                   menu->offset.y+menu->Get(4)->offset.y+100-i);
 i+=step;
 if(/*i+step==to||*/(i>to&&step>0)||(i<to&&step<0))
   {
    SetMainResponder(Parent);
	return;
   }
 ScreenGetBitmap(&p1,menu->offset.x,                                   menu->offset.y+menu->Get(0)->offset.y-100+i);
 ScreenGetBitmap(&p2,menu->offset.x+menu->Get(1)->offset.x-200+4+(i*2),menu->offset.y+menu->Get(1)->offset.y+0);
 ScreenGetBitmap(&p3,menu->offset.x+menu->Get(2)->offset.x+200-(i*2),  menu->offset.y+menu->Get(2)->offset.y+0);
 ScreenGetBitmap(&p4,menu->offset.x+menu->Get(3)->offset.x-200+4+(i*2),menu->offset.y+menu->Get(3)->offset.y+0);
 ScreenGetBitmap(&p5,menu->offset.x,                                   menu->offset.y+menu->Get(4)->offset.y+100-i);

 ScreenDrawBitmap(/*menu->Get(0)->selected?menu->Get(0)->btmpSelect:*/menu->Get(0)->btmpNoSelect,menu->offset.x,                                   menu->offset.y+menu->Get(0)->offset.y-100+i);
 ScreenDrawBitmap(/*menu->Get(1)->selected?menu->Get(1)->btmpSelect:*/menu->Get(1)->btmpNoSelect,menu->offset.x+menu->Get(1)->offset.x-200+4+(i*2),menu->offset.y+menu->Get(1)->offset.y+0);
 ScreenDrawBitmap(/*menu->Get(2)->selected?menu->Get(2)->btmpSelect:*/menu->Get(2)->btmpNoSelect,menu->offset.x+menu->Get(2)->offset.x+200-(i*2),  menu->offset.y+menu->Get(2)->offset.y+0);
 ScreenDrawBitmap(/*menu->Get(3)->selected?menu->Get(3)->btmpSelect:*/menu->Get(3)->btmpNoSelect,menu->offset.x+menu->Get(3)->offset.x-200+4+(i*2),menu->offset.y+menu->Get(3)->offset.y+0);
 ScreenDrawBitmap(/*menu->Get(4)->selected?menu->Get(4)->btmpSelect:*/menu->Get(4)->btmpNoSelect,menu->offset.x,                                   menu->offset.y+menu->Get(4)->offset.y+100-i);
}

void MInGameMenuShowHide::OnActivate(MResponder *resp)
{
 if(resp==Parent||resp==menu)
   {
    if(!menu||menu->Count()<5)
	  {
	   SetMainResponder(Parent);
	   return;
	  }
    SetAnimTimerElapsed(ANIM_TIMER1,10);
    SetAnimTimerEnabled(ANIM_TIMER1,true);
	if(Show) { i=0; to=100; step=10;}
	 else {i=100; to=0; step=-10;}
	if(Show)
	  {
	   p1.Create(237,54);
	   p2.Create(237,54);
	   p3.Create(237,54);
	   p4.Create(237,54);
	   p5.Create(237,54);
	   ScreenGetBitmap(&p1,menu->offset.x,                                   menu->offset.y+menu->Get(0)->offset.y-100+i);
	   ScreenGetBitmap(&p2,menu->offset.x+menu->Get(1)->offset.x-200+4+(i*2),menu->offset.y+menu->Get(1)->offset.y+0);
	   ScreenGetBitmap(&p3,menu->offset.x+menu->Get(2)->offset.x+200-(i*2),  menu->offset.y+menu->Get(2)->offset.y+0);
	   ScreenGetBitmap(&p4,menu->offset.x+menu->Get(3)->offset.x-200+4+(i*2),menu->offset.y+menu->Get(3)->offset.y+0);
	   ScreenGetBitmap(&p5,menu->offset.x,                                   menu->offset.y+menu->Get(4)->offset.y+100-i);
	  }
   }
}

void MInGameMenuShowHide::OnDeactivate(MResponder *resp)
{
 if(resp==Parent)
   {
    SetAnimTimerEnabled(ANIM_TIMER1,false);
   }
}

//---------------------------------------------------------------------------------------

MInGameMenuShowHide InGameMenuShowHide;

//--------------------------------------------------------------------------
MInGameMenu::MInGameMenu(){}
MInGameMenu::~MInGameMenu(){}

void MInGameMenu::AllocCache(void)
{
 MMenu::AllocCache();
 MMenuPolozka *p;
 for(unsigned int i=0;i<polozky.size();i++)
   {
    p=polozky[i];
	if(p)
	  {
	   if(p->btmpNoSelect) p->btmpNoSelect->UpdateMask(TRANSPARENT_COLOR);
	   if(p->btmpSelect) p->btmpSelect->UpdateMask(TRANSPARENT_COLOR);
	  }
   } 
}

void MInGameMenu::OnActivate(MResponder *resp)
{
 if(resp==Parent)
   {
    InGameMenuShowHide.Show=true;
	InGameMenuShowHide.menu=this;
	InGameMenuShowHide.Parent=this;
	SetMainResponder(&InGameMenuShowHide);
   }
  else
   {
    SelectMenuItem(defchoice);
    Draw();
   }
}

void SetGameMenuRespons(MGame *Game);

void MInGameMenu::OnDeactivate(MResponder *resp)
{
 if(resp!=&InGameMenuShowHide)
   {
    InGameMenuShowHide.Show=false;
	InGameMenuShowHide.menu=this;
	InGameMenuShowHide.Parent=resp;
	SetMainResponder(&InGameMenuShowHide);
	SetGameMenuRespons(NULL);
   }
}

//--------------------------------------------------------------------------

class MOptionsMenu: public MMenuSOcima
{
 private:
 public:
   MOptionsMenu();
   virtual ~MOptionsMenu();

   virtual void OnActivate(MResponder *resp);
   virtual void OnDeactivate(MResponder *resp);
};

MOptionsMenu::MOptionsMenu(){}
MOptionsMenu::~MOptionsMenu(){}

void MOptionsMenu::OnActivate(MResponder *resp)
{
 MMenuSlider *s;
 MMenuPolozka *p;
 p=GetByID(MENU_OPTIONS_ZVUK2);
 if(p)
   {
    s=dynamic_cast<MMenuSlider *>(p);
	if(s)
	  {
	   s->slidePos=(int)(s->slideSize*(double(GetSfxVolume())/100.0));
	   if(s->slidePos<0) s->slidePos=0;
	    else if(s->slidePos>s->slideSize) s->slidePos=s->slideSize;
	  }
   }
 p=GetByID(MENU_OPTIONS_HUDBA2);
 if(p)
   {
    s=dynamic_cast<MMenuSlider *>(p);
	if(s)
	  {
	   s->slidePos=(int)(s->slideSize*(double(GetMusicVolume())/100.0));
	   if(s->slidePos<0) s->slidePos=0;
	    else if(s->slidePos>s->slideSize) s->slidePos=s->slideSize;
	  }
   }
 MMenuSOcima::OnActivate(resp);
}

void MOptionsMenu::OnDeactivate(MResponder *resp)
{
 MMenuSOcima::OnDeactivate(resp);
 MMenuSlider *s;
 MMenuPolozka *p;
 p=GetByID(MENU_OPTIONS_ZVUK2);
 if(p)
   {
    s=dynamic_cast<MMenuSlider *>(p);
	if(s)
	  {
	   if(s->slideSize)
	     {
		  SetSfxVolume((int)(s->slidePos*(100.0/double(s->slideSize))));
		 }
	  }
   }
 p=GetByID(MENU_OPTIONS_HUDBA2);
 if(p)
   {
    s=dynamic_cast<MMenuSlider *>(p);
	if(s)
	  {
	   if(s->slideSize)
	     {
		  SetMusicVolume((int)(s->slidePos*(100.0/double(s->slideSize))));
		 }
	  }
   }
 SaveSetup();
}


//--------------------------------------------------------------------------
class MMenuCredits: public MTextScroller
{
 private:
 public:
   MMenuCredits();
   virtual ~MMenuCredits();

   virtual void OnActivate(MResponder *resp);
};

MMenuCredits::MMenuCredits(){}
MMenuCredits::~MMenuCredits(){}

void MMenuCredits::OnActivate(MResponder *resp)
{
 MTextScroller::OnActivate(resp);
 hraj_hudbu_credits();
}

//--------------------------------------------------------------------------

class MEndScreen:public MResponder
{
 private:
 public:
	MEndScreen(){}
	virtual ~MEndScreen(){}

	void KeyPress(UINT Key,UINT butts);
	void MouseMove(int x, int y, UINT butts){}
	void MouseLeft(int x, int y, UINT butts){if(butts&MOUSE_BUTT_LEFT) DoEnd();}
	void MouseMiddle(int x, int y, UINT butts){if(butts&MOUSE_BUTT_MIDDLE)DoEnd();}
	void MouseRight(int x, int y, UINT butts){if(butts&MOUSE_BUTT_RIGHT)DoEnd();}

	void OnAnimTimer(UINT timerID){}
	void OnActivate(MResponder *resp);
	void OnDeactivate(MResponder *resp){}

	void DoEnd(void);
};

void MEndScreen::KeyPress(UINT Key,UINT butts)
{
 switch(Key)
   {
    case KEY_ESCAPE:
	case KEY_RETURN:
	case KEY_SELECT:
	case KEY_SPACE:
	   DoEnd();
	   break;
   }
}
void MEndScreen::OnActivate(MResponder *resp)
{
 ScreenClearRect();
 ScreenDrawBitmap("ENDSCR",0,0);
}

void MEndScreen::DoEnd(void)
{
 QuitGame();
}

//--------------------------------------------------------------------------

MMainMenu MainMenu;
MMenuSOcima NewGameMenu,HelpMenu,QuitMenu;
MOptionsMenu OptionsMenu;
MMenuEnterPass PasswordMenu;
MMenuHelpWizard HelpItem11,HelpItem12;
MMenuCredits Credits;
MEndScreen EndScreen;

MEpisodes E1(1),E2(2),E3(3),E4(4),E5(5);
MInGameMenu InGameMenu;

MResponder *GetEndScreen(void)
{
 return &EndScreen;
}

void DrawMainMenuBackground(void)
{
 ScreenDrawBitmap(MainMenu.background,MainMenu.offset.x, MainMenu.offset.y);
}

MMenu *GetNewGameMenu(void)
{
 return &NewGameMenu;
}

MInGameMenu *GetInGameMenu(void)
{
 return &InGameMenu;
}

void ShowHelpMenu(MResponder *parent)
{
 HelpMenu.Parent=parent;
 ScreenClearRect();
 MMenuPolozka *pol=HelpMenu.GetByID(MENU_HELP_ZPET);
 if(pol)
   {
    pol->NextMenu=parent;
   }
 SetMainResponder(&HelpMenu);
}

void ShowOptionsMenu(MResponder *parent)
{
 OptionsMenu.Parent=parent;
 MMenuPolozka *pol=OptionsMenu.GetByID(MENU_OPTIONS_ZPET);
 if(pol)
   {
    pol->NextMenu=parent;
   }
 ScreenClearRect();
 SetMainResponder(&OptionsMenu);
}

void SFXMove(MMenuSlider *pSlider)
{
 if(!pSlider) return;
 SetSfxVolume((int)(pSlider->slidePos*(100.0/double(pSlider->slideSize))));
}

void MusicMove(MMenuSlider *pSlider)
{
 if(!pSlider) return;
 SetMusicVolume((int)(pSlider->slidePos*(100.0/double(pSlider->slideSize))));
}

MMenu *GetMainMenu(void)
{
 if(!MainMenu.Count())
   {
	MMenuPolozka *p;

    MainMenu.SetOffset(113,HMF);
    //LoadBitmap(MainMenu.background,"MENU_001");
	MainMenu.SetBackground("MENU_001");
	MainMenu.CanFreeBackground=false;
    MainMenu.ParentMenu=&QuitMenu;
    
	MainMenu.Add(new MMenuPolozka(MENU_MAIN_NEWGAME, "MENU_003","MENU_030", 257-113,119, 134,31,&NewGameMenu));
	MainMenu.Add(new MMenuPolozka(MENU_MAIN_PASSWORD,"MENU_004","MENU_031", 257-113,159, 134,31,&PasswordMenu));
	MainMenu.Add(new MMenuPolozka(MENU_MAIN_HELP,    "MENU_005","MENU_032", 257-113,199, 134,31,&HelpMenu));
	MainMenu.Add(new MMenuPolozka(MENU_MAIN_OPTIONS, "MENU_006","MENU_033", 257-113,239, 134,31,&OptionsMenu));
	MainMenu.Add(new MMenuPolozka(MENU_MAIN_EDITOR,  "MENU_007","MENU_034", 257-113,279, 134,31,GetLevelEditor()));
	MainMenu.Add(new MMenuPolozka(MENU_MAIN_QUIT,    "MENU_008","MENU_035", 257-113,319, 134,31,&QuitMenu));
	MainMenu.Add(new MMenuPolozka(MENU_MAIN_OPEN,    "MENU_092","MENU_091", 363,    363,  21,22));
	MainMenu.SelectMenuItem(0);

	QuitMenu.ParentMenu=&MainMenu;
    //LoadBitmap(QuitMenu.background,
	QuitMenu.SetBackground("MENU_022");
	QuitMenu.SetOffset(113+92,HMF+99);
	QuitMenu.Add(new MMenuPolozka(MENU_QUIT_YES,"MENU_023","MENU_041",268-113-92,221-99,53,25,GetEndScreen()));
	QuitMenu.defchoice=QuitMenu.Add(new MMenuPolozka(MENU_QUIT_NO ,"MENU_024","MENU_042",348-113-92,221-99,36,25,&MainMenu));

	E1.Parent=&MainMenu;
	E2.Parent=&MainMenu;
	E3.Parent=&MainMenu;
	E4.Parent=&MainMenu;
	E5.Parent=&MainMenu;

	NewGameMenu.ParentMenu=&MainMenu;
	//LoadBitmap(NewGameMenu.background,
	NewGameMenu.SetBackground("MENU_009");
	NewGameMenu.SetOffset(113+135,HMF+99);
	NewGameMenu.defchoice=NewGameMenu.Add(new MMenuPolozka(MENU_NEWGAME_TRENINK ,"MENU_010","MENU_036", 275-113-135,154-99, 102,22,&E1));
	NewGameMenu.Add(new MMenuPolozka(MENU_NEWGAME_LEHKA   ,"MENU_011","MENU_025", 275-113-135,185-99, 102,22,&E2));
	NewGameMenu.Add(new MMenuPolozka(MENU_NEWGAME_STREDNI ,"MENU_012","MENU_026", 275-113-135,214-99, 102,22,&E3));
	NewGameMenu.Add(new MMenuPolozka(MENU_NEWGAME_TEZKA   ,"MENU_013","MENU_027", 275-113-135,244-99, 102,22,&E4));
	NewGameMenu.Add(new MMenuPolozka(MENU_NEWGAME_NEMOZNA ,"MENU_014","MENU_028", 275-113-135,276-99, 102,22,&E5));
	NewGameMenu.Add(new MMenuPolozka(MENU_NEWGAME_ZPET    ,"MENU_015","MENU_029", 275-113-135,320-99, 102,31,&MainMenu));
	
	PasswordMenu.ParentMenu=&MainMenu;
	//LoadBitmap(PasswordMenu.background,
	PasswordMenu.SetBackground("MENU_016");
	PasswordMenu.SetOffset(113+127,HMF+99);
	//PasswordMenu.Add(new MMenuPolozka(MENU_PASSWORD_ZPET ,"MENU_015","MENU_029", 275-113-127,320-99, 102,31,&MainMenu));
	
	//PasswordMenu.Initialize(30,123,112,42);
    PasswordMenu.Add(new MMenuEnterText(MENU_PASSWORD_PASS,30,123,112,42,MAX_PASS_LEN,NULL));


	HelpMenu.ParentMenu=NULL;//&MainMenu;
	//LoadBitmap(HelpMenu.background,
	HelpMenu.SetBackground("MENU_017");
	HelpMenu.SetOffset(113+135,HMF+99);
	HelpMenu.Add(new MMenuPolozka(MENU_HELP_OVLADANI   ,"MENU_018","MENU_037", 272-113-135,167-99, 101,22,&HelpItem11));
	HelpMenu.Add(new MMenuPolozka(MENU_HELP_PRAVIDLA   ,"MENU_019","MENU_038", 272-113-135,201-99, 101,22,&HelpItem12));
	HelpMenu.Add(new MMenuPolozka(MENU_HELP_AUTORI     ,"MENU_020","MENU_039", 272-113-135,236-99, 101,22,&Credits));
	HelpMenu.Add(new MMenuPolozka(MENU_HELP_ZPET       ,"MENU_021","MENU_040", 272-113-135,320-99, 101,31,&MainMenu));

	HelpItem11.SetOffset(0,0);
	HelpItem11.ParentMenu=&HelpMenu;
	HelpItem11.wizID=WIZID_OVLADANI;

	HelpItem12.SetOffset(0,0);
	HelpItem12.ParentMenu=&HelpMenu;
	HelpItem12.wizID=WIZID_PRAVIDLA;

	Credits.Parent=&HelpMenu;
	mySIZE sz;
	ScreenGetSize(sz);
	Credits.Initialize("MENU_054",(sz.cx-618)/2,0, 0,150,sz.cx,sz.cy-150, "CREDIT",0,0,false,true);

	OptionsMenu.ParentMenu=NULL;//&MainMenu;
	//LoadBitmap(OptionsMenu.background,
	OptionsMenu.SetBackground("MENU_059");
	OptionsMenu.SetOffset(113+75,HMF+99);
	p=new MMenuPolozka(MENU_OPTIONS_ZVUK   ,"MENU_060","MENU_063", 94,50, 79,32);
	OptionsMenu.defchoice=OptionsMenu.Add(p);
	MMenuSlider *s=new MMenuSlider(MENU_OPTIONS_ZVUK2  ,"MENU_058","MENU_058", 25,85, 11,27, 200,true);
	s->Owner=p; p->Slider=s;
    s->OnMove=SFXMove;

	OptionsMenu.Add(s);
	//25,85 -- 225,100, 
	//OptionsMenu.Add(new MMenuPolozka();
	p=new MMenuPolozka(MENU_OPTIONS_HUDBA  ,"MENU_061","MENU_064", 94,131, 80,28);
	OptionsMenu.Add(p);
	//30,160 -- 225,160, 
	s=new MMenuSlider(MENU_OPTIONS_HUDBA2  ,"MENU_058","MENU_058", 25,160, 11,27, 200,true);
	s->Owner=p; p->Slider=s;
	s->OnMove=MusicMove;

	OptionsMenu.Add(s);
	OptionsMenu.Add(new MMenuPolozka(MENU_OPTIONS_ZPET   ,"MENU_062","MENU_065", 100,229, 67,34,&MainMenu));

	
	InGameMenu.needMainBackground=false;
	InGameMenu.SetOffset((320-(237/2)),(240-(249/2)));
	InGameMenu.defchoice=InGameMenu.Add(new MMenuPolozka(MENU_GAME_RESTART,"MENU_044","MENU_049", 0,0  ,237,52,NULL)); //restart
	InGameMenu.Add(new MMenuPolozka(MENU_GAME_LOAD,"MENU_045","MENU_050", 0,51 ,237,54,NULL)); //load
	InGameMenu.Add(new MMenuPolozka(MENU_GAME_SAVE,"MENU_046","MENU_051", 0,106,237,52,NULL)); //save
	InGameMenu.Add(new MMenuPolozka(MENU_GAME_KONEC,"MENU_047","MENU_052", 0,156,237,52,GetMainMenu())); //konec
	InGameMenu.Add(new MMenuPolozka((UINT)-1,"MENU_048","MENU_053", 0,208,237,52,NULL)); //zpet
	InGameMenu.AllocCache();


	MainMenu.AllocCache();
   }
 return &MainMenu;
}

//--------------------------------------------------------------------------

void SetLevelAndEpisodeByPass(const char *mpass)
{
 if(!mpass||strlen(mpass)!=5) return;
 const char *fls[]={"S_TREN","S_LEHKY","S_STRED","S_TEZKY","S_NEMOZ",NULL};
 char level[13];
 char *txt;
 int i,lpos;
 bool done=false;
 for(i=0;!done&&fls[i]!=NULL&&i<5;i++)
   {
    txt=LoadText(fls[i],0,0,false,false);
	if(txt)
	  {
       UINT pos=0,cnt=0;
	   bool cancnt=true;
	   bool nextcomm=true;
       while(txt[pos]!=0)
	     {
	      if(nextcomm)
	        {
		     nextcomm=false;
		     cancnt=txt[pos]!=';';
			 level[0]=0;
			 lpos=0;
		    }
	      if(txt[pos]==0x0a&&cancnt)
	        {
		     cnt++;
		     if(stricmp(level,mpass)==0)
		       {
			    MEpisodes *ep=NULL;
			    switch(i+1)
				  {
				   case 1: ep=&E1; break;
				   case 2: ep=&E2; break;
				   case 3: ep=&E3; break;
				   case 4: ep=&E4; break;
				   case 5: ep=&E5; break;
				  }
				if(ep)
				  {
				   ep->level=cnt/2;
				   SetMainResponder(ep);
				  }
				done=true;
			    break;
			   }
		    }
		  if(txt[pos]!=0x0d&&txt[pos]!=0x0a&&cancnt&&lpos<12)
		    {
			 level[lpos]=txt[pos]; lpos++; level[lpos]=0;
			}
	      if(txt[pos]==0x0a) 
	        {
		     nextcomm=true;
		    }
	      pos++;
	     }
	   free(txt);
	  }
   }
}

void SetLevelInEpisode(int episode, int level)
{
 MEpisodes *ep=NULL;
 switch(episode)
   {
    case 1: ep=&E1; break;
    case 2: ep=&E2; break;
    case 3: ep=&E3; break;
    case 4: ep=&E4; break;
    case 5: ep=&E5; break;
   }
 if(ep)
   {
    ep->level=level;
	//SetMainResponder(ep);
   }
}
