#ifndef MENU_HEADER
#define MENU_HEADER

#include "berdy.h"
#include <vector>

using namespace std;

class MMenu;
class MMenuSlider;

typedef void (*MOnSliderMove)(MMenuSlider *pSlider); //evena co se vola, kdyz se zmeni pozice slideru

class MMenuPolozka
{
 private:
   char *selImgNm;
   char *noSelImgNm;
 public:
   MMenuPolozka(UINT pID);
   MMenuPolozka(UINT pID, const char *selImg, const char *noSelImg, int x, int y, int cx, int cy, MResponder *nextMenu=NULL);
   virtual ~MMenuPolozka();

   void SetSelImg(const char *selImg);
   const char *GetSelImg(void);

   void SetNoSelImg(const char *noSelImg);
   const char *GetNoSelImg(void);

   virtual void AllocCache(void);
   virtual void FreeCache(void);

   MBitmap *btmpNoSelect;
   MBitmap *btmpSelect;

   UINT ID; //identifikator menu
   bool selected; //rika, jestli ma byt zvyrazneny
   myPOINT offset; //kde je v menu
   mySIZE size; //velikost polozky menu
   bool Enabled; //default jsou vsechny polozky pristupne

   MResponder *NextMenu; //po stisknuti, se prejde sem, kdyz neni NULL, vyhodnocuje se pred ParentMenu
   MMenuSlider *Slider; //prirazeny slider k teto polzoce; NULL kdyz neni zadny

   virtual bool MouseOver(int x, int y);
   virtual void Draw(const myPOINT &menuOffset);
   virtual bool KeyPress(UINT Key, UINT butts); //vraci, jestli pouzil klavesu
   void SetOffset(int x, int y);
   void SetSize(int cx, int cy);
};

class MMenuSlider: public MMenuPolozka
{
 private:
 public:
   MMenuSlider(UINT pID);
   MMenuSlider(UINT pID, const char *selImg, const char *noSelImg, int x, int y, int cx, int cy, int sz, bool phorz,MResponder *nextMenu=NULL);
   virtual ~MMenuSlider();

   virtual void AllocCache(void);
   virtual void FreeCache(void);

   void InitSliderBack(MBitmap *backbmp);

   MBitmap *btmpSliderBack; //pozadi slideru, x-cx,y,x+cx+sz,y+cy
   bool MouseOver(int x, int y);
   void Draw(const myPOINT &menuOffset);

   bool horzSlider; //jestli je horizontalni nebo vertikalni
   int slidePos; //kde je posunovatko, cislo mezi 0..slideSize
   int slideSize;
   MMenuPolozka *Owner; //zvyrazni tohoto, pokud je MouseOver

   void CallOnMove(void);
   MOnSliderMove OnMove;
};

#define MAX_ENTER_TEXT 50

class MMenuEnterText:public MMenuPolozka
{
 private:
   char buff[MAX_ENTER_TEXT];
   myRECT where; //kde je misto na zadavani textu
   MBitmap *txtbackground;
   int maxlen;

   myPOINT curpos;
   bool showed; //jestli je cursor videt
   int znaku;
 public:
   MMenuEnterText(UINT pID);
   MMenuEnterText(UINT pID, int x, int y, int cx, int cy, int pmaxlen, const char *txt);
   virtual ~MMenuEnterText();

   void Initialize(int x, int y, int cx, int cy, int pmaxlen, const char *txt);
   void DrawTxt(void);

   void SetText(const char *txt);
   const char *GetText(void);

   bool CanShowCursor;

   bool KeyPress(UINT Key,UINT butts);
   void OnAnimTimer(UINT timerID);
   void OnActivate(const myPOINT &menuOffset);
   void OnDeactivate(void);
};

class MMenu: public MResponder
{
 private:
   int lastchoice; //co bylo naposledy vybrano
   myRECT rect;
   char *backImgNm;
 protected:
   vector<MMenuPolozka *> polozky;
   void SelectByMouse(int x, int y); //vybere menu podle toho, nad kterym je mys, x,y jsou jeste neupravene souradnice
   MMenuPolozka *GetSelected(void);
   MMenuPolozka *GetOverMouse(int x,int y); //vraci prvek, ktery je opravdu nad mysi
 public:
   MMenu();
   virtual ~MMenu();

   virtual void AllocCache(void);
   virtual void FreeCache(void);
   virtual void Cancel(void);

   bool CanFreeBackground; //defautl true, ale MainMenu nemuze
   void SetBackground(const char *backImg);
   const char *GetBackground(void);

   MMenu *ParentMenu;
   MBitmap *background;
   myPOINT offset;
   int defchoice;
   bool needMainBackground;

   MMenuSlider *actSlider; //aktualni slider


   void Clear(void);
   void ClearPolozky(void);
   UINT Add(MMenuPolozka *polozka); //prida na konec tuto polozku; prebira pointer; vraci cislo polozky
   void Del(UINT pos); //smaze polozku na danem miste
   MMenuPolozka *Get(UINT pos);
   MMenuPolozka *GetByID(UINT polID); //vrati polozku ze seznamu pdole ID
   UINT Count(void); //vraci pocet polozek
   virtual void Draw(void); //vykresli menu na obrazovku
   void DrawSelected(void); //zvyrazni v menu vybrany prvek
   void SetOffset(int x, int y);

   void SelectMenuItem(UINT pos, bool dir_forward=true); //dir_forward rika, kterym smerem pripadne menit polozky
   virtual void ActivateMenuItem(UINT itemID=UINT(-1));
   virtual void MenuItemChanged(void); //zmenila se vybrana polozka menu

   //MResponder implementace
   void KeyPress(UINT Key,UINT butts);
   void MouseMove(int x, int y, UINT butts);
   void MouseLeft(int x, int y, UINT butts);
   void MouseMiddle(int x, int y, UINT butts);
   void MouseRight(int x, int y, UINT butts);

   void OnAnimTimer(UINT timerID);
   void OnActivate(MResponder *resp);
   void OnDeactivate(MResponder *resp);
};


class MInGameMenu:public MMenu
{
 private:
 public:
   MInGameMenu();
   virtual ~MInGameMenu();

   void AllocCache(void);
   void OnActivate(MResponder *resp);
   void OnDeactivate(MResponder *resp);

};

#define MENU_MAIN_NEWGAME  1
#define MENU_MAIN_PASSWORD MENU_MAIN_NEWGAME+1
#define MENU_MAIN_HELP     MENU_MAIN_PASSWORD+1
#define MENU_MAIN_OPTIONS  MENU_MAIN_HELP+1
#define MENU_MAIN_EDITOR   MENU_MAIN_OPTIONS+1
#define MENU_MAIN_QUIT     MENU_MAIN_EDITOR+1

#define MENU_NEWGAME_TRENINK MENU_MAIN_QUIT+1
#define MENU_NEWGAME_LEHKA   MENU_NEWGAME_TRENINK+1
#define MENU_NEWGAME_STREDNI MENU_NEWGAME_LEHKA+1
#define MENU_NEWGAME_TEZKA   MENU_NEWGAME_STREDNI+1
#define MENU_NEWGAME_NEMOZNA MENU_NEWGAME_TEZKA+1
#define MENU_NEWGAME_ZPET    MENU_NEWGAME_NEMOZNA+1

#define MENU_HELP_OVLADANI   MENU_NEWGAME_ZPET+1
#define MENU_HELP_PRAVIDLA   MENU_HELP_OVLADANI+1
#define MENU_HELP_AUTORI     MENU_HELP_PRAVIDLA+1
#define MENU_HELP_ZPET       MENU_HELP_AUTORI+1

#define MENU_QUIT_YES MENU_HELP_ZPET+1
#define MENU_QUIT_NO  MENU_QUIT_YES+1

#define MENU_OPTIONS_ZVUK   MENU_QUIT_NO+1
#define MENU_OPTIONS_ZVUK2  MENU_OPTIONS_ZVUK+1
#define MENU_OPTIONS_HUDBA  MENU_OPTIONS_ZVUK2+1
#define MENU_OPTIONS_HUDBA2 MENU_OPTIONS_HUDBA+1
#define MENU_OPTIONS_ZPET   MENU_OPTIONS_HUDBA2+1

#define MENU_WIZ_PREV MENU_OPTIONS_ZPET+1
#define MENU_WIZ_ZPET MENU_WIZ_PREV+1
#define MENU_WIZ_NEXT MENU_WIZ_ZPET+1

#define MENU_GAME_RESTART MENU_WIZ_NEXT+1
#define MENU_GAME_LOAD    MENU_GAME_RESTART+1
#define MENU_GAME_SAVE    MENU_GAME_LOAD+1
#define MENU_GAME_KONEC   MENU_GAME_SAVE+1

#define MENU_MAIN_OPEN     MENU_GAME_KONEC+1

#define MENU_FC_SCROLLUP   MENU_MAIN_OPEN+1
#define MENU_FC_SCROLLDOWN MENU_FC_SCROLLUP+1
#define MENU_FC_OK         MENU_FC_SCROLLDOWN+1
#define MENU_FC_ZRUSIT     MENU_FC_OK+1
#define MENU_FC_FILELIST   MENU_FC_ZRUSIT+1
#define MENU_FC_FILENAME   MENU_FC_FILELIST+1


#define MENU_PASSWORD_PASS   MENU_FC_FILENAME+1


MMenu *GetMainMenu(void);
MMenu *GetNewGameMenu(void);
MInGameMenu *GetInGameMenu(void);

void ActivateFileChoose(MResponder *pParent,bool pLoadDlg);
MResponder *GetFileChoose(void);
const char *GetFileChooseResult(void);

typedef void (* MOnMenuActivate)(UINT menuID); //callback na volani menu items
void SetOnMenuActivate(MOnMenuActivate func);
MOnMenuActivate GetOnMenuItemActivate(void);


#endif
