#include "sound.h"
#include "berdy.h"
#include <SDL.h>
#include <SDL_mixer.h>

const char *GetExeDir(void);
//void WriteToLog(const char *format,...);

//-----------------------------------------------------------------------------
bool soundInitialized=false;

#define MixChannels 3
Mix_Chunk *chunks[MixChannels];
Mix_Music *music=NULL;

Mix_Chunk *chunkCache[SFX_COUNT];

void InitSoundAndMusic(void)
{
 soundInitialized=(SDL_WasInit(SDL_INIT_AUDIO)&SDL_INIT_AUDIO)!=0;
 if(soundInitialized)
   {
    soundInitialized=(Mix_OpenAudio(22050,MIX_DEFAULT_FORMAT,2,1024)==0);
   }
 if(soundInitialized)
   {
    Mix_AllocateChannels(MixChannels);
   }
 int i;
 for(i=0;i<MixChannels;i++)
   {
    chunks[i]=NULL;
   }
 for(i=0;i<SFX_COUNT;i++)
   {
    chunkCache[i]=NULL;
   }
}

void AssignChunk(int pos,Mix_Chunk *newone)
{
 if(pos<0||pos>MixChannels) return;
 if(chunks[pos]!=NULL)
   {
    if(soundInitialized)
	  {
	   Mix_HaltChannel(pos);
       //Mix_FreeChunk(chunks[pos]);
	  }
	chunks[pos]=NULL;
   }
 chunks[pos]=newone;
}

void AssignMusic(Mix_Music *newone)
{
 if(music!=NULL&&soundInitialized)
   {
    Mix_HaltMusic();
	Mix_FreeMusic(music);
    music=NULL;
   }
 music=newone;
}


void FreeSoundAndMusic(void)
{
 int i;
 for(i=0;i<MixChannels;i++)
   {
	AssignChunk(i,NULL);
   }
 for(i=0;i<SFX_COUNT;i++)
   {
    if(chunkCache[i]!=NULL)
	  {
	   Mix_FreeChunk(chunkCache[i]);
	   chunkCache[i]=NULL;
	  }
   }
 AssignMusic(NULL);
 if(soundInitialized)
   {
    Mix_CloseAudio();
   }
}

void StopPlayingChunk(Mix_Chunk *ch)
{
 if(!ch) return;
 for(int i=0;i<MixChannels;i++)
   {
    if(chunks[i]==ch)
	  {
	   AssignChunk(i,NULL);
	  }
   }
}

void PlayWav(const char *file,int devID,unsigned char sfx_id)
{
 if(/*sfx_id<0||*/sfx_id>=SFX_COUNT) return;
 if(soundInitialized)
   {
    Mix_Chunk *ch=chunkCache[sfx_id];
	if(ch==NULL) 
	  {
	   ch=Mix_LoadWAV(file);
	   chunkCache[sfx_id]=ch;
	  }
	 else
	  {
	   StopPlayingChunk(ch);
	  }
	if(ch)
	  {
       AssignChunk(devID,ch);
	   Mix_PlayChannel(devID,ch,0);
	  }
   }
}

const char *GetPathDelimiter(void);

void PlaySfx(unsigned char sfx_id, unsigned char sfx_prior)
{
 if(/*sfx_prior<0||*/sfx_prior>=3) return;

 char file[1024];
 sprintf(file,"%sSFX%ssmp_%03d.wav",GetExeDir(),GetPathDelimiter(),sfx_id);
 PlayWav(file,sfx_prior,sfx_id);
}

void StopSfx(void)
{
 if(soundInitialized)
   {
    Mix_HaltChannel(-1);
   }
}

int musicVolume=-1, sfxVolume=-1;

int GetSfxVolume(void)
{
 int volume=sfxVolume;
 if(sfxVolume==-1&&soundInitialized)
   {
	sfxVolume=Mix_Volume(-1,-1);
	if(sfxVolume<1){
	   sfxVolume=MIX_MAX_VOLUME/2;
	}
   }
 int res=volume;
 return res*100/MIX_MAX_VOLUME;
}

void SetSfxVolume(int val)
{
 if(val<0) val=0;
  else if(val>100) val=100;
 int volume=int((((((double)val)/100.0)*MIX_MAX_VOLUME)));
 sfxVolume=volume; //(volume<<16) + volume;
 //waveOutSetVolume((HWAVEOUT)waveID,sfxVolume);
 if(soundInitialized) Mix_Volume(-1,sfxVolume);
}

//-----------------------------------------------------------------------------
int GetMusicVolume(void)
{
 if(soundInitialized&&musicVolume==-1)
   {
    musicVolume=Mix_VolumeMusic(-1);
   }
 int res=musicVolume;
 return res*100/MIX_MAX_VOLUME;
}

void SetMusicVolume(int val)
{
 if(val<0) val=0;
  else if(val>100) val=100;
 int volume=int((((((double)val)/100.0)*MIX_MAX_VOLUME)));
 musicVolume=volume;
 if(soundInitialized) Mix_VolumeMusic(musicVolume);
}

void hraj_modul(int mod, int loop)
{
 if(!soundInitialized) return;
 char file[1024];
 stop_modul();
 sprintf(file,"%sMusic%sx_%03d.xm",GetExeDir(),GetPathDelimiter(),mod);
 Mix_Music *m=Mix_LoadMUS(file);
 AssignMusic(m);
 if(m)
   {
    if(loop) loop=-1; else loop=0;
    Mix_PlayMusic(m,loop);
    Mix_VolumeMusic(musicVolume);
   }
}

void stop_modul(void)
{
 AssignMusic(NULL);
}
